/*******************************************************************************
 * Licensed Materials - Property of IBM
 *  Copyright IBM Corporation 2006, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.ant.listener;

import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.Project;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.ant.task.CompleteBuildTask;
import com.ibm.team.build.ant.task.StartBuildTask;
import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.ant.AntMessages;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;

/**
 * Ant build listener for team builds. It listens for all
 * <code>StartBuildTask</code> ant tasks to finish and records the build
 * result started and the repository it was started in. It then guarantees in
 * the build finish event that all builds that have been started are completed
 * if they have not already been completed.
 * <p>
 * Verbose logging for the listener can be enabled by defining the property
 * "team.build.listener.verbose" in the ant project.
 * 
 * @since 0.5
 */
public class TeamBuildListener implements BuildListener {

    static {
        if (!TeamPlatform.isStarted()) {
            TeamPlatform.startup();
        }
    }

    /**
     * The name of the project property to use to determine if verbose logging
     * should be used.
     */
    public static final String VERBOSE_PROPERTY = "team.build.listener.verbose"; //$NON-NLS-1$

    /**
     * Minimal properties required when retrieving the build result for
     * processing.
     */
    private Collection MIN_PROPERTIES = Arrays.asList(new String[] { IBuildResult.PROPERTY_BUILD_STATE,
            IBuildResult.PROPERTY_BUILD_DEFINITION, IBuildResult.PROPERTY_BUILD_STATUS });

    private static Hashtable<String, ITeamRepository> fgStartedBuilds = new Hashtable<String, ITeamRepository>();

    /**
     * Total time in milliseconds spent in <code>AbstractTeamBuildTask</code>
     * and its subclasses.
     */
    private static long fgTotalBuildToolkitTime = 0;

    /**
     * The start time of each task.
     */
    private Hashtable<String, Long> fTaskStartTimes = new Hashtable<String, Long>();

    /**
     * The duration of each task in the order it was executed.
     */
    private List<TaskDuration> fTaskDurations = new LinkedList<TaskDuration>();

    /**
     * The total duration of each task. Durations for tasks that are executed
     * more than once are combined into one entry in the map.
     */
    private Map<String, TaskDuration> fCombinedTaskDurations = new HashMap<String, TaskDuration>();

    /**
     * Used to record all builds that are started with the
     * {@link StartBuildTask} ant task.
     * 
     * @param buildResultHandle
     *            The build result that was started.
     * @param teamRepository
     *            The instance of the repository that build was started on.
     */
    public static void startBuild(IBuildResultHandle buildResultHandle, ITeamRepository teamRepository) {
        fgStartedBuilds.put(buildResultHandle.getItemId().getUuidValue(), teamRepository);
    }

    /**
     * Used to record all builds that are started with the
     * {@link CompleteBuildTask} ant task.
     * 
     * @param buildResultHandle
     *            The build result that was completed.
     */
    public static void completeBuild(IBuildResultHandle buildResultHandle) {
        fgStartedBuilds.remove(buildResultHandle.getItemId().getUuidValue());
    }

    /**
     * Used to record the time taken by all children of
     * <code>AbstractTeamBuildTask</code>.
     * 
     * @param toolkitTimeTaken
     *            The time in milliseconds to add to the total.
     */
    public static void incrementBuildToolkitTime(long toolkitTimeTaken) {
        fgTotalBuildToolkitTime += toolkitTimeTaken;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void buildFinished(BuildEvent event) {
        if (verboseLogging(event.getProject())) {
            printTaskDurations(event);
        }

        Enumeration enumeratedBuilds = fgStartedBuilds.keys();
        while (enumeratedBuilds.hasMoreElements()) {
            String buildResultUUID = (String) enumeratedBuilds.nextElement();
            try {
                ITeamRepository teamRepository = fgStartedBuilds.get(buildResultUUID);
                if (!teamRepository.loggedIn()) {
                    teamRepository.login(getProgressMonitor());
                }
                IItemHandle itemHandle = IBuildResult.ITEM_TYPE.createItemHandle(UUID.valueOf(buildResultUUID), null);
                IBuildResult buildResult = (IBuildResult) teamRepository.itemManager().fetchPartialItem(itemHandle,
                        IItemManager.REFRESH, MIN_PROPERTIES, getProgressMonitor());
                if (buildResult.getState() != BuildState.COMPLETED) {
                	BuildStatus buildStatus = BuildStatus.OK;
                    if (event.getException() != null) {
                        buildStatus = BuildStatus.ERROR;
                    }
                    ClientFactory.getTeamBuildRequestClient(teamRepository).makeBuildCompleteWithStatus(buildResult, buildStatus,
                            IBuildResult.PROPERTIES_REQUIRED, getProgressMonitor());
                    completeBuild(buildResult);
                    if (verboseLogging(event.getProject())) {
                        IBuildDefinition definition = (IBuildDefinition) teamRepository.itemManager().fetchCompleteItem(
                                buildResult.getBuildDefinition(), IItemManager.DEFAULT, getProgressMonitor());
                        event.getProject().log(
                                NLS.bind(AntMessages.TeamBuildListener_AUTO_COMPLETE_BUILD, definition.getId(),
                                        buildResultUUID), Project.MSG_INFO);
                    }
                }
                teamRepository.logout();
            } catch (TeamRepositoryException e) {
                /*
                 * If the build result failed to complete...just log and move to
                 * the next one
                 */
                fgStartedBuilds.remove(buildResultUUID);
                event.getProject().log(
                        NLS.bind(AntMessages.TeamBuildListener_FAILED_TO_AUTO_COMPLETE_BUILD, buildResultUUID),
                        Project.MSG_WARN);
                event.getProject().log(e.getLocalizedMessage(), Project.MSG_WARN);
            }

        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void buildStarted(BuildEvent event) {
        fgTotalBuildToolkitTime = 0;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void messageLogged(BuildEvent event) {
        // Nothing to do
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void targetFinished(BuildEvent event) {
        // Nothing to do
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void targetStarted(BuildEvent event) {
        // Nothing to do
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void taskFinished(BuildEvent event) {
        long taskFinishedTime = System.currentTimeMillis();
        String taskName = event.getTask().getTaskName();
        Long taskStartTime = fTaskStartTimes.get(taskName);

        if (taskStartTime != null) {
            Long duration = taskFinishedTime - taskStartTime;

            fTaskDurations.add(new TaskDuration(taskName, duration));

            TaskDuration taskDuration = fCombinedTaskDurations.get(taskName);
            if (taskDuration != null) {
                taskDuration = new TaskDuration(taskName, duration + taskDuration.getDuration());
            } else {
                taskDuration = new TaskDuration(taskName, duration);
            }
            fCombinedTaskDurations.put(taskName, taskDuration);
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void taskStarted(BuildEvent event) {
        String taskName = event.getTask().getTaskName();
        if (taskName.equalsIgnoreCase("property") || taskName.equalsIgnoreCase("taskdef") || taskName.equalsIgnoreCase("echo") || taskName.equalsIgnoreCase("fail")) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
            // Ignore uninteresting tasks.
            return;
        }

        fTaskStartTimes.put(event.getTask().getTaskName(), System.currentTimeMillis());
    }

    /**
     * Determines if verbose logging is turned on for the project.
     * 
     * @param project
     *            The project to check for verbose logging.
     * @return <code>true</code> if verbose logging for the project is turned
     *         on.
     */
    private boolean verboseLogging(Project project) {
        return project.getProperty(VERBOSE_PROPERTY) != null;
    }

    private void printTaskDurations(BuildEvent event) {

        event.getProject().log(System.getProperty("line.separator")); //$NON-NLS-1$
        event.getProject().log(System.getProperty("line.separator")); //$NON-NLS-1$

        event.getProject().log(
                NLS.bind(AntMessages.TeamBuildListener_TOTAL_TOOLKIT_TIME,
                        Float.valueOf(fgTotalBuildToolkitTime) / 1000));

        if (!fCombinedTaskDurations.isEmpty()) {
            event.getProject().log(System.getProperty("line.separator")); //$NON-NLS-1$
            event.getProject().log(AntMessages.TeamBuildListener_COMBINED_TASK_TIME);

            SortedSet<TaskDuration> sortedTaskDurations = getSortedTaskDurations();

            for (TaskDuration taskDuration : sortedTaskDurations) {
                Float duration = Float.valueOf(taskDuration.getDuration()) / 1000;
                event.getProject().log(
                        NLS.bind(AntMessages.TeamBuildListener_TASK_DURATION, taskDuration.getTaskName(), duration));
            }
        }

        if (!fTaskDurations.isEmpty()) {
            event.getProject().log(System.getProperty("line.separator")); //$NON-NLS-1$
            event.getProject().log(AntMessages.TeamBuildListener_SEPARATE_TASK_TIME);

            for (TaskDuration taskDuration : fTaskDurations) {
                Float duration = Float.valueOf(taskDuration.getDuration()) / 1000;
                event.getProject().log(
                        NLS.bind(AntMessages.TeamBuildListener_TASK_DURATION, taskDuration.getTaskName(), duration));
            }
        }

    }

    private SortedSet<TaskDuration> getSortedTaskDurations() {
        SortedSet<TaskDuration> sortedTaskDurations = new TreeSet<TaskDuration>(new Comparator<TaskDuration>() {
            public int compare(TaskDuration o1, TaskDuration o2) {
                // Sort in descending order.
                return Long.valueOf(o2.getDuration()).compareTo(o1.getDuration());
            }
        });

        sortedTaskDurations.addAll(fCombinedTaskDurations.values());
        return sortedTaskDurations;
    }
    
    /**
     * Retrieves a progress monitor that can be used in the task. The monitor is
     * the equivalent of a <code>NullProgressMonitor</code> that does nothing.
     * 
     * @return a progress monitor that can be used in the task.
     */
    private IProgressMonitor getProgressMonitor() {
        return new NullProgressMonitor();
    }
    

    private class TaskDuration {
        final String fTaskName;
        final long fDuration;

        /**
         * Create the task duration.
         * 
         * @param taskName
         *            The task name.
         * @param duration
         *            The duration of the task.
         */
        public TaskDuration(String taskName, long duration) {
            fTaskName = taskName;
            fDuration = duration;
        }

        protected String getTaskName() {
            return fTaskName;
        }

        protected long getDuration() {
            return fDuration;
        }
    }
}
