/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.ant.task;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.EnumeratedAttribute;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildAction;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildRequestHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.ant.AntMessages;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;

/**
 * Base ant task for publishing to an existing build result in the team
 * repository. Ant tasks that need to publish information to an existing build
 * result should subclass this class.
 * <p>
 * <p>
 * Required attributes are:
 * <p>
 * <ul>
 * <li><b>buildResultUUID</b> - The uuid of the build result to update.</li>
 * </ul>
 * <p>
 * Optional attributes are:
 * <ul>
 * <li><b>componentName</b> - Denotes a specific component that this
 * contribution should be associated with.
 * </ul>
 * <b>See {@link AbstractTeamBuildTask} for additional required and option
 * attributes.</b>
 * 
 * @since 0.5
 */
public abstract class AbstractPublisherTask extends AbstractTeamBuildTask {

    /**
     * The ant attribute for this task in the ant script representing the uuid
     * of the build result to be updated.
     */
    public static final String BUILD_RESULT_UUID = "buildResultUUID"; //$NON-NLS-1$

    /**
     * The ant attribute for this task in the ant script representing the
     * component that the published contribution should be associated with.
     */
    public static final String COMPONENT_NAME = "componentName"; //$NON-NLS-1$

    private String fBuildResultUUID = null;
    private String fComponentName = null;

    /**
     * Returns the name of the component that the published contribution will be
     * associated with.
     */
    protected String getComponentName() {
        return fComponentName;
    }

    /**
     * Returns the name of the component that the published contribution will be
     * associated with.
     * 
     * @param componentName
     *            The name of the component.
     */
    public void setComponentName(String componentName) {
        fComponentName = componentName;
    }

    /**
     * Abstract method that is invoked to do the work to update the build
     * result.
     * 
     * @throws Exception
     *             If derived client needs to abort processing of the ant task
     */
    protected abstract void updateBuildResult() throws Exception;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    final protected void doExecute() throws Exception {
        updateBuildResult();
        updateLastContactTime();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void collectAntAttributes(List antAttributes) {
        antAttributes.add(new UUIDAntAttribute(BUILD_RESULT_UUID, fBuildResultUUID, true));
        antAttributes.add(new AntAttribute(COMPONENT_NAME, fComponentName, false));
    }

    /**
     * Retrieves the <code>IBuildResult</code> associated with the build
     * result UUID for the task using the specified properties.
     * 
     * @param properties
     *            the properties to fetch when retrieving the build result
     * @return the build result associated with the build result uuid for the
     *         task
     * @throws BuildException
     *             If the existing build result could not be successfully
     *             retrieved
     */
    final protected IBuildResult getBuildResult(String[] properties) throws BuildException {
        try {
            IBuildResultHandle resultHandle = getBuildResultHandle();

            if (properties == IBuildResult.PROPERTIES_COMPLETE) {
                return (IBuildResult) getTeamRepository().itemManager().fetchCompleteItem(resultHandle,
                        IItemManager.REFRESH, getProgressMonitor());
            } else {
                return (IBuildResult) getTeamRepository().itemManager().fetchPartialItem(resultHandle,
                        IItemManager.REFRESH, Arrays.asList(properties), getProgressMonitor());
            }

        } catch (ItemNotFoundException e) {
            throw new BuildException(NLS.bind(AntMessages._ITEM_WITH_ID_NOT_FOUND, IBuildResult.ITEM_TYPE.getName(),
                    getBuildResultUUID()), e);
        } catch (TeamRepositoryException e) {
            throw new BuildException(e);
        }
    }

    /**
     * Retrieves a string that can be used to identify the build. It attempts to
     * use the label but if a valid label does not exist the UUID for the build
     * result is used.
     */
    protected String getBuildIdentifier() {
        IBuildResult buildResult = getBuildResult(new String[] { IBuildResult.PROPERTY_LABEL });
        String identifier = buildResult.getLabel();
        if (identifier == null || identifier.trim().length() == 0) {
            identifier = buildResult.getItemId().getUuidValue();
        }
        return identifier;
    }

    @SuppressWarnings("unused")
    protected IBuildResultHandle getBuildResultHandle() throws TeamRepositoryException {
        return (IBuildResultHandle) IBuildResult.ITEM_TYPE.createItemHandle(UUID.valueOf(getBuildResultUUID()), null);
    }

    /**
     * Updates the last contact time of the engine that is currently updating
     * the request.
     */
    private void updateLastContactTime() {

        try {
            IBuildResult buildResult = getBuildResult(new String[] { IBuildResult.PROPERTY_BUILD_STATE,
                    IBuildResult.PROPERTY_BUILD_REQUESTS });
            List buildRequests = buildResult.getBuildRequests();
            if (buildRequests.size() > 0 && buildResult.getState() == BuildState.IN_PROGRESS) {
                Collection requestProperties = Arrays.asList(new String[] { IBuildRequest.PROPERTY_BUILD_ACTION,
                        IBuildRequest.PROPERTY_HANDLER });
                for (Iterator iter = buildRequests.iterator(); iter.hasNext();) {
                    IBuildRequestHandle buildRequestHandle = (IBuildRequestHandle) iter.next();
                    IBuildRequest buildRequest = (IBuildRequest) getTeamRepository().itemManager().fetchPartialItem(
                            buildRequestHandle, IItemManager.REFRESH, requestProperties, getProgressMonitor());
                    if (buildRequest.getBuildAction().getAction().equals(IBuildAction.REQUEST_BUILD)
                            && buildRequest.getHandler() != null) {
                        getTeamBuildClient().setLastContactTime(buildRequest.getHandler(), null, getProgressMonitor());
                    }
                }
            }
        } catch (Exception e) {
            log(NLS.bind(AntMessages.AbstractPublisherTask_UPDATE_LAST_CONTACT_TIME_FAILED, e.getLocalizedMessage()),
                    Project.MSG_WARN);
            logException(e, Project.MSG_WARN);
        }

    }

    /**
     * Set the uuid of the build result to update.
     * 
     * @param buildResultUUID
     *            The uuid of the build result to update.
     */
    public void setBuildResultUUID(String buildResultUUID) {
        fBuildResultUUID = buildResultUUID;
    }

    /**
     * Retrieves the uuid of the build result that is the target of this publish
     * task.
     * 
     * @return The uuid of the build result that is the target of this publish
     *         task.
     */
    protected String getBuildResultUUID() {
        return fBuildResultUUID;
    }

    /**
     * An enumerated attribute representing the valid build statuses for a build
     * result.
     */
    public static class BuildStatusAttribute extends EnumeratedAttribute {

        /*
         * (non-Javadoc)
         * 
         * @see org.apache.tools.ant.types.EnumeratedAttribute#getValues()
         */
        public String[] getValues() {
            BuildStatus[] values = BuildStatus.values();
            String validValues[] = new String[values.length];
            for (int i = 0; i < values.length; i++) {
                validValues[i] = values[i].name();
            }
            return validValues;
        }
    }
}
