/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.ant.task;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.ant.task.AbstractTeamBuildTask.IllegalAntAttributeException;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildAction;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildRequestHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.ant.AntMessages;
import com.ibm.team.build.internal.common.model.query.BaseBuildResultQueryModel.BuildResultQueryModel;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.query.IItemQueryPage;
import com.ibm.team.repository.common.query.ast.IPredicate;

/**
 * Base ant task for retrieving data from an existing build result in the team
 * repository. Ant tasks that need to retrieve information from an existing build
 * result should subclass this class.
 * <p>
 * Required attributes are:
 * <p>
 * <b>Note: Only one of the two below is required in any given invocation of the
 * task. They are mutually exclusively.</b>
 * <ul>
 * <li><b>buildResultUUID</b> - The uuid of the build result to retrieve.</li>
 * <li><b>buildDefinitionId</b> - The build definition id of which to get the last completed non-personal build result.</li>
 * </ul>
 * <p>
 * <b>See {@link AbstractTeamBuildTask} for additional required and option
 * attributes.</b>
 * 
 * @since 0.8.5
 */
@SuppressWarnings({ "restriction", "unused" })
public abstract class AbstractRetrieverTask extends AbstractTeamBuildTask {

    /**
     * The ant attribute for this task in the ant script representing the uuid
     * of the build result to be retrieved.
     */
    public static final String BUILD_RESULT_UUID = "buildResultUUID"; //$NON-NLS-1$

    /**
     * The ant attribute for this task in the ant script representing the build
     * definition id for which the last completed non-personal build result
     * should be retrieved.
     */
    public static final String BUILD_DEFINITION_ID = "buildDefinitionId"; //$NON-NLS-1$
    
    private String fBuildResultUUID = null;
    private String fBuildDefinitionId = null;

    /**
     * Set the uuid of the build result to retrieve.
     * 
     * @param buildResultUUID
     *            The uuid of the build result to retrieve.
     */
    public void setBuildResultUUID(String buildResultUUID) {
        fBuildResultUUID = buildResultUUID;
    }

    /**
     * Retrieves the uuid of the build result that is the target of this retrieve
     * task.
     * 
     * @return The uuid of the build result that is the target of this retrieve
     *         task.
     */
    public String getBuildResultUUID() {
        return fBuildResultUUID;
    }
    
    /**
     * Set the build definition id for which the last completed
     * non-personal build result should be retrieved.
     * 
     * @param buildDefinitionId The build definition id
     */
    public void setBuildDefinitionId(String buildDefinitionId) {
        fBuildDefinitionId = buildDefinitionId;
    }
    
    /**
     * Retrieves the build definition id for which the last 
     * completed non-personal build result should be retrieved.
     * 
     * @return A build definition id
     */
    public String getBuildDefinitionId() {
        return fBuildDefinitionId;
    }
    
    /**
     * Abstract method that is invoked to do the work to retrieve data
     * from the build result.
     * 
     * @throws Exception
     *             If derived client needs to abort processing of the ant task
     */
    protected abstract void retrieveDataFromBuildResult() throws Exception;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    final protected void doExecute() throws Exception {
        retrieveDataFromBuildResult();
        updateLastContactTime();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void collectAntAttributes(List antAttributes) {
        boolean isBuildDefinitionDefined = getBuildResultUUID() != null;
        antAttributes.add(new UUIDAntAttribute(BUILD_RESULT_UUID, getBuildResultUUID(), isBuildDefinitionDefined){
            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public void validate() throws IllegalAntAttributeException {
                super.validate();
                if (getBuildResultUUID() != null && getBuildDefinitionId() != null) {
                    throw new IllegalAntAttributeException(NLS.bind(
                            AntMessages.AbstractRetrieverTask_ONLY_RESULT_UUID_OR_BUILD_DEFINITION_ID_ALLOWED,
                            BUILD_RESULT_UUID, BUILD_DEFINITION_ID));
                }
            }
        });
        antAttributes.add(new AntAttribute(BUILD_DEFINITION_ID, getBuildDefinitionId(), !isBuildDefinitionDefined));
    }

    /**
     * Retrieves the <code>IBuildResult</code> associated with the build
     * result UUID for the task using the specified properties.
     * 
     * @param properties
     *            the properties to fetch when retrieving the build result
     * @return the build result associated with the build result uuid for the
     *         task
     * @throws BuildException
     *             If the existing build result could not be successfully
     *             retrieved
     */
    final protected IBuildResult getBuildResult(String[] properties) throws BuildException {
        try {
            IBuildResultHandle resultHandle = null;
        	//If the buildResultUUID was defined in the ant task, then use it to fetch the build result.  
            //Or else just use the buildDefinitionId to get the build result handle.
            if (getBuildResultUUID() != null) {
                resultHandle = getBuildResultHandle();
            } else {
                resultHandle = getBuildResultHandle(getBuildDefinitionId());
            }
            return fetchBuildResult(resultHandle, properties);
        } catch (ItemNotFoundException e) {
            throw new BuildException(NLS.bind(AntMessages._ITEM_WITH_ID_NOT_FOUND, IBuildResult.ITEM_TYPE.getName(),
                    getBuildResultUUID()), e);
        } catch (TeamRepositoryException e) {
            throw new BuildException(e);
        }
    }
    
    /**
     * Fetches the actual IBuildResult based on the IBuildResultHandle.  Also may fetch partial items depending on the properties String array.
     * 
     * @param resultHandle the IBuildResult handle to fetch from RTC
     * @param properties the properties to fetch when retrieving the build result
     * @return
     * @throws TeamRepositoryException 
     */
    protected IBuildResult fetchBuildResult(IBuildResultHandle resultHandle, String[] properties) throws TeamRepositoryException {
        if (properties == IBuildResult.PROPERTIES_COMPLETE) {
            return (IBuildResult) getTeamRepository().itemManager().fetchCompleteItem(resultHandle,
                    IItemManager.REFRESH, getProgressMonitor());
        } else {
            return (IBuildResult) getTeamRepository().itemManager().fetchPartialItem(resultHandle,
                    IItemManager.REFRESH, Arrays.asList(properties), getProgressMonitor());
        }
    }
    
    /**
     * Returns a build result handle.
     * 
     * @return The build result handle
     */
    protected IBuildResultHandle getBuildResultHandle() {
        return (IBuildResultHandle) IBuildResult.ITEM_TYPE.createItemHandle(UUID.valueOf(getBuildResultUUID()), null);
    }
       
    /**
     * Returns the last completed non-personal build result for a build
     * definition id.
     * 
     * @param buildDefinitionId The build definition id for which to return a build result
     * @param properties The array of properties to fetch from the item
     * @return The build result for the given build definition id, never <code>null</code>
     * @throws TeamRepositoryException If an error occurs getting the build result or a build result isn't found
     */
    protected IBuildResultHandle getBuildResultHandle(String buildDefinitionId) throws TeamRepositoryException {
        IBuildDefinition buildDefinition = getBuildDefinition(buildDefinitionId);
        if (buildDefinition == null) {
            throw new BuildException(NLS.bind(AntMessages.AbstractRetrieverTask_NO_BUILD_DEFINITION,
                    buildDefinitionId));
        }
        IBuildDefinitionHandle buildDefinitionHandle = (IBuildDefinitionHandle) buildDefinition.getItemHandle();
        BuildResultQueryModel buildResultQueryModel = BuildResultQueryModel.ROOT;
        IItemQuery query = IItemQuery.FACTORY.newInstance(buildResultQueryModel);
        IPredicate left = buildResultQueryModel.buildDefinition()._eq(query.newItemHandleArg());
        IPredicate middle = buildResultQueryModel.personalBuild()._isFalse();
        IPredicate right = buildResultQueryModel.buildState()._eq(BuildState.COMPLETED.name());
        query.filter(left._and(middle)._and(right));
        query.orderByDsc(buildResultQueryModel.buildStartTime());
        query.setResultLimit(1);
        IItemQueryPage resultPage = getTeamBuildClient().queryItems(query, new Object[] { buildDefinitionHandle }, 1, getProgressMonitor());
        IBuildResultHandle buildResultHandle = null;
        if (resultPage.getSize() > 0) {
            buildResultHandle = (IBuildResultHandle) resultPage.getItemHandles().get(0);
        } else {
            throw new BuildException(NLS.bind(AntMessages.AbstractRetrieverTask_NO_BUILD_RESULT, buildDefinitionId));
        }
        return buildResultHandle;
    }
    
    /**
     * Helper method to get the build definition. If it is not found a
     * <code>null</code> will be returned.
     * 
     * @param buildDefinitionId
     *            The id of the build definition to retrieve.
     * @return The build definition represented by the build definition id or
     *         <code>null</code> if the build definition is not found
     * @throws TeamRepositoryException
     *             If an error occurred retrieving the build definition.
     * @throws BuildException
     *             If a build definition for the build definition id could not
     *             be found.
     */
    protected IBuildDefinition getBuildDefinition(String buildDefinitionId) throws TeamRepositoryException,
            BuildException {
        IBuildDefinition buildDefinition = getTeamBuildClient().getBuildDefinition(buildDefinitionId,
                getProgressMonitor());
        return buildDefinition;
    }
    
    /**
     * Updates the last contact time of the engine that is currently retrieving
     * the request.
     */
    private void updateLastContactTime() {
        try {
            IBuildResult buildResult = getBuildResult(new String[] { IBuildResult.PROPERTY_BUILD_STATE,
                    IBuildResult.PROPERTY_BUILD_REQUESTS });
            List buildRequests = buildResult.getBuildRequests();
            if (buildRequests.size() > 0 && buildResult.getState() == BuildState.IN_PROGRESS) {
                Collection requestProperties = Arrays.asList(new String[] { IBuildRequest.PROPERTY_BUILD_ACTION,
                        IBuildRequest.PROPERTY_HANDLER });
                for (Iterator iter = buildRequests.iterator(); iter.hasNext();) {
                    IBuildRequestHandle buildRequestHandle = (IBuildRequestHandle) iter.next();
                    IBuildRequest buildRequest = (IBuildRequest) getTeamRepository().itemManager().fetchPartialItem(
                            buildRequestHandle, IItemManager.REFRESH, requestProperties, getProgressMonitor());
                    if (buildRequest.getBuildAction().getAction().equals(IBuildAction.REQUEST_BUILD)
                            && buildRequest.getHandler() != null) {
                        getTeamBuildClient().setLastContactTime(buildRequest.getHandler(), null, getProgressMonitor());
                    }
                }
            }
        } catch (Exception e) {
            //Need to unwrap the exception to see if it is an expected ItemNotFoundException
            if (e.getCause() instanceof ItemNotFoundException) {
              //Swallow ItemNotFoundException and just don't update the last contact time.    
              return;
            }
            log(NLS.bind(AntMessages.AbstractPublisherTask_UPDATE_LAST_CONTACT_TIME_FAILED, e.getLocalizedMessage()),
                    Project.MSG_WARN);
            logException(e, Project.MSG_WARN);
        }
    }

}

