/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.util.ObfuscationHelper;
import com.ibm.team.repository.common.util.Base64;

/**
 * Helper class for dealing with passwords.
 */
public class PasswordHelper {
    private static final String DES3EncType = "DESede";  //$NON-NLS-1$
    private static final String PasswordTag = "password";  //$NON-NLS-1$
    private static final String VersionTag = "version";  //$NON-NLS-1$
    private static final String DefaultVersion = "0";  //$NON-NLS-1$
    // The version of the password file - this version uses 3DES with the following secretKey
    private static final String CurrentVersion = "1";  //$NON-NLS-1$ 
    private static final byte [] tripleDESKey = new byte[] { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, 0x01, 0x02, 0x03, 0x04, 0x04, 0x03, 0x02, 0x01 };
    
    /**
     * Get the decrypted password from the given password file.
     * 
     * @param passwordFile
     *            The file containing the encrypted password.
     * @return The decrypted password.
     * @throws GeneralSecurityException
     *             If there is a problem decrypting.
     * @throws IOException
     *             If there is a problem reading the file.
     */
    public static String getPassword(File passwordFile) throws GeneralSecurityException, IOException {
        if (!passwordFile.exists()) {
            throw new FileNotFoundException(Messages.PasswordHelper_FILE_MISSING);
        }
        if (passwordFile.isDirectory()) {
            throw new FileNotFoundException(Messages.PasswordHelper_FILE_IS_DIRECTORY);
        }
        String password = null;
        FileInputStream inputStream = new FileInputStream(passwordFile);
        try {
            Properties properties = new Properties();
            properties.loadFromXML(inputStream);
            password = properties.getProperty(PasswordTag);
            if (password == null) {
                throw new IOException(Messages.PasswordHelper_INVALID_FORMAT);
            }
            String version = properties.getProperty(VersionTag, DefaultVersion);
            if (version.equals(CurrentVersion)) {
                password = decryptString(password, tripleDESKey);
            }
            else {
                password = ObfuscationHelper.decryptString(password);
            }
        } catch (InvalidPropertiesFormatException ipfe) {
            try {
                // close stream before reopening it
                inputStream.close();
            } catch (IOException e) {
                // ignore
            }
            inputStream = new FileInputStream(passwordFile);
            StringBuffer fileContent = new StringBuffer();
            byte[] buffer = new byte[256];
            int bytesRead = inputStream.read(buffer);
            while (bytesRead != -1) {
                fileContent.append(new String(buffer, 0, bytesRead));
                bytesRead = inputStream.read(buffer);
            }
            String content = fileContent.toString();
            if (content.length() == 0) {
                throw new IOException(Messages.PasswordHelper_FILE_EMPTY);
            }
            password = new String(ObfuscationHelper.decrypt(content.getBytes()));
        } finally {
            try {
                inputStream.close();
            } catch (IOException e) {
                // ignore
            }
        }
        return password;
    }
    
    /**
     * Encrypt the given <code>password</code> bytes in a non-secure way and store it in the given <code>passwordFile</code>
     * @param passwordFile
     * @param password
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public static void createPasswordFile(File passwordFile, String password) throws IOException, GeneralSecurityException {
        FileOutputStream outputStream = null;
        try
        {
            outputStream = new FileOutputStream(passwordFile);
            Properties properties = new Properties();
            properties.put(PasswordTag, encryptString(password, tripleDESKey)); 
            properties.put(VersionTag, CurrentVersion);
            properties.storeToXML(outputStream, "6.0", IContent.ENCODING_UTF_8); //$NON-NLS-1$
        }
        finally {
            if (outputStream != null) {
                outputStream.close();
            }
        }
    }
    
    /**
     * Encrypt the given <code>clearText</code> string in UTF-8 using <code>key</code>
     * 
     * @param clearText
     *            The string to encrypt.
     * @param key
     *            The cipher key in bytes            
     * @return The encrypted string. The string can be decrypted by calling
     *           {@link #decryptString}.
     * @throws GeneralSecurityException
     *             If there is a problem encrypting.
     * @throws UnsupportedEncodingException
     *             If the required encodings were not found.
     */
    private static String encryptString(String clearText, byte[] key) throws UnsupportedEncodingException, GeneralSecurityException {
        if (clearText == null) {
            throw new IllegalArgumentException("Parameter \"clearText\" must not be null."); //$NON-NLS-1$
        }
        
        if (key == null) {
            throw new IllegalArgumentException("Parameter \"key\" must not be null."); //$NON-NLS-1$
        }

        SecretKeySpec keySpec = new SecretKeySpec(key, DES3EncType);
        Cipher cp = Cipher.getInstance(DES3EncType);
        cp.init(Cipher.ENCRYPT_MODE, keySpec);
        byte[] cipherText = cp.doFinal(clearText.getBytes("UTF-8")); //$NON-NLS-1$
        return new String(Base64.encode(cipherText), "UTF-8"); //$NON-NLS-1$
    }
    
    /***
     * Decrypt the given <code>cipherText</code> using <code>key</code>
     * 
     * @param cipherText
     * @param key
     * @return the cleartext encoded in UTF-8
     * @throws UnsupportedEncodingException
     * @throws GeneralSecurityException
     */
    
    private static String decryptString(String cipherText, byte [] key) throws UnsupportedEncodingException, GeneralSecurityException {
        if (cipherText == null) {
            throw new IllegalArgumentException("Parameter \"cipherText\" must not be null."); //$NON-NLS-1$
        }
        
        if (key == null) {
            throw new IllegalArgumentException("Parameter \"key\" must not be null."); //$NON-NLS-1$
        }
        
        SecretKeySpec keySpec = new SecretKeySpec(key, DES3EncType);
        Cipher cp = Cipher.getInstance(DES3EncType);
        cp.init(Cipher.DECRYPT_MODE, keySpec);
        byte [] clearText = cp.doFinal(Base64.decode(cipherText.getBytes("UTF-8"))); //$NON-NLS-1$
        return new String(clearText, "UTF-8"); //$NON-NLS-1$   
    }
}
