/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal;

import java.lang.reflect.Method;

import com.ibm.team.repository.client.IRetryPolicy;
import com.ibm.team.repository.common.RetryableDatabaseException;
import com.ibm.team.repository.common.transport.ConnectionException;
import com.ibm.team.repository.common.transport.HostUnknownException;
import com.ibm.team.repository.common.transport.ServiceNotAvailableException;
import com.ibm.team.repository.transport.client.NoResponseException;

/**
 * Retry policy for the Jazz Build engine
 * To be used only when retries are enabled for builds (build engine/ant tasks)
 * 
 * @since 0.9.5 (RTC 4.0.5)
 */
public class RetryPolicy implements IRetryPolicy {
    
    /**
     * Environment variable for determining if this retry policy is to be retried.
     */
    private static final String ENABLE_RETRY_VARIABLE = "team.build.retry"; //$NON-NLS-1$
    
    /**
     * Flag for whether to enable this retry policy. Default is false.
     */
    private static final boolean ENABLE_RETRY = Boolean.parseBoolean(System.getProperty(
            ENABLE_RETRY_VARIABLE, "false"));  //$NON-NLS-1$
    

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    @Override
    public boolean autoReLogin() {
        return true;
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    @Override
    public int getMaxRetries(String serviceName, Method method) {
        return 5;
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    @Override
    public void pauseBeforeRetry(String serviceName, Method method, int retryNumber) {
        assert retryNumber >= 1;
        try {
            // sleep for 2^retryNumber seconds, max 60
            int secs = 60;
            if (retryNumber < 6) { // check up-front to avoid overflow
                secs = (1 << retryNumber);
            }
            Thread.sleep(secs * 1000);
        } catch (InterruptedException exception) {
            // ignore
        }
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    @Override
    public boolean isRetryable(String serviceName, Method method, Throwable failure) {

        // TODO consider supporting an annotation that can be put on a method
        // to further exclude it from retry or to enable if for retry with other
        // types of failures (i.e. SocketTimeoutException).

        // The ITeamContentService.put method parameters are handled in a different
        // manner. There is an input stream embedded within the TeamContent parameter.
        // If there is a failure, it can't be retried because the input stream needs
        // to be reset.
        if ("ITeamContentService".equals(serviceName) && "put".equals(method)) {  //$NON-NLS-1$//$NON-NLS-2$
            return false;
        }
        
        // See if the failure is retryable.
        return isRetryable(failure);
    }

    /**
     * @param failure The failure we want to know about
     * @return <code>true</code> if the failure is a retryable type of failure
     * <code>false</code> otherwise.
     */
    public static boolean isRetryable(Throwable failure) {
        // See if the failure is a retryable failure.
        for (; failure != null; failure = failure.getCause()) {
            if (failure instanceof RetryableDatabaseException) {
                return true;
            }
            // specific TeamServiceException subclasses
            if (failure instanceof ConnectionException) {
                return true;
            }
            if (failure instanceof HostUnknownException) {
                return true;
            }
            if (failure instanceof NoResponseException) {
                return true;
            }
            if (failure instanceof ServiceNotAvailableException) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Whether the retry policy should be enabled for build. Present for
     * initial release to allow us control over when to activate.
     * @return <code>true</code> if this policy should be enabled
     * <code>false</code> otherwise.
     */
    public static boolean isRetryEnabled() {
        return ENABLE_RETRY;
    }
}
