/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.osgi.util.NLS;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.DefaultHandler2;

import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * Abstract base class for parser handler classes.
 * 
 * <pre></pre>
 * 
 * This parser class works by maintaining a stack of contexts - associated with
 * XML elements - to track the current parsing environment. The contexts are
 * represented by java enums that implement an internal <code>ContextType</code>
 * interface. Implementing child parser classes must implement their own
 * <code>ContextType</code> enum to represent the elements they parse.
 * 
 * <pre></pre>
 * 
 * They must also implement methods to map element names to these context types,
 * validate child contexts given a parent context and parse the appropriate
 * element using the current context as a guide.
 * 
 * <pre></pre>
 * 
 * The parser is implemented to allow for very lenient behavior in the presence
 * of poorly-formed XML. Implementers can decide to ignore unrecognized elements
 * and specify that certain attributes and sub-elements are unnecessary for a
 * successful parse. This base class provides several helper methods for parsing
 * and validating attributes. These methods easily allow the implementer to
 * specify whether attributes are required, necessary for continued parsing, may
 * be non-empty, etc.
 * 
 * <pre></pre>
 * 
 * See <code>ComponentMapParser</code> and <code>JdtXmlCompileLogParser</code>
 * as examples.
 * 
 * <pre></pre>
 * 
 * Limitations: The class does not currently maintain an xpath-type memory of
 * the current element hierarchy. This means that child classes can only
 * validate a single level of context - i.e. validate that node A is valid in
 * the context of node B, but not that it is in context D/C/B as opposed to
 * context E/F/B.
 */
public abstract class AbstractParserHandlerBase extends DefaultHandler2 {

    /**
     * The file to be parsed by this handler.
     */
    private File fFileToParse;

    /**
     * The string to be parsed by this handler.
     */
    private String fStringToParse;

    /**
     * Maintain a stack of the current element context.
     */
    private Stack<ElementContext> fElementContext = new Stack<ElementContext>();

    /**
     * The list of errors that occurred while parsing the xml.
     */
    private List<String> fParseErrors = new LinkedList<String>();

    /**
     * Instantiate this parser base class with the given File to be parsed.
     * 
     * @param fileToParse
     *            The file to be parsed.
     */
    protected AbstractParserHandlerBase(File fileToParse) {
        ValidationHelper.validateNotNull("fileToParse", fileToParse); //$NON-NLS-1$
        fFileToParse = fileToParse;
    }

    /**
     * Instantiate this parser base class with the given String to be parsed.
     * 
     * @param stringToParse
     *            The string to be parsed.
     */
    protected AbstractParserHandlerBase(String stringToParse) {
        ValidationHelper.validateNotEmpty("stringToParse", stringToParse); //$NON-NLS-1$
        fStringToParse = stringToParse;
    }

    /**
     * Parses the file or string specified in the constructor.
     * 
     * @throws SAXException
     *             if a problem occurs while parsing the XML file
     * @throws IOException
     *             if the file to be parsed cannot be open or read
     * @throws ParserConfigurationException 
     */
    public void parse() throws SAXException, IOException, ParserConfigurationException {
        SAXParserFactory inst = SAXParserFactory.newInstance();
        // Disable dereferencing external entities declared with SYSTEM/PUBLIC keyword
        inst.setFeature("http://xml.org/sax/features/external-general-entities", false); //$NON-NLS-1$
        // Java 7 by default seems to have the FEATURE_SECURE_PROCESSING flag already set
        // This code just sets the flag explicitly. 
        inst.setFeature(javax.xml.XMLConstants.FEATURE_SECURE_PROCESSING, true);

        SAXParser sx = inst.newSAXParser();
        XMLReader xmlReader = sx.getXMLReader();
        xmlReader.setContentHandler(this);
        xmlReader.setErrorHandler(this);
        xmlReader.setProperty("http://xml.org/sax/properties/lexical-handler", this); //$NON-NLS-1$
        // Disable loading external dtds
        xmlReader.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false); //$NON-NLS-1$

        BufferedInputStream bufferedInputStream = null;
        InputSource source = new InputSource();

        try {
            if (fFileToParse != null) {
                bufferedInputStream = new BufferedInputStream(new FileInputStream(fFileToParse));
                source.setByteStream(bufferedInputStream);
            } else {
                source.setCharacterStream(new StringReader(fStringToParse));
            }
            xmlReader.parse(source);
        } finally {
            if (bufferedInputStream != null) {
                bufferedInputStream.close();
            }
        }
    }

    protected void error(String errorMessage) {
        addParseError(errorMessage);
    }

    /*
     * If the "fatal" flag is set, we throw an exception to force skipping the
     * node currently being parsed. Otherwise we just log the error message.
     */
    protected void error(String errorMessage, boolean fatal) throws SkipElementException {
        addParseError(errorMessage);
        if (fatal) {
            throw new SkipElementException();
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    public void error(SAXParseException e) {
        addParseError(e.getLocalizedMessage());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        fElementContext.peek().addCharacterData(ch, start, length);
    }

    protected StringBuffer getCurrentCharacterData() {
        return fElementContext.peek().characterData;
    }

    /**
     * Retrieves the non fatal errors that occurred while parsing the xml.
     * 
     * @return The errors that occurred while parsing the xml. It may be empty
     *         but never <code>null</code>.
     */
    public String[] getParseErrors() {
        return fParseErrors.toArray(new String[fParseErrors.size()]);
    }

    /**
     * Add a non-fatal parse error to the list of errors that occurred while
     * parsing the xml.
     * 
     * @param message
     *            The error message detailing the parse error.
     */
    public void addParseError(String message) {
        fParseErrors.add(message);
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    public final void startElement(String uri, String localName, String qName, Attributes attributes)
            throws SAXException {
        String name = localName.equals("") ? qName : localName; //$NON-NLS-1$

        if (!startNewContext(name)) {
            return;
        }

        if (getCurrentContext() == BaseContextType.PARSING_IGNORED_NODE) {
            return;
        }

        try {
            startChildElement(attributes);
        } catch (SkipElementException exception) {
            /*
             * If this exception is thrown, it means that there are
             * irrecoverable errors in the element being parsed, so skip parsing
             * this element.
             */
            endCurrentContext();
            startNewIgnoredElementContext();
            return;
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public final void endElement(String uri, String localName, String qName) throws SAXException {
        if (!(getCurrentContext() instanceof BaseContextType)) {
            try {
                endChildElement();
            } catch (SkipElementException exception) {
                // current element was skipped
            }
        }

        endCurrentContext();
    }

    /*
     * Called to begin parsing an element.
     */
    protected abstract void startChildElement(Attributes attributes) throws SAXException, SkipElementException;

    /*
     * Called to end parsing any non-root-element
     */
    protected abstract void endChildElement() throws SAXException, SkipElementException;

    /*
     * Child classes should implement this method to return the appropriate
     * context for the specified element name. Should return null if the element
     * name is not recognized.
     */
    protected abstract ContextType getContextForElement(String elementName);

    /*
     * Child classes should implement this method to validate that the specified
     * child context is valid in the specified parent context and return the
     * appropriate context.
     */
    protected abstract ContextType getValidChildElementContext(ContextType childContext, ContextType parentContext);

    /*
     * Child classes should implement this method to validate that the specified
     * context is a valid root context and return the appropriate context.
     */
    protected abstract ContextType getValidRootElementContext(ContextType rootContext) throws SAXParseException;

    public final void startDocument() {
        startNewContext(BaseContextType.PARSING_NONE, null);
    }

    protected String getCurrentContextName() {
        return fElementContext.peek().contextName;
    }

    protected ContextType getCurrentContext() {
        return fElementContext.peek().contextType;
    }

    protected ContextType getParentContext() {
        return fElementContext.peek().parentContext == null ? null : fElementContext.peek().parentContext.contextType;
    }

    protected String getParentContextName() {
        return fElementContext.peek().parentContext == null ? null : fElementContext.peek().parentContext.contextName;
    }

    private ContextType endCurrentContext() {
        return fElementContext.pop().contextType;
    }

    /*
     * Push a new element context onto the stack to represent the parser
     * descending into another child element in the xml.
     */
    private void startNewContext(ContextType newContext, String contextName) {
        ElementContext parentContext = null;

        if (fElementContext.size() > 0) {
            parentContext = fElementContext.peek();
        }

        fElementContext.push(new ElementContext(newContext, contextName, parentContext));
    }

    /*
     * Call getContext to map the given elementName to the correct context, then
     * pass the returned context to startNewContext.
     */
    private boolean startNewContext(String elementName) throws SAXParseException {

        ContextType newContext = null;
        boolean rootContext = false;

        if (getCurrentContext() instanceof BaseContextType) {
            switch ((BaseContextType) getCurrentContext()) {
                case PARSING_IGNORED_NODE:
                    startNewIgnoredElementContext();
                    return false;
                case PARSING_EMPTY_NODE:
                    unexpectedElementError(elementName);
                    startNewIgnoredElementContext();
                    return false;
                case PARSING_NONE:
                    rootContext = true;
            }
        }

        newContext = getContextForElement(elementName);

        if (newContext != null) {
            if (rootContext) {
                newContext = getValidRootElementContext(newContext);
            } else {
                newContext = getValidChildElementContext(newContext, getCurrentContext());
            }
        }

        if (newContext == null) {
            unexpectedElementError(elementName);
            startNewIgnoredElementContext();
            return false;
        }

        startNewContext(newContext, elementName);
        return true;
    }

    /*
     * Push a new element context onto the stack to represent the parser
     * descending into an element that the handler wants to ignore.
     */
    private void startNewIgnoredElementContext() {
        startNewContext(BaseContextType.PARSING_IGNORED_NODE, null);
    }

    protected ContextType getIgnoreContext() {
        return BaseContextType.PARSING_IGNORED_NODE;
    }

    /*
     * Push a new element context onto the stack that represents the parser
     * descending into what should be an empty element (no content and no
     * children).
     */
    protected ContextType getEmptyContext() {
        return BaseContextType.PARSING_EMPTY_NODE;
    }

    /*
     * Convenience method for throwing an exception when an expected sub-element
     * is missing.
     */
    protected void verifySubElement(String subElementName, boolean isSet, boolean necessary)
            throws SkipElementException {
        if (!isSet) {
            error(NLS.bind(ParserMessages.MISSING_REQUIRED_CHILD_ELEMENT, subElementName, getCurrentContextName()),
                    necessary);
        }
    }

    /*
     * Return the value of the specified attribute, or null it if is not set.
     */
    protected String getOptionalAttribute(Attributes attributes, String attributeName) throws SkipElementException {
        return getOptionalAttribute(attributes, attributeName, true);
    }

    /*
     * Return the value of the specified attribute, or null it if is not set.
     * Can also specify if the empty string is a valid value.
     */
    protected String getOptionalAttribute(Attributes attributes, String attributeName, boolean mayBeEmpty)
            throws SkipElementException {
        return doGetAttribute(attributes, attributeName, false, false, mayBeEmpty);
    }

    /*
     * Validate that the specified attribute is set and that it is not empty.
     * The "necessary" flag specifies whether this attribute is essential to
     * successful parsing of the node. If "necessary" is true, we'll throw an
     * exception that causes the parser to skip the node.
     */
    protected String getRequiredAttribute(Attributes attributes, String attributeName, boolean necessary)
            throws SkipElementException {
        String value = doGetAttribute(attributes, attributeName, necessary, true);
        if (value == null) {
            /*
             * We'll only get to here if the attribute was not set (or was
             * empty) and "necessary" was false (if it was true, an exception
             * would have been thrown). So set the value to the empty string so
             * that the caller will have a valid String to continue with.
             */
            value = ""; //$NON-NLS-1$
        }
        return value;
    }

    /*
     * Same behavior as the above method, but caller can also specify that the
     * attribute may be the empty String by setting "mayBeEmpty" to true.
     */
    protected String getRequiredAttribute(Attributes attributes, String attributeName, boolean necessary,
            boolean mayBeEmpty) throws SkipElementException {
        return doGetAttribute(attributes, attributeName, necessary, true, mayBeEmpty);
    }

    private String doGetAttribute(Attributes attributes, String attributeName, boolean necessary, boolean required)
            throws SkipElementException {
        return doGetAttribute(attributes, attributeName, necessary, required, false);
    }

    private String doGetAttribute(Attributes attributes, String attributeName, boolean necessary, boolean required,
            boolean mayBeEmpty) throws SkipElementException {
        String attributeValue = attributes.getValue(attributeName);
        String errorMessage = null;

        if (attributeValue == null) {
            if (required) {
                errorMessage = NLS.bind(ParserMessages.MISSING_REQUIRED_ATTRIBUTE, attributeName,
                        getCurrentContextName());
            }
        } else if (attributeValue.length() == 0 && !mayBeEmpty) {
            errorMessage = NLS.bind(ParserMessages.INVALID_ATTRIBUTE_VALUE, new String[] { attributeValue,
                    attributeName, getCurrentContextName() });

            if (required) {
                attributeValue = null;
            }
        }

        if (errorMessage != null) {
            error(errorMessage, necessary);
        }

        return attributeValue;
    }

    /*
     * If the specified attribute is set, verify that its value is one of the
     * specified valid values, then return it.
     */
    protected String getOptionalAttribute(Attributes attributes, String attributeName, String[] validValues)
            throws SkipElementException {
        return doGetAttribute(attributes, attributeName, validValues, false, false);
    }

    /*
     * Verify that the specified attribute is set and that it is set to one of
     * the specified valid values.
     */
    protected String getRequiredAttribute(Attributes attributes, String attributeName, String[] validValues,
            boolean necessary) throws SkipElementException {
        return doGetAttribute(attributes, attributeName, validValues, necessary, true);
    }

    private String doGetAttribute(Attributes attributes, String attributeName, String[] validValues, boolean necessary,
            boolean required) throws SkipElementException {
        String attributeValue = doGetAttribute(attributes, attributeName, necessary, required);

        if (attributeValue != null) {
            boolean valid = false;
            for (String validValue : validValues) {
                if (validValue.equals(attributeValue)) {
                    valid = true;
                }
            }

            if (!valid) {
                error(NLS.bind(ParserMessages.INVALID_ATTRIBUTE_VALUE, new String[] { attributeValue, attributeName,
                        getCurrentContextName() }), necessary);
                attributeValue = ""; //$NON-NLS-1$
            }
        }

        return attributeValue;
    }

    /*
     * If the specified attribute is set, verify that it is a number. If <code>positive</code>
     * is set, validate that it is positive and, if <code>nonZero</code> is
     * set, verify that it is not zero.
     */
    protected Float getOptionalNumericAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero) throws SkipElementException {
        return doGetNumericAttribute(attributes, attributeName, positive, nonZero, false, false);
    }

    /*
     * If the specified attribute is set, verify that it is a number. If <code>positive</code>
     * is set, validate that it is positive and, if <code>nonZero</code> is
     * set, verify that it is not zero. If the attribute is not set or is
     * invalid return the specified default value.
     */
    protected Float getOptionalNumericAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero, float defaultValue) throws SkipElementException {
        Float value = doGetNumericAttribute(attributes, attributeName, positive, nonZero, false, false);
        return value == null ? defaultValue : value;
    }

    /*
     * Validate that the specified attribute is set, and that it is a number. If
     * <code>positive</code> is set, also validate that it is positive.
     */
    protected Float getRequiredNumericAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero) throws SkipElementException {
        return doGetNumericAttribute(attributes, attributeName, positive, nonZero, true, true);
    }

    /*
     * Validate that the specified attribute is set, and that it is a number. If
     * <code>positive</code> is set, also validate that it is positive. If the
     * attribute is not set or is invalid, return the specified default value.
     */
    protected float getRequiredNumericAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero, float defaultValue) throws SkipElementException {
        Float value = doGetNumericAttribute(attributes, attributeName, positive, nonZero, false, true);
        return value == null ? defaultValue : value;
    }

    private Float doGetNumericAttribute(Attributes attributes, String attributeName, boolean positive, boolean nonZero,
            boolean necessary, boolean required) throws SkipElementException {

        Float value = null;
        String attributeValue = doGetAttribute(attributes, attributeName, necessary, required);

        if (attributeValue != null) {
            boolean invalid = false;

            try {
                value = Float.parseFloat(attributeValue);
            } catch (NumberFormatException exception) {
                invalid = true;
            }

            if (invalid || (positive && value < 0) || (nonZero && value == 0)) {
                error(NLS.bind(ParserMessages.INVALID_ATTRIBUTE_VALUE, new String[] { attributeValue, attributeName,
                        getCurrentContextName() }), necessary);
                value = null;
            }
        }

        return value;
    }

    /*
     * If the specified attribute is set, verify that it is a whole number. If
     * <code>positive</code> is set, verify that it is positive and, if <code>nonZero</code>
     * is set, verify that it is not zero.
     */
    protected Long getOptionalWholeNumberAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero) throws SkipElementException {
        return doGetWholeNumberAttribute(attributes, attributeName, positive, nonZero, false, false);
    }

    /*
     * If the specified attribute is set, verify that it is a whole number. If
     * <code>positive</code> is set, verify that it is positive and, if <code>nonZero</code>
     * is set, verify that it is not zero. If the attribute is not set or is
     * invalid, return the specified default value.
     */
    protected Long getOptionalWholeNumberAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero, long defaultValue) throws SkipElementException {
        Long value = doGetWholeNumberAttribute(attributes, attributeName, positive, nonZero, false, false);
        return value == null ? defaultValue : value;
    }

    /*
     * Validate that the specified attribute is set, and that it is a whole
     * number. If <code>positive</code> is set, also validate that it is
     * positive.
     */
    protected Long getRequiredWholeNumberAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero) throws SkipElementException {
        return doGetWholeNumberAttribute(attributes, attributeName, positive, nonZero, true, true);
    }

    /*
     * Validate that the specified attribute is set, and that it is a whole
     * number. If <code>positive</code> is set, also validate that it is
     * positive. If the attribute is not set or is invalid, return the specified
     * default value.
     */
    protected long getRequiredWholeNumberAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero, long defaultValue) throws SkipElementException {
        Long value = doGetWholeNumberAttribute(attributes, attributeName, positive, nonZero, false, true);
        return value == null ? defaultValue : value;
    }

    private Long doGetWholeNumberAttribute(Attributes attributes, String attributeName, boolean positive,
            boolean nonZero, boolean necessary, boolean required) throws SkipElementException {

        Long value = null;
        String attributeValue = doGetAttribute(attributes, attributeName, necessary, required);

        if (attributeValue != null) {
            boolean invalid = false;

            try {
                value = Long.parseLong(attributeValue);
            } catch (NumberFormatException exception) {
                invalid = true;
            }

            if (invalid || (positive && value < 0) || (nonZero && value == 0)) {
                error(NLS.bind(ParserMessages.INVALID_ATTRIBUTE_VALUE, new String[] { attributeValue, attributeName,
                        getCurrentContextName() }), necessary);
                value = null;
            }
        }

        return value;
    }

    protected void unexpectedElementError() {
        unexpectedElementError(getCurrentContextName(), getParentContextName());
    }

    protected void unexpectedElementError(String unexpectedElementName) {
        unexpectedElementError(unexpectedElementName, getCurrentContextName());
    }

    protected void unexpectedElementError(String unexpectedElementName, String parentElementName) {
        if (parentElementName == null) {
            error(NLS.bind(ParserMessages.UNEXPECTED_ELEMENT, unexpectedElementName));
        } else {
            error(NLS.bind(ParserMessages.UNEXPECTED_CHILD_ELEMENT, unexpectedElementName, parentElementName));
        }
    }

    protected interface ContextType {
        // empty
    }

    protected enum BaseContextType implements ContextType {
        /**
         * Currently in the global context, not yet parsing an element.
         */
        PARSING_NONE,

        /**
         * Currently parsing a node the handler doesn't care about
         */
        PARSING_IGNORED_NODE,

        /**
         * Currently parsing a leaf node with no content.
         */
        PARSING_EMPTY_NODE
    }

    protected class ElementContext {
        ContextType contextType;
        String contextName;
        StringBuffer characterData;
        ElementContext parentContext;

        ElementContext(ContextType type, String name, ElementContext parent) {
            contextType = type;
            contextName = name;
            parentContext = parent;
        }

        void addCharacterData(char[] ch, int start, int length) {
            if (characterData == null) {
                characterData = new StringBuffer();
            }
            characterData.append(new String(ch, start, length));
        }
    }

}
