/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser;

import java.io.File;
import java.util.Hashtable;

import org.eclipse.osgi.util.NLS;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * The class parses xml mapping files containing the information specifying
 * which packages map to which components. An example entry is:
 * 
 * <pre>
 *       &lt;component_mapping&gt;
 *            &lt;component name=&quot;Build&quot;&gt;
 *                &lt;includePackage name=&quot;com.ibm.team.build.*&quot;/&gt;
 *            &lt;/component&gt;
 *       &lt;/component_mapping&gt;
 * </pre>
 * 
 * The package name can either be a prefix (ending in .*) as in the example
 * above, or it can be a specific package name.
 * 
 * <pre></pre>
 */
public class ComponentMapParser extends AbstractParserHandlerBase {

    /**
     * During parsing, stores the name of the <component> currently being
     * parsed.
     */
    private String fCurrentComponentName = null;

    /*
     * Table containing the package-name-to-component mappings
     */
    Hashtable<String, String> fPackageNameToComponentMappings = new Hashtable<String, String>();

    /*
     * Table containing the package-prefix-to-component mappings
     */
    Hashtable<String, String> fPackagePrefixToComponentMappings = new Hashtable<String, String>();

    /*
     * The default component name - specified by "*".
     */
    private String fDefaultComponentName = null;

    /**
     * Create a ComponentMapParser object.
     * 
     * @param mappingFile
     *            The file containing the mapping information in XML format.
     */
    public ComponentMapParser(File mappingFile) {
        super(mappingFile);
    }

    /**
     * Provide access to the default component name that was specified in the
     * mapping file using "*" as the package name.
     * 
     * @return The default component name or null if none was specified.
     */
    public String getDefaultComponentName() {
        return fDefaultComponentName;
    }

    /**
     * Provide access to the {@link Hashtable} containing the mappings from
     * specific package names (not prefixes) to component names.
     * 
     * @return The {@link Hashtable} containing the mappings.
     */
    public Hashtable<String, String> getPackageNameToComponentMappings() {
        return fPackageNameToComponentMappings;
    }

    /**
     * Provide access to the {@link Hashtable} containing the mappings from
     * package prefixes to component names.
     * 
     * @return The {@link Hashtable} containing the mappings.
     */
    public Hashtable<String, String> getPackagePrefixToComponentMappings() {
        return fPackagePrefixToComponentMappings;
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    public void startChildElement(Attributes attributes) throws SAXException, SkipElementException {
        switch ((MappingContextType) getCurrentContext()) {
            case PARSING_COMPONENT:
                fCurrentComponentName = getRequiredAttribute(attributes, TAG_NAME, true);
                break;
            case PARSING_INCLUDE_PACKAGE:
                addPackageMapping(getRequiredAttribute(attributes, TAG_NAME, true, true));
                break;
        }
    }

    protected String getCurrentComponentName() {
        return fCurrentComponentName;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getValidChildElementContext(ContextType childContext, ContextType parentContext) {

        switch ((MappingContextType) parentContext) {
            case PARSING_COMPONENT_MAPPING:
                if (childContext != MappingContextType.PARSING_COMPONENT) {
                    return null;
                }

                break;
            case PARSING_COMPONENT:
                if (childContext != MappingContextType.PARSING_INCLUDE_PACKAGE) {
                    return null;
                }

                break;
            case PARSING_INCLUDE_PACKAGE:
                return null;
        }

        return childContext;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getValidRootElementContext(ContextType rootContext) throws SAXParseException {
        if (rootContext != MappingContextType.PARSING_COMPONENT_MAPPING) {
            return null;
        }

        return rootContext;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getContextForElement(String elementName) {

        if (elementName.equals(TAG_COMPONENT)) {
            return MappingContextType.PARSING_COMPONENT;
        } else if (elementName.equals(TAG_COMPONENT_MAPPING)) {
            return MappingContextType.PARSING_COMPONENT_MAPPING;
        } else if (elementName.equals(TAG_INCLUDE_PACKAGE)) {
            return MappingContextType.PARSING_INCLUDE_PACKAGE;
        }

        return null;
    }

    /*
     * Add a new mapping from the given package name to the <component> element
     * currently being parsed. Can be either a prefix or an absolute package
     * name.
     */
    protected void addPackageMapping(String packageName) throws SAXException {
        boolean prefix = false;
        if (packageName.endsWith(".*")) { //$NON-NLS-1$
            packageName = packageName.substring(0, packageName.length() - 2);
            prefix = true;
        } else if (packageName.endsWith("*")) { //$NON-NLS-1$
            if (packageName.length() == 1) {
                if (fDefaultComponentName != null && !getCurrentComponentName().equals(fDefaultComponentName)) {
                    throw new SAXException(NLS.bind(ParserMessages.ComponentMapRegistry_DUPLICATE_PACKAGE,
                            new String[] { packageName, fDefaultComponentName, getCurrentComponentName() }));
                }

                fDefaultComponentName = getCurrentComponentName();
                return;
            }
            packageName = packageName.substring(0, packageName.length() - 1);
            prefix = true;
        }

        if (prefix) {
            if (fPackagePrefixToComponentMappings.containsKey(packageName)) {
                if (!(fPackagePrefixToComponentMappings.get(packageName).equals(getCurrentComponentName()))) {
                    throw new SAXException(NLS.bind(ParserMessages.ComponentMapRegistry_DUPLICATE_PACKAGE,
                            new String[] { packageName + ".*", fPackagePrefixToComponentMappings.get(packageName), //$NON-NLS-1$
                                    getCurrentComponentName() }));
                }
            } else {
                fPackagePrefixToComponentMappings.put(packageName, getCurrentComponentName());
            }
        } else {
            if (fPackageNameToComponentMappings.containsKey(packageName)) {
                if (!(fPackageNameToComponentMappings.get(packageName).equals(getCurrentComponentName()))) {
                    throw new SAXException(NLS.bind(ParserMessages.ComponentMapRegistry_DUPLICATE_PACKAGE,
                            new String[] { packageName, fPackageNameToComponentMappings.get(packageName),
                                    getCurrentComponentName() }));
                }
            } else {
                fPackageNameToComponentMappings.put(packageName, getCurrentComponentName());
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void endChildElement() {
        // intentionally empty
    }

    protected enum MappingContextType implements ContextType {
        /**
         * Parsing a <component_mapping> element
         */
        PARSING_COMPONENT_MAPPING,

        /**
         * Parsing a <component> element
         */
        PARSING_COMPONENT,

        /**
         * Parsing a <includePackage> element
         */
        PARSING_INCLUDE_PACKAGE
    }

    /**
     * The name of the root component mapping node.
     */
    public final static String TAG_COMPONENT_MAPPING = "component_mapping"; //$NON-NLS-1$

    /**
     * The name of the component node.
     */
    public final static String TAG_COMPONENT = "component"; //$NON-NLS-1$

    /**
     * The name of the "name" attribute.
     */
    public final static String TAG_NAME = "name"; //$NON-NLS-1$

    /**
     * The name of the "includePackage" node.
     */
    public final static String TAG_INCLUDE_PACKAGE = "includePackage"; //$NON-NLS-1$
}
