/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.ibm.team.build.internal.parser.data.JUnitParserErrorData;
import com.ibm.team.build.internal.parser.data.JUnitParserFailureData;
import com.ibm.team.build.internal.parser.data.JUnitParserTestCaseData;
import com.ibm.team.build.internal.parser.data.JUnitParserTestSuiteData;

/**
 * Class for parsing the junit xml log format.
 */
public class JUnitLogParser extends AbstractParserHandlerBase {

    /**
     * The list of JUnit test suites parsed from this JUnit log file.
     */
    private List<JUnitParserTestSuiteData> fTestSuites = new ArrayList<JUnitParserTestSuiteData>();

    /**
     * The JUnit test suite currently being parsed.
     */
    private JUnitParserTestSuiteData fCurrentTestSuite;

    /**
     * The JUnit test case currently being parsed.
     */
    private JUnitParserTestCaseData fCurrentTestCase;

    /**
     * The current failure data being parsed.
     */
    private JUnitParserFailureData fFailureData;

    /**
     * The current error data being parsed.
     */
    private JUnitParserErrorData fErrorData;

    /**
     * Constructs the parser with a file to parse.
     * 
     * @param logFile
     *            The file to parse.
     */
    public JUnitLogParser(File logFile) {
        super(logFile);
    }

    /**
     * Constructs the parser with a String to parse
     * 
     * @param logString
     *            The string to parse.
     */
    public JUnitLogParser(String logString) {
        super(logString);
    }

    /**
     * Retrieves the test suites that were contained in the file that was
     * parsed.
     * 
     * @return the test suites that were contained in the file that was parsed.
     */
    public JUnitParserTestSuiteData[] getTestSuites() {
        return (JUnitParserTestSuiteData[]) fTestSuites.toArray(new JUnitParserTestSuiteData[fTestSuites.size()]);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getContextForElement(String elementName) {

        if (elementName.equals(TAG_ERROR)) {
            return JUnitContextType.PARSING_ERROR;
        } else if (elementName.equals(TAG_FAILURE)) {
            return JUnitContextType.PARSING_FAILURE;
        } else if (elementName.equals(TAG_TESTCASE)) {
            return JUnitContextType.PARSING_TESTCASE;
        } else if (elementName.equals(TAG_TESTSUITES)) {
            return JUnitContextType.PARSING_TESTSUITES;
        } else if (elementName.equals(TAG_TESTSUITE)) {
            return JUnitContextType.PARSING_TESTSUITE;
        } else if (elementName.equals(TAG_PROPERTIES)) {
            return JUnitContextType.PARSING_PROPERTIES;
        } else if (elementName.equals(TAG_SYSTEM_OUT)) {
            return JUnitContextType.PARSING_SYSTEM_OUT;
        } else if (elementName.equals(TAG_SYSTEM_ERR)) {
            return JUnitContextType.PARSING_SYSTEM_ERR;
        }

        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getValidChildElementContext(ContextType childContext, ContextType parentContext) {

        if (parentContext instanceof JUnitContextType) {
            switch ((JUnitContextType) parentContext) {
                case PARSING_TESTSUITES:
                    if (childContext == JUnitContextType.PARSING_TESTSUITE) {
                        return childContext;
                    }
                    break;
                case PARSING_TESTSUITE:
                    switch ((JUnitContextType) childContext) {
                        case PARSING_ERROR:
                        case PARSING_TESTCASE:
                            return childContext;
                        case PARSING_PROPERTIES:
                        case PARSING_SYSTEM_OUT:
                        case PARSING_SYSTEM_ERR:
                            return getIgnoreContext();
                    }
                    break;
                case PARSING_TESTCASE:
                    switch ((JUnitContextType) childContext) {
                        case PARSING_ERROR:
                        case PARSING_FAILURE:
                            return childContext;
                    }
                    break;
            }
        }

        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected ContextType getValidRootElementContext(ContextType rootContext) throws SAXParseException {
        if (rootContext == JUnitContextType.PARSING_TESTSUITES || rootContext == JUnitContextType.PARSING_TESTSUITE) {
            return rootContext;
        }

        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void startChildElement(Attributes attributes) throws SAXException, SkipElementException {

        switch ((JUnitContextType) getCurrentContext()) {
            case PARSING_TESTSUITE:
                handleStartTestSuite(attributes);
                break;
            case PARSING_TESTCASE:
                handleStartTestcase(attributes);
                break;
            case PARSING_FAILURE:
                handleStartFailure(attributes);
                break;
            case PARSING_ERROR:
                handleStartError(attributes);
                break;
        }

    }

    private void handleStartTestSuite(Attributes attributes) throws SkipElementException {
        fCurrentTestSuite = null;
        String name = getRequiredAttribute(attributes, TAG_NAME, true);

        /*
         * All of these attributes are required, but none of them are strictly
         * necessary to parse the test suite, so specify default values of zero
         * for all of them in case they are missing so we can continue with a
         * best-case parse.
         */
        int errorCount = (int) getRequiredWholeNumberAttribute(attributes, TAG_ERRORS, true, false, 0);
        int failureCount = (int) getRequiredWholeNumberAttribute(attributes, TAG_FAILURES, true, false, 0);
        int testCount = (int) getRequiredWholeNumberAttribute(attributes, TAG_TESTS, true, false, 0);
        long timeTaken = (long) (getRequiredNumericAttribute(attributes, TAG_TIME, true, false, 0) * 1000);

        fCurrentTestSuite = new JUnitParserTestSuiteData(name, testCount, failureCount, errorCount, timeTaken);
        fTestSuites.add(fCurrentTestSuite);
    }

    private void handleStartTestcase(Attributes attributes) throws SkipElementException {
        fCurrentTestCase = null;

        String testcaseName = getRequiredAttribute(attributes, TAG_NAME, true);
        String testcaseClassName = getRequiredAttribute(attributes, TAG_CLASS_NAME, true);

        // must not be negative, but can be zero
        long timeTaken = (long) (getRequiredNumericAttribute(attributes, TAG_TIME, true, false, 0) * 1000);

        fCurrentTestCase = new JUnitParserTestCaseData(testcaseName, testcaseClassName, timeTaken);
    }

    private void handleStartFailure(Attributes attributes) throws SkipElementException {
        fFailureData = null;

        String type = getOptionalAttribute(attributes, TAG_TYPE);
        String message = getOptionalAttribute(attributes, TAG_MESSAGE);

        fFailureData = new JUnitParserFailureData(type, message);
    }

    private void handleStartError(Attributes attributes) throws SkipElementException {
        fErrorData = null;

        String type = getOptionalAttribute(attributes, TAG_TYPE);
        String message = getOptionalAttribute(attributes, TAG_MESSAGE);

        fErrorData = new JUnitParserErrorData(type, message);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void endChildElement() {
        switch ((JUnitContextType) getCurrentContext()) {
            case PARSING_TESTSUITE:
                if (fErrorData != null) {
                    fCurrentTestSuite.setErrorData(fErrorData);
                    fErrorData = null;
                }
                break;
            case PARSING_TESTCASE:
                if (fCurrentTestCase != null) {
                    if (fErrorData != null) {
                        fCurrentTestCase.setErrorData(fErrorData);
                    }
                    if (fFailureData != null) {
                        fCurrentTestCase.setFailureData(fFailureData);
                    }
                    fCurrentTestSuite.addTestCase(fCurrentTestCase);
                }

                fErrorData = null;
                fFailureData = null;
                break;
            case PARSING_FAILURE:
                if (getCurrentCharacterData() != null && getCurrentCharacterData().length() != 0) {
                    fFailureData.setStackTrace(getCurrentCharacterData().toString());
                }
                break;
            case PARSING_ERROR:
                if (getCurrentCharacterData() != null && getCurrentCharacterData().length() != 0) {
                    fErrorData.setStackTrace(getCurrentCharacterData().toString());
                }
                break;
        }
    }

    enum JUnitContextType implements ContextType {
        /**
         * Currently parsing a <testsuites> element
         */
        PARSING_TESTSUITES,

        /**
         * Currently parsing a <testsuite> element
         */
        PARSING_TESTSUITE,

        /**
         * Currently parsing a <properties> element
         */
        PARSING_PROPERTIES,

        /**
         * Currently parsing a <testcase> element
         */
        PARSING_TESTCASE,

        /**
         * Currently parsing a <failure> element
         */
        PARSING_FAILURE,

        /**
         * Currently parsing a <error> element
         */
        PARSING_ERROR,

        /**
         * Currently parsing a <system-out> element
         */
        PARSING_SYSTEM_OUT,

        /**
         * Currently parsing a <system-err> element
         */
        PARSING_SYSTEM_ERR
    }

    /**
     * The name of the test suites element.
     */
    public static final String TAG_TESTSUITES = "testsuites"; //$NON-NLS-1$

    /**
     * The name of the test suite element.
     */
    public static final String TAG_TESTSUITE = "testsuite"; //$NON-NLS-1$

    /**
     * The name of the test case element.
     */
    public static final String TAG_TESTCASE = "testcase"; //$NON-NLS-1$

    /**
     * The name of the properties element.
     */
    public static final String TAG_PROPERTIES = "properties"; //$NON-NLS-1$

    /**
     * The name of the system-out element.
     */
    public static final String TAG_SYSTEM_OUT = "system-out"; //$NON-NLS-1$

    /**
     * The name of the system-err element.
     */
    public static final String TAG_SYSTEM_ERR = "system-err"; //$NON-NLS-1$

    /**
     * The name of the failure element.
     */
    public static final String TAG_FAILURE = "failure"; //$NON-NLS-1$

    /**
     * The name of the error element.
     */
    public static final String TAG_ERROR = "error"; //$NON-NLS-1$

    /**
     * The name of the type attribute for the failure and error elements.
     */
    public static final String TAG_TYPE = "type"; //$NON-NLS-1$

    /**
     * The name of the message attribute for the failure and error elements.
     */
    public static final String TAG_MESSAGE = "message"; //$NON-NLS-1$

    /**
     * The name of the name attribute for the test suite and test case elements.
     */
    public static final String TAG_NAME = "name"; //$NON-NLS-1$

    /**
     * The name of the classname attribute for the test case element.
     */
    public static final String TAG_CLASS_NAME = "classname"; //$NON-NLS-1$

    /**
     * The name of the error count attribute for the test suite element.
     */
    public static final String TAG_ERRORS = "errors"; //$NON-NLS-1$

    /**
     * The name of the failure count attribute for the test suite element.
     */
    public static final String TAG_FAILURES = "failures"; //$NON-NLS-1$

    /**
     * The name of the test count attribute for the test suite element.
     */
    public static final String TAG_TESTS = "tests"; //$NON-NLS-1$

    /**
     * The name of the time taken attribute for the test suite and test case
     * elements.
     */
    public static final String TAG_TIME = "time"; //$NON-NLS-1$
}
