/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import com.ibm.team.build.internal.common.helper.FileHelper;

/**
 * Log parser for parsing a single JDT compile log. An example of what an entry
 * in the log looks like follows:
 * 
 * <pre>
 * ----------
 * 1. WARNING in /plugins/com.ibm.sample.hello/src/com/ibm/sample/hello/Hello.java
 * (at line 8)
 *    import com.ibm.sample.goodbye.Goodbye;
 *           &circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;&circ;
 * The import com.ibm.sample.goodbye.Goodbye is never used
 * ----------
 * </pre>
 */
public class JdtSimpleCompileLogParser {
    private static final String NEW_LINE = System.getProperty("line.separator"); //$NON-NLS-1$

    private int fErrorCount;
    private int fWarningCount;
    private String fErrors = ""; //$NON-NLS-1$
    private String fWarnings = ""; //$NON-NLS-1$
    private String fFilePath;

    /**
     * Create a parser on the given file.
     * 
     * @param filePath
     *            The absolute path to a JDT compiler output .bin.log file.
     */
    public JdtSimpleCompileLogParser(String filePath) {
        fFilePath = filePath;
    }

    /**
     * Parses the JDT compile log.
     * 
     * @throws IOException
     *             if the file to parse could not be open or read.
     */
    public void parse() throws IOException {
        String contents = FileHelper.fileToString(fFilePath);
        parseErrorsAndWarnings(contents);
    }

    /**
     * Retrieves the number of errors in the log.
     * 
     * @return The number of errors in the log.
     */
    public int getErrorCount() {
        return fErrorCount;
    }

    /**
     * Retreves a string containing the error text for all errors. If there are
     * no errors, returns empty string. Never returns <code>null</code>.
     * 
     * @return A string containing the error text for all errors. If there are
     *         no errors, returns empty string. Never returns <code>null</code>.
     */
    public String getErrors() {
        return fErrors;
    }

    /**
     * Retrieves the number of warnings in the log.
     * 
     * @return The number of warnings in the log.
     */
    public int getWarningCount() {
        return fWarningCount;
    }

    /**
     * Retreves a string containing the warning text for all warnings. If there
     * are no warnings, returns empty string. Never returns <code>null</code>.
     * 
     * @return A string containing the warning text for all warnings. If there
     *         are no warnings, returns empty string. Never returns
     *         <code>null</code>.
     */
    public String getWarnings() {
        return fWarnings;
    }

    protected void parseErrorsAndWarnings(String contents) throws IOException {
        BufferedReader reader = null;
        try {
            int warningCount = 0;
            int errorCount = 0;
            StringBuffer warnings = new StringBuffer();
            StringBuffer errors = new StringBuffer();

            reader = new BufferedReader(new StringReader(contents));

            String line = reader.readLine();
            while (line != null) {
                if (lineContains(line, ". WARNING in")) { //$NON-NLS-1$
                    // Parse warning lines. Read lines until we get a
                    // line with only ----------
                    warningCount++;
                    warnings.append(line);
                    warnings.append(NEW_LINE);
                    line = reader.readLine();
                    while ((line != null) && (!line.equals("----------"))) { //$NON-NLS-1$
                        warnings.append(line);
                        warnings.append(NEW_LINE);
                        line = reader.readLine();
                    }
                    if ((line != null) && (line.equals("----------"))) { //$NON-NLS-1$
                        warnings.append(line);
                        warnings.append(NEW_LINE);
                    }
                } else if (lineContains(line, ". ERROR in")) { //$NON-NLS-1$
                    // Parse error lines. Read lines until we get a
                    // line with only ----------
                    errorCount++;
                    errors.append(line);
                    errors.append(NEW_LINE);
                    line = reader.readLine();
                    while ((line != null) && (!line.equals("----------"))) { //$NON-NLS-1$
                        errors.append(line);
                        errors.append(NEW_LINE);
                        line = reader.readLine();
                    }
                    if ((line != null) && (line.equals("----------"))) { //$NON-NLS-1$
                        errors.append(line);
                        errors.append(NEW_LINE);
                    }
                }
                line = reader.readLine();
            }

            fErrorCount = errorCount;
            fWarningCount = warningCount;
            fErrors = errors.toString();
            fWarnings = warnings.toString();

        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException e) {
                // Intentionally empty...not a lot can be done if the reader
                // won't close.
            }
        }
    }

    /**
     * Return true if <code>line</code> contains <code>subString</code>.
     */
    private boolean lineContains(String line, String subString) {
        if (line.indexOf(subString) >= 0) {
            return true;
        }
        return false;
    }
}
