/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.ibm.team.build.internal.parser.data.JdtParserCompileData;
import com.ibm.team.build.internal.parser.data.JdtParserProblemData;
import com.ibm.team.build.internal.parser.data.JdtParserSourceData;
import com.ibm.team.build.internal.parser.data.JdtParserProblemData.Severity;

/**
 * This class implements a Sax parser to parse the XML log output of ECJ
 * (Eclipse Compiler for Java).
 */
public class JdtXmlCompileLogParser extends AbstractParserHandlerBase {

    /*
     * The JdtXmlCompileData object containing the information parsed from the
     * logfile
     */
    private JdtParserCompileData fCompileData;

    /*
     * The <source> element currently being parsed
     */
    private JdtParserSourceData fCurrentSource;

    /*
     * The <problem> element currently being parsed
     */
    private JdtParserProblemData fCurrentProblem;

    /*
     * Flags used to verify that required subelements are actually found
     */
    private boolean fFoundMessage;
    private boolean fFoundSourceContext;

    /**
     * Create a JdtXmlCompileLogParser to parse the specified xml log file.
     * 
     * @param logFile
     *            The xml log file.
     */
    public JdtXmlCompileLogParser(File logFile) {
        super(logFile);
    }

    /**
     * Provides access to the {@link JdtParserCompileData} object containing the
     * data parsed from the logfile.
     * 
     * @return The {@link JdtParserCompileData} object containing the parsed
     *         data.
     */
    public JdtParserCompileData getParsedData() {
        return fCompileData;
    }

    /*
     * Parse the systemId to determine the dtd version of the log file. We only
     * support dtd versions 32.002 and later. The systemId is of the following
     * format. "http://www.eclipse.org/jdt/core/compiler_32_002.dtd"
     */
    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        if (systemId == null) {
            return;
        }

        /*
         * Match "compiler_XXX_XX.dtd", where "X" is any number
         */
        String compilerPrefix = "compiler_"; //$NON-NLS-1$
        Pattern dtdNamePattern = Pattern.compile(compilerPrefix + "[0-9][0-9]_[0-9][0-9][0-9]\\.dtd"); //$NON-NLS-1$
        Matcher dtdNameMatcher = dtdNamePattern.matcher(systemId);

        if (!dtdNameMatcher.find()) {
            return;
        }

        String dtdName = dtdNameMatcher.group();
        String dtdVersion = dtdName.substring(compilerPrefix.length(), compilerPrefix.length() + 6);

        int majorVersion = Integer.valueOf(dtdVersion.substring(0, 2));
        int minorVersion = Integer.valueOf(dtdVersion.substring(3, 6));

        /*
         * Dtd version most be greater than 32.001
         */
        if (majorVersion < 32 || (majorVersion == 32 && minorVersion == 1)) {
            throw new SAXException(ParserMessages.JdtXmlCompileLogParser_UNSUPPORTED_COMPILER_VERSION);
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    protected ContextType getValidRootElementContext(ContextType rootContext) {

        if (rootContext != JdtXmlContextType.PARSING_COMPILER) {
            return null;
        }

        return rootContext;
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    protected ContextType getValidChildElementContext(ContextType childContext, ContextType parentContext) {

        switch ((JdtXmlContextType) parentContext) {
            case PARSING_COMPILER:
                if (childContext == JdtXmlContextType.PARSING_COMMAND_LINE
                        || childContext == JdtXmlContextType.PARSING_OPTIONS
                        || childContext == JdtXmlContextType.PARSING_CLASSPATHS) {
                    return getIgnoreContext();
                } else if (childContext == JdtXmlContextType.PARSING_SOURCES
                        || childContext == JdtXmlContextType.PARSING_STATS) {
                    return childContext;
                }
                break;
            case PARSING_SOURCES:
                if (childContext == JdtXmlContextType.PARSING_SOURCE) {
                    return childContext;
                }
                break;
            case PARSING_SOURCE:
                if (childContext == JdtXmlContextType.PARSING_PROBLEMS
                        || childContext == JdtXmlContextType.PARSING_CLASSFILE) {
                    return childContext;
                }
                break;
            case PARSING_PROBLEMS:
                if (childContext == JdtXmlContextType.PARSING_PROBLEM) {
                    return childContext;
                }
                break;
            case PARSING_PROBLEM:
                if (childContext == JdtXmlContextType.PARSING_MESSAGE
                        || childContext == JdtXmlContextType.PARSING_SOURCE_CONTEXT) {
                    return childContext;
                } else if (childContext == JdtXmlContextType.PARSING_ARGUMENTS) {
                    return getIgnoreContext();
                }
                break;
            case PARSING_STATS:
                if (childContext == JdtXmlContextType.PARSING_TIME
                        || childContext == JdtXmlContextType.PARSING_NUMBER_OF_LINES
                        || childContext == JdtXmlContextType.PARSING_NUMBER_OF_CLASSFILES
                        || childContext == JdtXmlContextType.PARSING_PROBLEM_SUMMARY) {
                    return childContext;
                }
                break;
        }

        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    protected void startChildElement(Attributes attributes) throws SAXException, SkipElementException {

        Long value;

        switch ((JdtXmlContextType) getCurrentContext()) {
            case PARSING_COMPILER:
                fCompileData = new JdtParserCompileData();
                break;
            case PARSING_SOURCE:
                handleStartSource(attributes);
                break;
            case PARSING_PROBLEM:
                handleStartProblem(attributes);
                break;
            case PARSING_SOURCE_CONTEXT:
                handleStartSourceContext(attributes);
                break;
            case PARSING_CLASSFILE:
                fCurrentSource.addClassfileName(getRequiredAttribute(attributes, TAG_PATH, true));
                break;
            case PARSING_MESSAGE:
                fCurrentProblem.setMessage(getRequiredAttribute(attributes, TAG_VALUE, false));
                fFoundMessage = true;
                break;
            case PARSING_TIME:
                fCompileData.setCompileTime(getOptionalWholeNumberAttribute(attributes, TAG_VALUE, true, true));
                break;
            case PARSING_NUMBER_OF_LINES:
                value = getOptionalWholeNumberAttribute(attributes, TAG_VALUE, true, false);
                if (value != null) {
                    fCompileData.setLinesCompiled(value.intValue());
                }
                break;
            case PARSING_NUMBER_OF_CLASSFILES:
                value = getOptionalWholeNumberAttribute(attributes, TAG_VALUE, true, false);
                if (value != null) {
                    fCompileData.setNumberOfClassfiles(value.intValue());
                }
                break;
        }
    }

    /**
     * Handle the <source> element - example:
     * 
     * <pre>
     *       &lt;source output=&quot;\out&quot; package=&quot;com\foo&quot; path=&quot;C:\my\src\com\foo\Bar.java&quot;&gt; 
     * </pre>
     * 
     * For now we ignore the output attribute.
     */
    private void handleStartSource(Attributes attributes) throws SkipElementException {
        String path = getRequiredAttribute(attributes, TAG_PATH, true);

        /*
         * It's not clear if the xml output will always use either '\' or '/'
         * and whether or not it's platform dependent - so we just check for
         * both.
         */
        String className = path.substring(path.lastIndexOf('/') + 1);
        className = className.substring(path.lastIndexOf('\\') + 1);

        String packageName = getOptionalAttribute(attributes, TAG_PACKAGE, true);

        /*
         * If package name is not set, then one of two cases applies: either the
         * source class is in the default package, in which case the "output"
         * attribute will be set, or the source file was completely empty in
         * which case the output attribute will NOT be set. So we need to check
         * the "output" attribute to determine which case we are seeing.
         */
        if (packageName == null) {
            String output = getOptionalAttribute(attributes, TAG_OUTPUT, true);

            /*
             * If output is not set, the source file was empty - so ignore this
             * element.
             */
            if (output == null) {
                throw new SkipElementException();
            }

            /*
             * The source file was in the default package, so set the package
             * name to "".
             */
            packageName = ""; //$NON-NLS-1$
        }

        /*
         * The package name may be in the form "com/ibm/team/...". Replace '/'
         * with '.'.
         */
        packageName = packageName.replace('/', '.');
        packageName = packageName.replace('\\', '.');

        fCurrentSource = new JdtParserSourceData(className, packageName, path);
    }

    /**
     * Handle the <problem> element - examples:
     * 
     * <pre>
     *                &lt;problem charEnd=&quot;90&quot; charStart=&quot;82&quot; id=&quot;JavadocInvalidParamName&quot; line=&quot;7&quot; severity=&quot;WARNING&quot;&gt;
     *                &lt;problem charEnd=&quot;52&quot; charStart=&quot;49&quot; id=&quot;UndefinedMethod&quot; line=&quot;5&quot; severity=&quot;ERROR&quot;&gt;
     * </pre>
     */
    private void handleStartProblem(Attributes attributes) throws SkipElementException {

        int charStart = (int) getRequiredWholeNumberAttribute(attributes, TAG_CHAR_START, true, false, 0);
        int charEnd = (int) getRequiredWholeNumberAttribute(attributes, TAG_CHAR_END, true, true, 0);
        int lineNumber = (int) getRequiredWholeNumberAttribute(attributes, TAG_LINE, true, true, 0);

        String id = getRequiredAttribute(attributes, TAG_ID, true);
        String severityValue = getRequiredAttribute(attributes, TAG_SEVERITY, VALID_SEVERITY_ATTRIBUTE_VALUES, true);
        Severity severity = severityValue.equals(ERROR) ? Severity.ERROR : Severity.WARNING;

        fCurrentProblem = new JdtParserProblemData(charStart, charEnd, lineNumber, id, severity);

        fFoundMessage = false;
        fFoundSourceContext = false;
    }

    /**
     * Handle the <source_context> element - example:
     * 
     * <pre>
     *        &lt;source_context sourceEnd=&quot;17&quot; sourceStart=&quot;13&quot; value=&quot;private void foo() {&quot;/&gt;
     * </pre>
     */
    private void handleStartSourceContext(Attributes attributes) throws SkipElementException {

        int sourceStart = (int) getRequiredWholeNumberAttribute(attributes, TAG_SOURCE_START, true, false, 0);
        int sourceEnd = (int) getRequiredWholeNumberAttribute(attributes, TAG_SOURCE_END, true, true, 0);
        String value = getRequiredAttribute(attributes, TAG_VALUE, true);

        fCurrentProblem.setSource(value);
        fCurrentProblem.setSourceStart(sourceStart);
        fCurrentProblem.setSourceEnd(sourceEnd);

        fFoundSourceContext = true;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void endChildElement() throws SAXException, SkipElementException {
        switch ((JdtXmlContextType) getCurrentContext()) {
            case PARSING_PROBLEM:
                verifySubElement(TAG_MESSAGE, fFoundMessage, true);
                verifySubElement(TAG_SOURCE_CONTEXT, fFoundSourceContext, false);
                fCurrentSource.addProblem(fCurrentProblem);
                break;
            case PARSING_SOURCE:
                fCompileData.addSource(fCurrentSource);
                break;
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented.
     */
    @Override
    protected ContextType getContextForElement(String elementName) {
        if (elementName.equals(TAG_COMPILER)) {
            return JdtXmlContextType.PARSING_COMPILER;

        } else if (elementName.equals(TAG_COMMAND_LINE)) {
            return JdtXmlContextType.PARSING_COMMAND_LINE;

        } else if (elementName.equals(TAG_CLASSPATHS)) {
            return JdtXmlContextType.PARSING_CLASSPATHS;

        } else if (elementName.equals(TAG_SOURCES)) {
            return JdtXmlContextType.PARSING_SOURCES;

        } else if (elementName.equals(TAG_SOURCE)) {
            return JdtXmlContextType.PARSING_SOURCE;

        } else if (elementName.equals(TAG_PROBLEMS)) {
            return JdtXmlContextType.PARSING_PROBLEMS;

        } else if (elementName.equals(TAG_PROBLEM)) {
            return JdtXmlContextType.PARSING_PROBLEM;

        } else if (elementName.equals(TAG_STATS)) {
            return JdtXmlContextType.PARSING_STATS;

        } else if (elementName.equals(TAG_ARGUMENTS)) {
            return JdtXmlContextType.PARSING_ARGUMENTS;

        } else if (elementName.equals(TAG_CLASSFILE)) {
            return JdtXmlContextType.PARSING_CLASSFILE;

        } else if (elementName.equals(TAG_MESSAGE)) {
            return JdtXmlContextType.PARSING_MESSAGE;

        } else if (elementName.equals(TAG_TIME)) {
            return JdtXmlContextType.PARSING_TIME;

        } else if (elementName.equals(TAG_NUMBER_OF_LINES)) {
            return JdtXmlContextType.PARSING_NUMBER_OF_LINES;

        } else if (elementName.equals(TAG_PROBLEM_SUMMARY)) {
            return JdtXmlContextType.PARSING_PROBLEM_SUMMARY;

        } else if (elementName.equals(TAG_NUMBER_OF_CLASSFILES)) {
            return JdtXmlContextType.PARSING_NUMBER_OF_CLASSFILES;

        } else if (elementName.equals(TAG_OPTIONS)) {
            return JdtXmlContextType.PARSING_OPTIONS;

        } else if (elementName.equals(TAG_SOURCE_CONTEXT)) {
            return JdtXmlContextType.PARSING_SOURCE_CONTEXT;
        }

        return null;
    }

    /*
     * Enum denoting values for the current element context
     */
    enum JdtXmlContextType implements ContextType {
        /**
         * Currently parsing the <compiler> element
         */
        PARSING_COMPILER,

        /**
         * Currently parsing the <command-line> element
         */
        PARSING_COMMAND_LINE,

        /**
         * Currently parsing the <options> element
         */
        PARSING_OPTIONS,

        /**
         * Currently parsing the <option> element
         */
        PARSING_OPTION,

        /**
         * Currently parsing the <classpaths> element
         */
        PARSING_CLASSPATHS,

        /**
         * Currently parsing the <classpath> element
         */
        PARSING_CLASSPATH,

        /**
         * Currently parsing the <sources> element
         */
        PARSING_SOURCES,

        /**
         * Currently parsing the <source> element
         */
        PARSING_SOURCE,

        /**
         * Currently parsing the <problems> element
         */
        PARSING_PROBLEMS,

        /**
         * Currently parsing the <problem> element
         */
        PARSING_PROBLEM,

        /**
         * Currently parsing the <stats> element
         */
        PARSING_STATS,

        /**
         * Currently parsing the <arguments> element
         */
        PARSING_ARGUMENTS,

        /**
         * Currently parsing the <argument> element
         */
        PARSING_ARGUMENT,

        /**
         * /** Currently parsing the <classfile> element
         */
        PARSING_CLASSFILE,

        /**
         * Currently parsing the <message> element
         */
        PARSING_MESSAGE,

        /**
         * Currently parsing the <time> element
         */
        PARSING_TIME,

        /**
         * Currently parsing the <number_of_lines> element
         */
        PARSING_NUMBER_OF_LINES,

        /**
         * Currently parsing the <problem_summary> element
         */
        PARSING_PROBLEM_SUMMARY,

        /**
         * Currently parsing the <number_of_classfiles> element
         */
        PARSING_NUMBER_OF_CLASSFILES,

        /**
         * Currently parsing the <source_context> element
         */
        PARSING_SOURCE_CONTEXT
    };

    /*
     * The attribute and element name constants
     */
    /*
     * I've commented out the tags we aren't currently using
     */
    // protected final static String TAG_NAME = "name"; //$NON-NLS-1$
    // protected final static String TAG_COPYRIGHT = "copyright"; //$NON-NLS-1$
    // protected final static String TAG_VERSION = "version"; //$NON-NLS-1$
    // protected final static String TAG_OPTION = "option"; //$NON-NLS-1$
    // protected final static String TAG_KEY = "key"; //$NON-NLS-1$
    // protected final static String TAG_ARGUMENT = "argument"; //$NON-NLS-1$
    // protected final static String TAG_CLASSPATH = "classpath"; //$NON-NLS-1$
    protected final static String TAG_COMPILER = "compiler"; //$NON-NLS-1$ 
    protected final static String TAG_OPTIONS = "options"; //$NON-NLS-1$
    protected final static String TAG_COMMAND_LINE = "command_line"; //$NON-NLS-1$ 
    protected final static String TAG_ARGUMENTS = "arguments"; //$NON-NLS-1$ 
    protected final static String TAG_CLASSPATHS = "classpaths"; //$NON-NLS-1$ 
    protected final static String TAG_CLASSFILE = "classfile"; //$NON-NLS-1$ 
    protected final static String TAG_ID = "id"; //$NON-NLS-1$ 
    protected final static String TAG_PROBLEMS = "problems"; //$NON-NLS-1$
    protected final static String TAG_PROBLEM = "problem"; //$NON-NLS-1$
    protected final static String TAG_CHAR_START = "charStart"; //$NON-NLS-1$
    protected final static String TAG_CHAR_END = "charEnd"; //$NON-NLS-1$ 
    protected final static String TAG_SOURCES = "sources"; //$NON-NLS-1$ 
    protected final static String TAG_SOURCE = "source"; //$NON-NLS-1$
    protected final static String TAG_OUTPUT = "output"; //$NON-NLS-1$
    protected final static String TAG_PACKAGE = "package"; //$NON-NLS-1$
    protected final static String TAG_PATH = "path"; //$NON-NLS-1$ 
    protected final static String TAG_SEVERITY = "severity"; //$NON-NLS-1$ 
    protected final static String TAG_MESSAGE = "message"; //$NON-NLS-1$ 
    protected final static String TAG_VALUE = "value"; //$NON-NLS-1$ 
    protected final static String TAG_LINE = "line"; //$NON-NLS-1$ 
    protected final static String TAG_STATS = "stats"; //$NON-NLS-1$ 
    protected final static String TAG_TIME = "time"; //$NON-NLS-1$ 
    protected final static String TAG_SOURCE_CONTEXT = "source_context"; //$NON-NLS-1$
    protected final static String TAG_NUMBER_OF_LINES = "number_of_lines"; //$NON-NLS-1$
    protected final static String TAG_PROBLEM_SUMMARY = "problem_summary"; //$NON-NLS-1$
    protected final static String TAG_NUMBER_OF_CLASSFILES = "number_of_classfiles"; //$NON-NLS-1$
    protected final static String TAG_SOURCE_START = "sourceStart"; //$NON-NLS-1$
    protected final static String TAG_SOURCE_END = "sourceEnd"; //$NON-NLS-1$

    protected final static String ERROR = "ERROR"; //$NON-NLS-1$
    protected final static String WARNING = "WARNING"; //$NON-NLS-1$
    protected final static String[] VALID_SEVERITY_ATTRIBUTE_VALUES = new String[] { WARNING, ERROR };

}
