/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import com.ibm.team.build.internal.common.helper.FileHelper;

/**
 * Log parser for parsing a folder containing MSBuild compile log files. This
 * log is not a standard log but generated by a logger written by us An example
 * of what an entry in the log looks like follows: --- Project Start ---
 * SolutionName:AutoTests ProjectName:AutoWorld --- Errors Start ---
 * Program.cs(10,28): "; expected" --- Errors End --- --- Warnings Start ---
 * Program.cs(10,18): "The field 'HelloWorld.Program.i' is never used" ---
 * Warnings End --- --- Project End ---
 */
public class MSBuildSimpleCompileLogParser {

    /**
     * Holds the compile data of the project
     */
    public class ProjectData {
        private int fErrorCount;
        private int fWarningCount;
        private String fErrors = ""; //$NON-NLS-1$
        private String fWarnings = ""; //$NON-NLS-1$
        private String fProjectName = ""; //$NON-NLS-1$;
        private String fSolutionName = ""; //$NON-NLS-1$;

        /**
         * @return returns the number of errors in the project
         */
        public int getErrorCount() {
            return fErrorCount;
        }

        private void setErrorCount(int errorCount) {
            fErrorCount = errorCount;
        }

        /**
         * @return returns the number of warnings in the project
         */
        public int getWarningCount() {
            return fWarningCount;
        }

        private void setWarningCount(int warningCount) {
            fWarningCount = warningCount;
        }

        /**
         * @return returns the errors string in the project
         */
        public String getErrors() {
            return fErrors;
        }

        private void setErrors(String errors) {
            fErrors = errors;
        }

        /**
         * @return returns the warnings string in the project
         */
        public String getWarnings() {
            return fWarnings;
        }

        private void setWarnings(String warnings) {
            fWarnings = warnings;
        }

        /**
         * @return returns the project name
         */
        public String getProjectName() {
            return fProjectName;
        }

        private void setProjectName(String projectName) {
            fProjectName = projectName;
        }

        /**
         * @return returns the solution name
         */
        public String getSolutionName() {
            return fSolutionName;
        }

        private void setSolutionName(String solutionName) {
            fSolutionName = solutionName;
        }

    }

    private static final String NEW_LINE = System.getProperty("line.separator"); //$NON-NLS-1$

    private static final String PROJECT_START = "--- Project Start ---"; //$NON-NLS-1$
    private static final String PROJECT_END = "--- Project End ---"; //$NON-NLS-1$

    private static final String ERRORS_START = "--- Errors Start ---"; //$NON-NLS-1$
    private static final String ERRORS_END = "--- Errors End ---"; //$NON-NLS-1$

    private static final String WARNINGS_START = "--- Warnings Start ---"; //$NON-NLS-1$
    private static final String WARNINGS_END = "--- Warnings End ---"; //$NON-NLS-1$

    private static final String SOLUTION_NAME = "SolutionName:"; //$NON-NLS-1$
    private static final String PROJECT_NAME = "ProjectName:"; //$NON-NLS-1$

    private String fLogDirectory;

    private List<ProjectData> fProjectsData;

    /**
     * Create a parser on the given a .
     * 
     * @param logDirectory
     *            The absolute path to a MSBuild compiler output directory.
     */
    public MSBuildSimpleCompileLogParser(String logDirectory) {
        fLogDirectory = logDirectory;
        fProjectsData = new ArrayList<ProjectData>();
    }

    /**
     * @return the projectsData, which holds the compile data of each project
     */
    public List<ProjectData> getProjectsData() {
        return fProjectsData;
    }

    /**
     * Parses the MSBuild compile log.
     * 
     * @throws IOException
     *             if their is a parser error.
     */
    public void parse() throws IOException {
        String logFiles[] = getLogFiles(fLogDirectory);
        if (logFiles != null && logFiles.length >= 1) {
            for (int index = 0; index < logFiles.length; index++) {
                String contents = FileHelper.fileToString(new File(fLogDirectory, logFiles[index]));
                parseErrorsAndWarnings(contents);
            }
        }
    }

    protected void parseErrorsAndWarnings(String contents) throws IOException {
        BufferedReader reader = null;
        String projectName = null;
        String solutionName = null;
        StringBuilder errors = new StringBuilder();
        StringBuilder warnings = new StringBuilder();
        int errorCount = 0;
        int warningCount = 0;

        try {
            reader = new BufferedReader(new StringReader(contents));
            String line = reader.readLine();
            // Checkpoint-1
            if (!PROJECT_START.equals(line))
                return;
            // Checkpoint-2
            line = reader.readLine();
            if (line == null || !line.startsWith(SOLUTION_NAME))
                return;
            solutionName = line.substring(SOLUTION_NAME.length());
            // Checkpoint-3
            line = reader.readLine();
            if (line == null || !line.startsWith(PROJECT_NAME))
                return;
            projectName = line.substring(PROJECT_NAME.length());

            line = reader.readLine();
            while (line != null && !line.equals(PROJECT_END)) {
                if (line.equals(ERRORS_START)) {
                    line = reader.readLine();
                    while (line != null && !line.equals(ERRORS_END)) {
                        errorCount++;
                        errors.append(line);
                        errors.append(NEW_LINE);
                        line = reader.readLine();
                    }
                } else if (line.equals(WARNINGS_START)) {
                    line = reader.readLine();
                    while (line != null && !line.equals(WARNINGS_END)) {
                        warningCount++;
                        warnings.append(line);
                        warnings.append(NEW_LINE);
                        line = reader.readLine();
                    }
                }
                line = reader.readLine();
            }

            ProjectData prjData = new ProjectData();
            prjData.setProjectName(projectName);
            prjData.setSolutionName(solutionName);
            prjData.setErrorCount(errorCount);
            prjData.setErrors(errors.toString());
            prjData.setWarningCount(warningCount);
            prjData.setWarnings(warnings.toString());
            fProjectsData.add(prjData);

        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException e) {
                // Intentionally empty...not a lot can be done if the reader
                // won't close.
            }
        }

    }

    private String[] getLogFiles(String logDirectory) {
        if (logDirectory != null) {
            File lDir = new File(logDirectory);
            if (lDir.exists() && lDir.isDirectory()) {
                FilenameFilter filter = new FilenameFilter() {
                    public boolean accept(File dir, String name) {
                        return name.endsWith(".jbemslog"); //$NON-NLS-1$
                    }
                };
                return lDir.list(filter);
            }
        }
        return null;
    }

}
