/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * Helper class to store the key information about the test cases in a JUnit log
 * being parsed.
 */
public class JUnitParserTestCaseData {

    /**
     * The name of this testcase
     */
    private String fTestCaseName;

    /**
     * The fully qualified classname of the class containing this testcase
     */
    private String fTestClassName;

    /**
     * The time in milliseconds that this testcase took to run
     */
    private long fTimeTaken;

    /**
     * Ordinal used to sort test case by order of execution.
     */
    private int fOrdinal;

    /**
     * The data describing the test case failure.
     */
    private JUnitParserFailureData fFailureData;

    /**
     * The data describing the test case error.
     */
    private JUnitParserErrorData fErrorData;

    /**
     * Constructs a parser test case data instance with the attributes of a test
     * case element.
     * 
     * @param testName
     *            The name of this JUnit test case.
     * @param testClassName
     *            The name of the JUnit test class containing this JUnit test
     *            case.
     * @param timeTaken
     *            The time taken in milliseconds to execute this JUnit test
     *            case.
     * @throws IllegalArgumentException
     *             If the <code>attributes<code> is <code>null</code>.
     */
    public JUnitParserTestCaseData(String testName, String testClassName, long timeTaken)
            throws IllegalArgumentException {
        ValidationHelper.validateNotEmpty("testName", testName); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("testClassName", testClassName); //$NON-NLS-1$
        fTestCaseName = testName;
        fTestClassName = testClassName;
        fTimeTaken = timeTaken;
    }

    /**
     * Retrives the name of the test case.
     * 
     * @return The name of the test case.
     */
    public String getName() {
        return fTestCaseName;
    }

    /**
     * Retrieves the name of the containing test class.
     * 
     * @return The name of the containing test class.
     */
    public String getTestClassName() {
        return fTestClassName;
    }

    /**
     * Retrieves the time taken to execute the test case.
     * 
     * @return The time taken to execute the test case.
     */
    public long getTimeTaken() {
        return fTimeTaken;
    }

    /**
     * Retrieves the ordinal used to sort by order of execution.
     * 
     * @return The ordinal used to sort by order of execution.
     */
    public int getOrdinal() {
        return fOrdinal;
    }

    /**
     * Sets the ordinal used to sort by order of execution.
     * 
     * @param ordinal
     *            The ordinal used to sort by order of execution.
     */
    public void setOrdinal(int ordinal) {
        fOrdinal = ordinal;
    }

    /**
     * Retrieves the data describing the test case failure.
     * 
     * @return The data describing the test case failure.
     */
    public JUnitParserFailureData getFailureData() {
        return fFailureData;
    }

    /**
     * Sets the data describing the test case failure.
     * 
     * @param failureData
     *            The data describing the test case failure.
     */
    public void setFailureData(JUnitParserFailureData failureData) {
        fFailureData = failureData;

    }

    /**
     * Determines if the test case is a failure
     * 
     * @return <code>true</code> if the test case is a failure.
     */
    public boolean isFailure() {
        return getFailureData() != null;
    }

    /**
     * Retrieves the data describing the test case error.
     * 
     * @return The data describing the test case error.
     */
    public JUnitParserErrorData getErrorData() {
        return fErrorData;
    }

    /**
     * Sets the data describing the test case error.
     * 
     * @param errorData
     *            The data describing the test case error.
     */
    public void setErrorData(JUnitParserErrorData errorData) {
        fErrorData = errorData;

    }

    /**
     * Determines if the test case ended in error.
     * 
     * @return <code>true</code> if the test case ended in error
     */
    public boolean isError() {
        return getErrorData() != null;
    }

}
