/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

import java.util.LinkedList;
import java.util.List;

import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * Helper class to store information about test cases from a single JUnit test
 * class. Note that JUnit recognizes only the concepts of test suites and test
 * cases. This level of granularity is introduced here simply to make the
 * organization of large, multi-class JUnit test suites more efficient and
 * easier to show in the UI.
 */
public class JUnitParserTestClassData {

    /**
     * The fully qualified classname of the JUnit test class
     */
    private String fTestClassName;

    /**
     * The number of tests contained in this JUnit test class
     */
    private int fTestCount = 0;

    /**
     * The number of errors that occurred in the test cases contained in this
     * JUnit test class
     */
    private int fErrorCount = 0;

    /**
     * The number of failures that occurred in the test cases contained in this
     * JUnit test class
     */
    private int fFailureCount = 0;

    /**
     * The time in milliseconds that the test cases contained in this JUnit test
     * class took to run
     */
    private long fTimeTaken;

    /**
     * Ordinal used to sort test class by order of execution.
     */
    private int fOrdinal;

    /**
     * The <code>JUnitParserTestCaseData</code> objects representing the JUnit
     * test cases contained in this JUnit test class
     */
    private List<JUnitParserTestCaseData> fTestCases = new LinkedList<JUnitParserTestCaseData>();

    /**
     * Create a <code>JUnitParserTestClassData</code> object to represent the
     * JUnit test class with the specified class name.
     * 
     * @param className
     *            The name of the JUnit test class
     * @param ordinal
     *            The ordinal used to sort by order of execution.
     */
    public JUnitParserTestClassData(String className, int ordinal) {
        ValidationHelper.validateNotEmpty("className", className); //$NON-NLS-1$
        fTestClassName = className;
        fOrdinal = ordinal;
    }

    /**
     * Add the specified <code>JUnitParserTestCaseData</code> object to the
     * list of test cases contained in this JUnit test class
     * 
     * @param testCase
     *            The test case to be added.
     */
    public void addTestCase(JUnitParserTestCaseData testCase) {
        ValidationHelper.validateNotNull("testCase", testCase); //$NON-NLS-1$
        fTestCases.add(testCase);
        testCase.setOrdinal(fTestCases.size());
        fTestCount++;
        if (testCase.isError()) {
            fErrorCount++;
        }
        if (testCase.isFailure()) {
            fFailureCount++;
        }
        fTimeTaken += testCase.getTimeTaken();
    }

    /**
     * Return the fully qualified class name of this JUnit test class
     * 
     * @return The fully qualified class name of this JUnit test class.
     */
    public String getTestClassName() {
        return fTestClassName;
    }

    /**
     * Retrieves the number of tests contained in this JUnit test class.
     * 
     * @return The number of tests contained in this JUnit test class.
     */
    public int getTestCount() {
        return fTestCount;
    }

    /**
     * Retrieves the number of tests contained in this JUnit test class that
     * failed.
     * 
     * @return The number of tests contained in this JUnit test class that
     *         failed.
     */
    public int getFailureCount() {
        return fFailureCount;
    }

    /**
     * Retrieves the number of tests contained in this JUnit test class that
     * resulted in an error.
     * 
     * @return The number of tests contained in this JUnit test class that
     *         resulted in an error.
     */
    public int getErrorCount() {
        return fErrorCount;
    }

    /**
     * Retrieves the amount of time it took the tests contained in this JUnit
     * test class to execute.
     * 
     * @return The amount of time it took the tests contained in this JUnit test
     *         class to execute.
     */
    public long getTimeTaken() {
        return fTimeTaken;
    }

    /**
     * Retrieves the ordinal used to sort by order of execution.
     * 
     * @return The ordinal used to sort by order of execution.
     */
    public int getOrdinal() {
        return fOrdinal;
    }

    /**
     * Return the JUnit test cases contained in this JUnit test class
     * 
     * @return An array of <code>JUnitParserTestCaseData</code> objects
     *         representing the JUnit test cases contained in this JUnit test
     *         class.
     */
    public JUnitParserTestCaseData[] getTestCases() {
        return fTestCases.toArray(new JUnitParserTestCaseData[fTestCases.size()]);
    }
}
