/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

import java.util.Hashtable;

import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * Helper class to store the key information about the test suites in a JUnit
 * log being parsed.
 */
public class JUnitParserTestSuiteData {

    /**
     * The name of this testsuite
     */
    private final String fTestSuiteName;

    /**
     * The number of tests contained in this testsuite. Note that this number is
     * always set explicitly in the constructor and is not updated dynamically
     * as test cases are added. This reflects the fact that this class is
     * instantiated from a JUnit results XML file where the test count is
     * explicitly stated.
     */
    private final int fTestCount;

    /**
     * The number of errors that occurred in this testsuite Note that this
     * number is always set explicitly in the constructor and is not updated
     * dynamically as test cases are added. This reflects the fact that this
     * class is instantiated from a JUnit results XML file where the error count
     * is explicitly stated.
     */
    private final int fErrorCount;

    /**
     * The number of failures that occurred in this testsuite Note that this
     * number is always set explicitly in the constructor and is not updated
     * dynamically as test cases are added. This reflects the fact that this
     * class is instantiated from a JUnit results XML file where the failure
     * count is explicitly stated.
     */
    private final int fFailureCount;

    /**
     * The time in milliseconds that this testsuite took to run Note that this
     * number is always set explicitly in the constructor and is not updated
     * dynamically as test cases are added. This reflects the fact that this
     * class is instantiated from a JUnit results XML file where the time taken
     * is explicitly stated.
     */
    private final long fTimeTaken;

    /**
     * The data describing any test suite errors.
     */
    private JUnitParserErrorData fErrorData;

    /**
     * The test classes contained in the test suite.
     */
    private Hashtable<String, JUnitParserTestClassData> fTestClasses = new Hashtable<String, JUnitParserTestClassData>();

    /**
     * Constructs a parser test case data instance with the attributes of a test
     * case element.
     * 
     * @param testSuiteName
     *            The name of this JUnit test suite.
     * @param testCount
     *            The number of JUnit tests contained in this JUnit test suite.
     * @param failureCount
     *            The number of JUnit tests that failed in this JUnit test
     *            suite.
     * @param errorCount
     *            The number of JUnit tests that ended in error in this JUnit
     *            test suite.
     * @param timeTaken
     *            The time taken in milliseconds to execute this JUnit test
     *            suite.
     */
    public JUnitParserTestSuiteData(String testSuiteName, int testCount, int failureCount, int errorCount,
            long timeTaken) {
        fTestSuiteName = testSuiteName;
        fTestCount = testCount;
        fFailureCount = failureCount;
        fErrorCount = errorCount;
        fTimeTaken = timeTaken;
    }

    /**
     * Adds a test case to the test suite.
     * 
     * @param testCase
     *            The test case to add to the test suite.
     * @throws IllegalArgumentException
     *             If <code>testCase<code> is <code>null</code>.
     */
    public void addTestCase(JUnitParserTestCaseData testCase) {
        ValidationHelper.validateNotNull("testCase", testCase); //$NON-NLS-1$
        String className = testCase.getTestClassName();
        JUnitParserTestClassData testClass = fTestClasses.get(className);
        if (testClass == null) {
            testClass = new JUnitParserTestClassData(className, fTestClasses.size() + 1);
            fTestClasses.put(className, testClass);
        }
        testClass.addTestCase(testCase);
    }

    /**
     * Retrieves the test classes that are contained in the test suite.
     * 
     * @return The test classes that are contained in the test suite.
     */
    public JUnitParserTestClassData[] getTestClasses() {
        return fTestClasses.values().toArray(new JUnitParserTestClassData[fTestClasses.size()]);
    }

    /**
     * Retrieves the name of the test suite class.
     * 
     * @return The name of the test suite class, may be <code>null</code>.
     */
    public String getTestSuiteName() {
        return fTestSuiteName;
    }

    /**
     * Retrieves the number of tests in the test suite.
     * 
     * @return The number of tests in the test suite.
     */
    public int getTestCount() {
        return fTestCount;
    }

    /**
     * Retrieves the number of tests that failed in the suite.
     * 
     * @return The number of tests that failed in the suite.
     */
    public int getFailureCount() {
        return fFailureCount;
    }

    /**
     * Retrieves the number of tests that resulted in an error in the suite.
     * 
     * @return The number of tests that resulted in an error in the suite.
     */
    public int getErrorCount() {
        return fErrorCount;
    }

    /**
     * Retrieves the amount of time it took the test suite to execute.
     * 
     * @return The amount of time it took the test suite to execute.
     */
    public long getTimeTaken() {
        return fTimeTaken;
    }

    /**
     * Retrieves the data describing the test suite error.
     * 
     * @return The data describing the test suite error.
     */
    public JUnitParserErrorData getErrorData() {
        return fErrorData;
    }

    /**
     * Sets the data describing the test suite error.
     * 
     * @param errorData
     *            The data describing the test suite error.
     */
    public void setErrorData(JUnitParserErrorData errorData) {
        fErrorData = errorData;

    }

    /**
     * Determines if the test suite ended in error.
     * 
     * @return <code>true</code> if the test suite ended in error
     */
    public boolean isError() {
        return getErrorData() != null;
    }

}
