/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import com.ibm.team.build.internal.common.helper.ValidationHelper;

/**
 * This class represents data parsed from an XML log file produced by a
 * compilation run of ECJ (Eclipse Compiler for Java)
 */
public class JdtParserCompileData {

    /*
     * Hashtable of all the <code>source</code> elements that are part of this
     * compile. Keys are the package names of the java source.
     */
    private Hashtable<String, List<JdtParserSourceData>> fSources = new Hashtable<String, List<JdtParserSourceData>>();

    /*
     * The time in milliseconds it took this compile to run.
     */
    private long fCompileTime;

    /*
     * The number of lines compiled during this compilation run.
     */
    private int fLinesCompiled;

    /*
     * The number of classfiles generated by this compilation run.
     */
    private int fNumberOfClassfiles;

    /**
     * Add a {@link JdtParserSourceData} object representing a
     * <code>source</code> element in the compile log.
     * 
     * @param source
     *            The <code>source</code> element to add to the compile data.
     */
    public void addSource(JdtParserSourceData source) {
        ValidationHelper.validateNotNull("source", source); //$NON-NLS-1$
        
        synchronized (fSources) {
            if (fSources.get(source.getPackageName()) == null) {
                fSources.put(source.getPackageName(), new LinkedList<JdtParserSourceData>());
            }
        }
        fSources.get(source.getPackageName()).add(source);
    }
    
    /**
     * Return the total number of sourcefiles compiled.
     * 
     * @return The number of source files compiled.
     */
    public int getSourceFileCount() {
        int numSources = 0;
        for (String packageName : fSources.keySet()) {
            List<JdtParserSourceData> sources = fSources.get(packageName);
            numSources += sources.size();
        }
        return numSources;
    }
    
    /*
     * Return the data structure containing the source objects.
     */
    protected Hashtable getSources() {
        return fSources;
    }

    /**
     * Allows access to a {@link Collection} containing the set of package names
     * for all of the compiled source files. Each package name is only included
     * once.
     * 
     * @return A {@Collection} containing the set of package names.
     */
    public Collection<String> getPackageNames() {
        return fSources.keySet();
    }

    /**
     * Allow the user to request all the source data objects from a particular
     * java package.
     * 
     * @param packageName
     *            The name of the package whose source data objects are
     *            returned.
     * @return A {@link Collection} of source file data objects.
     */
    public Collection<JdtParserSourceData> getSourcesInPackage(String packageName) {
        ValidationHelper.validateNotNull("packageName", packageName); //$NON-NLS-1$
        if (!fSources.containsKey(packageName)) {
            return Collections.emptyList();
        }
        return fSources.get(packageName);
    }

    /**
     * Set the time in milliseconds it took this compile to run.
     * 
     * @param compileTime
     *            The time in milliseconds.
     */
    public void setCompileTime(long compileTime) {
        fCompileTime = compileTime;
    }

    /**
     * Provides access to the time it took this compile to finish.
     * 
     * @return The time in milliseconds it took this compile to run.
     */
    public long getCompileTime() {
        return fCompileTime;
    }

    /**
     * Sets the number of lines compiled during this compilation run.
     * 
     * @param lines
     *            The number of lines compile during this compilation.
     */
    public void setLinesCompiled(int lines) {
        fLinesCompiled = lines;
    }

    /**
     * Provides access to the number of lines compiled during this compilation
     * run.
     * 
     * @return The number of lines compiled.
     */
    public int getLinesCompiled() {
        return fLinesCompiled;
    }

    /**
     * Sets the number of classfiles generated by this compilation run.
     * 
     * @param numberOfClassfiles
     */
    public void setNumberOfClassfiles(int numberOfClassfiles) {
        fNumberOfClassfiles = numberOfClassfiles;
    }

    /**
     * Provides access to the number of classfiles generated by this compilation
     * run.
     * 
     * @return The number of generated classfiles.
     */
    public int getNumberOfClassfiles() {
        return fNumberOfClassfiles;
    }
    
    /**
     * Return the number of compile errors in the parsed data.
     * 
     * @return The number of compile errors found.
     */
    public int getErrorCount() {
        int errorCount = 0;
        for (Enumeration elements = fSources.elements(); elements.hasMoreElements(); ) {
            List<JdtParserSourceData> dataList = (List<JdtParserSourceData>) elements.nextElement();
            for (JdtParserSourceData sourceData : dataList) {
                errorCount += sourceData.getErrorCount();
            }
        }
        return errorCount;
    }

    /**
     * Return the number of compile warnings in the parsed data.
     * 
     * @return The number of compile warnings found.
     */
    public int getWarningCount() {
        int warningCount = 0;
        for (Enumeration elements = fSources.elements(); elements.hasMoreElements(); ) {
            List<JdtParserSourceData> dataList = (List<JdtParserSourceData>) elements.nextElement();
            for (JdtParserSourceData sourceData : dataList) {
                warningCount += sourceData.getWarningCount();
            }
        }
        return warningCount;
    }

    /**
     * Return the number of compile problems in the parsed data.
     * 
     * @return The number of compile problems found.
     */
    public int getProblemCount() {
        int problemCount = 0;
        for (Enumeration elements = fSources.elements(); elements.hasMoreElements(); ) {
            List<JdtParserSourceData> dataList = (List<JdtParserSourceData>) elements.nextElement();
            for (JdtParserSourceData sourceData : dataList) {
                problemCount += sourceData.getErrorCount();
                problemCount += sourceData.getWarningCount();
            }
        }
        return problemCount;
    }

}
