/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

/**
 * This class represents a <code>&lt;problem&rt;</code> element in the xml log
 * output of an ECJ compilation run.
 */
public class JdtParserProblemData {

    /**
     * Enum representing the different problem severity levels.
     */
    public enum Severity {
        /**
         * Represents a java compile warning.
         */
        WARNING,

        /**
         * Represents a java compile error.
         */
        ERROR
    };

    /**
     * Index in the java source file of the first character of the problem text
     */
    private final int fStartIndexInFile;

    /**
     * Index in the java source file of the last character of the problem text
     */
    private final int fEndIndexInFile;

    /**
     * Line number in the java source file where the problem occurred
     */
    private final int fLineNumber;

    /**
     * The ID string of the problem.
     */
    private final String fId;

    /**
     * The severity of the problem - set to one of the values of the
     * {@link Severity} enum
     */
    private final Severity fSeverity;

    /**
     * The message describing the problem.
     */
    private String fMessage;

    /**
     * The text of the line from the source file where the problem occurred.
     */
    private String fSource;

    /**
     * The index in the source line of the first character of the problem text.
     */
    private int fSourceStart = -1;

    /**
     * The index in the source line of the last character of the problem text.
     */
    private int fSourceEnd = -1;

    /**
     * Create a new JdtParserProblemData object.
     * 
     * @param charStart
     *            The start index in the java file of the problem.
     * @param charEnd
     *            The end index in the java file of the problem.
     * @param lineNumber
     *            The line number in the java file of the problem.
     * @param id
     *            The id of the problem.
     * @param severity
     *            The severity of the problem.
     */
    public JdtParserProblemData(int charStart, int charEnd, int lineNumber, String id, Severity severity) {
        fStartIndexInFile = charStart;
        fEndIndexInFile = charEnd;
        fLineNumber = lineNumber;
        fId = id;
        fSeverity = severity;
    }

    /**
     * Provides access to the starting index of the problem in the java file.
     * 
     * @return The index in the java file of the first character of the problem
     *         code.
     */
    public int getCharStart() {
        return fStartIndexInFile;
    }

    /**
     * Provides access to the end index of the problem in the java file.
     * 
     * @return The index in the java file of the last character of the problem
     *         code.
     */
    public int getCharEnd() {
        return fEndIndexInFile;
    }

    /**
     * Provides access to the line number where the problem occurs in the java
     * file.
     * 
     * @return The line number in the java file where the problem occurs.
     */
    public int getLineNumber() {
        return fLineNumber;
    }

    /**
     * Provides access to the ID of the problem.
     * 
     * @return The ID of the problem.
     */
    public String getId() {
        return fId;
    }

    /**
     * Provides access to the severity of the problem.
     * 
     * @return The severity of the problem.
     */
    public Severity getSeverity() {
        return fSeverity;
    }

    /**
     * Sets the message describing this problem.
     * 
     * @param message
     *            The message describing the problem.
     */
    public void setMessage(String message) {
        fMessage = message;
    }

    /**
     * Provides access to the message describing the problem.
     * 
     * @return The message describing the problem.
     */
    public String getMessage() {
        return fMessage;
    }

    /**
     * Sets the source line where the problem occurred.
     * 
     * @param source
     *            The text of the line in the source file where the problem
     *            occurred.
     */
    public void setSource(String source) {
        fSource = source;
    }

    /**
     * Provides access to the line from the source file where the problem
     * occurred.
     * 
     * @return The text of the line in the source file where the problem
     *         occurred.
     */
    public String getSource() {
        return fSource;
    }

    /**
     * Sets the index in the source line of the first character of the problem
     * text.
     * 
     * @param sourceStart
     *            The index in the source line of the first character of the
     *            problem text
     */
    public void setSourceStart(int sourceStart) {
        fSourceStart = sourceStart;
    }

    /**
     * Provides access to the index in the source line of the first character of
     * the problem text.
     * 
     * @return The index in the source line of the first character of the
     *         problem text
     */
    public int getSourceStart() {
        return fSourceStart;
    }

    /**
     * Sets the index in the source line of the last character of the problem
     * text.
     * 
     * @param sourceEnd
     *            The index in the source line of the last character of the
     *            problem text
     */
    public void setSourceEnd(int sourceEnd) {
        fSourceEnd = sourceEnd;
    }

    /**
     * Provides access to the index in the source line of the last character of
     * the problem text.
     * 
     * @return The index in the source line of the last character of the problem
     *         text
     */
    public int getSourceEnd() {
        return fSourceEnd;
    }

    /**
     * Return true if this object represents a compile error, false otherwise.
     * 
     * @return true if this object represents a compile error, false otherwise.
     */
    public boolean isError() {
        return fSeverity == Severity.ERROR;
    }
}
