/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.parser.data;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.parser.data.JdtParserProblemData.Severity;

/**
 * This class represents a <code>source</code> element in the xml log output
 * from and ECJ compile
 */
public class JdtParserSourceData {

    private final String fClassName;
    private final String fPackageName;
    private final String fPath;

    /*
     * The warnings associated with this java source.
     */
    private List<JdtParserProblemData> fWarnings = new LinkedList<JdtParserProblemData>();

    /*
     * The errors associated with this java source.
     */
    private List<JdtParserProblemData> fErrors = new LinkedList<JdtParserProblemData>();

    /*
     * The names of the classfiles associated with this java source.
     */
    private List<String> fClassfileNames = new LinkedList<String>();

    /**
     * Create a new JdtParserSourceData object representing a java source with
     * the specified classname and package name. The package name is assumed to
     * be in standard java format - i.e. "com.ibm.team.build" - using '.' as the
     * separator.
     * 
     * @param className
     *            The name of the source java class.
     * @param packageName
     *            The name of the source java class's package.
     * @param path
     *            The path to the source file.
     */
    public JdtParserSourceData(String className, String packageName, String path) {
        ValidationHelper.validateNotEmpty("className", className); //$NON-NLS-1$
        ValidationHelper.validateNotNull("packageName", packageName); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("path", path); //$NON-NLS-1$
        fClassName = className;
        fPackageName = packageName;
        fPath = path;
    }

    /**
     * Provides access to the class name of this source element.
     * 
     * @return The java class name.
     */
    public String getClassName() {
        return fClassName;
    }

    /**
     * Provides access to the java package name of this source element.
     * 
     * @return The java package name.
     */
    public String getPackageName() {
        return fPackageName;
    }
    
    /**
     * Provides access to the path to the java source file.
     * 
     * @return The path to the java source file.
     */
    public String getPath() {
        return fPath;
    }

    /**
     * Add the specified problem to the data associated with this source
     * element.
     * 
     * @param problem
     *            The problem to be added.
     */
    public void addProblem(JdtParserProblemData problem) {
        ValidationHelper.validateNotNull("problem", problem); //$NON-NLS-1$
        if (problem.getSeverity() == Severity.ERROR) {
            fErrors.add(problem);
        } else {
            fWarnings.add(problem);
        }
    }

    /**
     * Add a classfile name associated with this java source.
     * 
     * @param classfileName
     *            The name of the classfile to add.
     */
    public void addClassfileName(String classfileName) {
        ValidationHelper.validateNotNull("classfileName", classfileName); //$NON-NLS-1$
        fClassfileNames.add(classfileName);
    }

    /**
     * Provides access to the names of the classfiles associated with this java
     * source.
     * 
     * @return An array containing the classfile names associated with this java
     *         source.
     */
    public String[] getClassfileNames() {
        return fClassfileNames.toArray(new String[fClassfileNames.size()]);
    }

    /**
     * Provides access to the {@link JdtParserProblemData} objects representing
     * the warnings that occurred compiling this source.
     * 
     * @return The warnings that occurred while compiling this source.
     */
    public Collection<JdtParserProblemData> getWarnings() {
        return fWarnings;
    }

    /**
     * Provides access to the {@link JdtParserProblemData} objects representing
     * the errors that occurred compiling this source.
     * 
     * @return The errors that occurred while compiling this source.
     */
    public Collection<JdtParserProblemData> getErrors() {
        return fErrors;
    }

    /**
     * Returns the number of compile errors found in the parsed data.
     * 
     * @return The number of compile errors.
     */
    public int getErrorCount() {
        return fErrors.size();
    }

    /**
     * Returns the number of compile warnings found in the parsed data.
     * 
     * @return The number of compile warnings.
     */
    public int getWarningCount() {
        return fWarnings.size();
    }

    /**
     * Returns the number of compile problems found in the parsed data.
     * 
     * @return The number of compile problems.
     */
    public int getProblemCount() {
        return fErrors.size() + fWarnings.size();
    }
}
