/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.publishing;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Base class for publishers that add a contribution to an existing build
 * result.
 */
public abstract class AbstractContributionPublisher {
    /**
     * The extended contribution type id associated with this contribution.
     * 
     * @see com.ibm.team.build.common.model.IBuildResultContribution#getExtendedContributionTypeId()
     */
    private final String fExtendedContributionTypeId;

    /**
     * The label describing the file contribution to be published
     */
    private final String fLabel;

    /**
     * The name of the component this file contribution is associated with.
     */
    private String fComponentName;

    /**
     * Create a contribution publisher object.
     * 
     * @param extendedContributionTypeId
     *            the extended contribution type id for the contribution
     * @param label
     *            the label describing the file contribution to be published by
     *            this object
     */
    public AbstractContributionPublisher(String extendedContributionTypeId, String label) {
        ValidationHelper.validateNotNull("extendedContributionTypeId", extendedContributionTypeId); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("extendedContributionTypeId", extendedContributionTypeId); //$NON-NLS-1$
        ValidationHelper.validateNotNull("label", label); //$NON-NLS-1$

        fExtendedContributionTypeId = extendedContributionTypeId;
        fLabel = label;
    }

    /**
     * Subclasses should initialize the contribution with whatever content or
     * properties they require.
     * 
     * @param contribution
     *            The build result contribution.
     * @param teamRepository
     *            The repository context.
     */
    protected abstract void initializeContribution(IBuildResultContribution contribution, ITeamRepository teamRepository)
            throws TeamRepositoryException;

    /**
     * Set the component name that the published file contribution will be
     * associated with.
     * 
     * @param componentName
     *            The name of the component.
     */
    public void setComponentName(String componentName) {
        ValidationHelper.validateNotEmpty("componentName", componentName); //$NON-NLS-1$
        fComponentName = componentName;
    }

    /**
     * Return the name of the component that this file contribution will be
     * associated with.
     * 
     * @return The name of the component;
     */
    public String getComponentName() {
        return fComponentName;
    }

    /**
     * Create a build result contribution and add it to the given build result.
     * The build result must exist in the repository specified by
     * <code>teamRepository</code>.
     * 
     * @param buildResultHandle
     *            the build result in which to add the contribution
     * @param contributionStatus
     *            the status to set for the contribution
     * @param teamRepository
     * @return The contribution that was published.
     * @throws TeamRepositoryException
     */
    public final IBuildResultContribution publish(IBuildResultHandle buildResultHandle, BuildStatus contributionStatus,
            ITeamRepository teamRepository) throws TeamRepositoryException {

        IBuildResultContribution contribution = BuildItemFactory.createBuildResultContribution();
        contribution.setLabel(fLabel);
        contribution.setImpactsPrimaryResult(true);
        contribution.setStatus(contributionStatus);
        contribution.setExtendedContributionTypeId(fExtendedContributionTypeId);

        if (fComponentName != null) {
            contribution.setComponentName(fComponentName);
        }

        initializeContribution(contribution, teamRepository);

        ClientFactory.getTeamBuildClient(teamRepository).addBuildResultContribution(buildResultHandle, contribution,
                null);

        return contribution;
    }

    /**
     * Get the extended content type id associated with this contribution.
     * 
     * @return the extended content type id associated with this contribution
     * @see com.ibm.team.build.common.model.IBuildResultContribution#getExtendedContributionTypeId()
     */
    public String getExtendedContributionTypeId() {
        return fExtendedContributionTypeId;
    }

}
