/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.publishing;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.parser.ComponentMapParser;

/**
 * The class maps package names to a component name. The mapping is specified by
 * means of an xml mapping file provided when a ComponentMapRegistry object is
 * instantiated. An example entry is:
 * 
 * <pre>
 *     &lt;component_mapping&gt;
 *          &lt;component name=&quot;Build&quot;&gt;
 *              &lt;includePackage name=&quot;com.ibm.team.build.*&quot;/&gt;
 *          &lt;/component&gt;
 *     &lt;/component_mapping&gt;
 * </pre>
 * 
 * The package name can either be a prefix (ending in .*) as in the example
 * above, or it can be a specific package name.
 * 
 * <pre></pre>
 * 
 * When queried, the registry will first attempt to match the entire package
 * name and, if that fails, it will match against the longest matching prefix.
 * 
 * @since 0.5
 */
public class ComponentMapRegistry {

    /**
     * The parser used to parse the registry file.
     */
    private ComponentMapParser fParser;

    /**
     * Create a ComponentMapRegistry object, reading the mapping information
     * from the specified xml file.
     * 
     * @param mappingFilePath
     *            Path to the file containing the mapping information in XML
     *            format.
     * @throws SAXException
     *             if a problem occurs while parsing the XML file
     * @throws IOException
     *             if the file to parse can not be opened or read
     * @throws ParserConfigurationException 
     */
    public ComponentMapRegistry(String mappingFilePath) throws IOException, SAXException, ParserConfigurationException {
        ValidationHelper.validateNotEmpty(mappingFilePath, mappingFilePath);
        fParser = new ComponentMapParser(new File(mappingFilePath));
        fParser.parse();
    }

    /**
     * Map the given package name to a component name based on the mapping
     * information used to instantiate this ComponentMapRegistry. Returns null
     * if the package does not map to any components.
     * 
     * @param packageName
     *            The name of the package to map.
     * @return The component name this package maps to, or null if it maps to
     *         none.
     */
    public String mapPackageToComponent(String packageName) {
        ValidationHelper.validateNotNull("packageName", packageName); //$NON-NLS-1$

        Hashtable<String, String> packageNameToComponentMappings = fParser.getPackageNameToComponentMappings();
        Hashtable<String, String> packagePrefixToComponentMappings = fParser.getPackagePrefixToComponentMappings();
        String componentName = packageNameToComponentMappings.get(packageName);
        int longestMatch = 0;
        if (componentName == null) {
            for (String packagePrefix : packagePrefixToComponentMappings.keySet()) {
                if (packageName.startsWith(packagePrefix)) {
                    if (packagePrefix.length() > longestMatch) {
                        componentName = packagePrefixToComponentMappings.get(packagePrefix);
                        longestMatch = packagePrefix.length();
                    }
                }
            }
        }

        return componentName == null ? fParser.getDefaultComponentName() : componentName;
    }
}
