/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.publishing;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.nio.charset.Charset;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.internal.RetryPolicy;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.LineDelimiter;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Publisher that adds a contribution with content to an existing build result.
 * <p>
 * Files with a text mime-type published with this publisher are converted from
 * their original character encoding to UTF-8 when they are stored in the
 * repository.
 */
public class ContentPublisher extends AbstractFilePublisher {
    /**
     * Maximum number of times to retry publishing content.
     */
    private static final int MAX_RETRIES = 5;

    /**
     * The content type of the file to be published as Content.
     * 
     * @see com.ibm.team.repository.common.IContent#getContentType()
     */
    private final String fContentType;

    /**
     * The character encoding of the file to be published as content.
     * 
     * @see com.ibm.team.repository.common.IContent#getCharacterEncoding()
     */
    private final String fCharacterEncoding;

    /**
     * Create a ContentPublisher object.
     * 
     * @param typeId
     *            the extended contribution type id to associate with the file
     *            contribution being published by this object
     * @param fileLocation
     *            the location of the file to be used as content in the
     *            contribution being published by this object
     * @param label
     *            the label describing the contribution to be published by this
     *            object
     * @param contentType
     *            The content type of the data published as Content. May be
     *            null.
     * @param characterEncoding
     *            The character encoding of any textual data published as
     *            Content. May be null.
     */
    public ContentPublisher(String typeId, String fileLocation, String label, String contentType,
            String characterEncoding) {
        super(typeId, fileLocation, label);
        fContentType = contentType;
        fCharacterEncoding = characterEncoding;
    }

    protected IContent getFileContent(ITeamRepository teamRepository) throws TeamRepositoryException,
            TeamBuildException {
        File file = new File(getFilePath());
        IContent content = null;

        try {
            String contentType = getContentType();

            int retryNumber = 0;
            while (content == null && retryNumber <= MAX_RETRIES) {
                try {
                    if (ContentUtil.isTextContentType(contentType)) {
                        content = storeTextContent(contentType, file, teamRepository);
                    } else {
                        content = storeContent(contentType, file, teamRepository);
                    }
                } catch (TeamRepositoryException e) {
                    retryNumber++;
                    if (isRetryable(retryNumber, e)) {
                        pause(retryNumber);
                    } else {
                        throw e;
                    }
                }
            }
        } catch (FileNotFoundException exception) {
            throw new TeamBuildException(exception);
        }

        return content;
    }

    /**
     * Determine if the failure is retryable
     * @param retryNumber The retry that would be attempted if retry is allowed
     * @param e The exception that caused the failure
     * @return <code>true</code> if retry should proceed <code>false</code>
     * to propagate the exception
     */
    protected boolean isRetryable(int retryNumber, TeamRepositoryException e) {
        return RetryPolicy.isRetryEnabled() && RetryPolicy.isRetryable(e) && retryNumber <= MAX_RETRIES;
    }

    /**
     * Get the content type of the file to be published by this publisher. The
     * content type is determined by the following algorithm:
     * <p>
     * If the content type was specified by the user then that type will be
     * returned.
     * <p>
     * If no content type is specified by the user, then an attempt is made to
     * determine the content type base on the filename extension.
     * <p>
     * If that fails, the publisher is queried for a default content type. If a
     * default is set for this publisher, it is returned.
     * <p>
     * If the publisher does not specify a default content type, then
     * "application/unknown" is used.
     * 
     * @return the content type determined for the file to be published
     */
    public String getContentType() {
        /*
         * If the user specified a content type, we'll use that.
         */
        String contentType = fContentType;

        /*
         * If the user did not specify a content type, attempt to ascertain the
         * type based on the filename extension.
         */
        if (contentType == null) {
            contentType = ContentUtil.getContentTypeFromFileName(getFilePath());
        }

        /*
         * If content type could not be determined from the filename extension,
         * check for a default type for the publisher. If that is also not set,
         * default to unknown.
         */
        if (contentType == null) {
            contentType = getDefaultContentType() == null ? IContent.CONTENT_TYPE_UNKNOWN : getDefaultContentType();
        }

        return contentType;
    }

    /**
     * Store non-text content (example binary data) on the server.
     * 
     * @param contentType The content type of the data to store
     * @param file The file to upload to the server
     * @param teamRepository The server to which the file contents should be uploaded.
     * @return
     * @throws TeamRepositoryException
     * @throws FileNotFoundException
     */
    protected IContent storeContent(String contentType, File file, ITeamRepository teamRepository)
            throws TeamRepositoryException, FileNotFoundException {
        IContent content;
        content = teamRepository.contentManager().storeContent(contentType, null,
                LineDelimiter.LINE_DELIMITER_NONE, new FileInputStream(file), null, new NullProgressMonitor());
        return content;
    }

    protected IContent storeTextContent(String contentType, File textFile, ITeamRepository teamRepository)
            throws FileNotFoundException, TeamRepositoryException {
        FileInputStream inputStream = new FileInputStream(textFile);

        /*
         * If the user specified a character encoding, we'll use that.
         */
        String encoding = fCharacterEncoding;

        /*
         * If the user did not specify a character encoding, assume the platform
         * default encoding.
         */
        if (encoding == null) {
            encoding = Charset.defaultCharset().name();
        }

        return teamRepository.contentManager().storeText(contentType, encoding, LineDelimiter.LINE_DELIMITER_NONE,
                inputStream, null, new NullProgressMonitor());
    }

    /**
     * Pause before a retry
     * @param retryCount
     */
    protected void pause(int retryNumber) {
        assert retryNumber >= 1;
        try {
            // sleep for 2^retryNumber seconds, max 60
            int secs = 60;
            if (retryNumber < 6) { // check up-front to avoid overflow
                secs = (1 << retryNumber);
            }
            Thread.sleep(secs * 1000);
        } catch (InterruptedException exception) {
            throw new OperationCanceledException();
        }
    }

    /**
     * Get the character encoding of the content to be published if specified by
     * the user.
     * 
     * @return the character encoding used to read in a user-specified text
     *         file.
     * @see com.ibm.team.repository.common.IContent#getCharacterEncoding()
     */
    public String getCharacterEncoding() {
        return fCharacterEncoding;
    }

    /**
     * Returns the default content type for this publisher.
     * <p>
     * By default, publishers have no default content type. Subclasses may
     * override this method to provide one.
     * 
     * @return A string representing the default content type for this
     *         publisher. Returns null if this publisher has no default content
     *         type.
     */
    public String getDefaultContentType() {
        return null;
    }
}
