/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.publishing;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.util.NLS;
import org.xml.sax.SAXException;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IJUnitTestCase;
import com.ibm.team.build.common.model.IJUnitTestClass;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.common.helper.FileHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.parser.JUnitLogParser;
import com.ibm.team.build.internal.parser.data.JUnitParserTestCaseData;
import com.ibm.team.build.internal.parser.data.JUnitParserTestClassData;
import com.ibm.team.build.internal.parser.data.JUnitParserTestSuiteData;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Updates a build result with information from a junit log.
 */
public class JUnitLogPublisher {
    /**
     * The (optional) name of the component this JUnit contribution is
     * associated with.
     */
    private String fComponentName;

    /**
     * The (optional) file containing package-name-to-component mapping
     * information
     */
    private File fComponentMapFile;

    /**
     * The component map registry used if <code>fComponentMapFile</code> is
     * set.
     */
    private ComponentMapRegistry fComponentMapRegistry;

    /**
     * The contributions that were published to the build result.
     */
    private List<IBuildResultContribution> fPublishedContributions = new ArrayList<IBuildResultContribution>();

    /**
     * Set the component name that the published JUnit contribution will be
     * associated with.
     * 
     * @param componentName
     *            The name of the component.
     */
    public void setComponentName(String componentName) {
        ValidationHelper.validateNotEmpty("componentName", componentName); //$NON-NLS-1$
        fComponentName = componentName;
    }

    /**
     * Set the component mapping file containing mapping information that the
     * publisher will use to map packages in the compile result to components.
     * 
     * @param componentMap
     *            The file containing the package-name-to-component mapping
     *            information.
     */
    public void setComponentMap(File componentMap) {
        fComponentMapFile = componentMap;
    }

    /**
     * Retrieves the contributions that were published as result of the most
     * recent invocation of the
     * {@link #publish(IBuildResult, String, String, ITeamRepository, IProgressMonitor)}
     * method. Will be empty if the <code>publish</code> method has yet to be
     * invoked, but never <code>null</code>.
     * 
     * @return The contributions that were published as result of the most
     *         recent invocation of the
     *         {@link #publish(IBuildResult, String, String, ITeamRepository, IProgressMonitor)}
     *         method. It may be <code>null</code> if the <code>publish</code>
     *         method has yet to be invoked.
     */
    public IBuildResultContribution[] getPublishedContributions() {
        return (IBuildResultContribution[]) fPublishedContributions.toArray(new IBuildResultContribution[fPublishedContributions.size()]);
    }

    /**
     * Update the given build result with information from one or more JUnit xml
     * log files. The build result must exist in the repository specified by
     * <code>teamRepository</code>.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param logFileOrDirectory
     *            A JUnit xml log file, or a directory of JUnit xml log files.
     *            If a single log file is given, and the log file isn't found or
     *            is not parseable, a <code>TeamBuildException</code> is
     *            thrown.
     * @param mayFailPattern
     *            A substring of a test suite class name. Any test suite class
     *            name containing the pattern is allowed to fail without causing
     *            the overall build to fail. The pattern is a simple string (not
     *            a regular expression). May be empty or null, which causes the
     *            pattern to be ignored.
     * @param teamRepository
     *            The repository to publish the JUnit log files to.
     * @param monitor
     *            A progress monitor. May be <code>null</code> if progress
     *            reporting is not desired.
     * @return Any non-fatal errors that occurred during publishing of the JUnit
     *         log file, or all errors that ocurred during publishing of a
     *         directory.
     * @throws TeamRepositoryException
     *             If an exception occurs while publishing to the repository.
     */
    public String[] publish(IBuildResult buildResult, String logFileOrDirectory, String mayFailPattern,
            ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {

        fPublishedContributions = new ArrayList<IBuildResultContribution>();

        File fileOrDirectory = new File(logFileOrDirectory);
        if (fileOrDirectory.isDirectory()) {
            File[] files = fileOrDirectory.listFiles(new FileFilter() {
                public boolean accept(File pathname) {
                    if (pathname.isFile()) {
                        if (FileHelper.hasFileSuffix(pathname, ".xml")) { //$NON-NLS-1$
                            return true;
                        }

                    }
                    return false;
                }
            });

            List<String> warnings = new ArrayList<String>();
            for (File file : files) {
                String[] results = publishFile(buildResult, file.getAbsolutePath(), mayFailPattern, teamRepository,
                        monitor);
                for (String warning : results) {
                    warnings.add(NLS.bind(PublishingMessages.JUnitLogPublisher_PUBLISH_FILE_AND_WARNING,
                            file.getAbsolutePath(), warning));
                }
            }
            return (String[]) warnings.toArray(new String[warnings.size()]);
        } else {
            return publishFile(buildResult, fileOrDirectory.getAbsolutePath(), mayFailPattern, teamRepository, monitor);
        }
    }

    /**
     * Update the given build result with information from <code>logFile</code>.
     * The build result must exist in the repository specified by
     * <code>teamRepository</code>.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param logFile
     *            A JUnit xml log file. If the log file isn't found or is not
     *            parseable a <code>TeamBuildException</code> is thrown.
     * @param mayFailPattern
     *            A substring of a test suite class name. Any test suite class
     *            name containing the pattern is allowed to fail without causing
     *            the overall build to fail. The pattern is a simple string (not
     *            a regular expression). May be empty or null, which causes the
     *            pattern to be ignored.
     * @param teamRepository
     *            The repository to publish the JUnit log file to.
     * @param monitor
     *            A progress monitor. May be <code>null</code> if progress
     *            reporting is not desired.
     * @return Any non-fatal errors that occurred during publishing of the JUnit
     *         log file.
     * @throws TeamRepositoryException
     *             If an exception occurs while publishing to the repository.
     */
    private String[] publishFile(IBuildResult buildResult, String logFile, String mayFailPattern,
            ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {
        JUnitLogParser parser = null;

        try {
            parser = new JUnitLogParser(new File(logFile));
            parser.parse();
        } catch (SAXException exception) {
            throw new TeamBuildException(NLS.bind(PublishingMessages._EXCEPTION_PARSING, logFile), exception);
        } catch (IOException exception) {
            throw new TeamBuildException(NLS.bind(PublishingMessages._EXCEPTION_PARSING, logFile), exception);
        } catch (ParserConfigurationException exception) {
            throw new TeamBuildException(NLS.bind(PublishingMessages._EXCEPTION_PARSING, logFile), exception);
        }

        JUnitParserTestSuiteData[] testSuites = parser.getTestSuites();

        // Add the JUnit contributions to the build result
        IBuildResultContribution[] contributions = new IBuildResultContribution[testSuites.length];
        for (int i = 0; i < testSuites.length; i++) {
            contributions[i] = createBuildResultContribution(buildResult, testSuites[i], teamRepository);

            String componentName = null;
            try {
                componentName = getComponentName(testSuites[i]);
            } catch (Exception exception) {
                String testSuiteName = testSuites[i].getTestSuiteName() == null ? PublishingMessages.JUnitLogPublisher_UNKNOWN_TEST_SUITE_NAME
                        : testSuites[i].getTestSuiteName();
                throw new TeamRepositoryException(NLS.bind(
                        PublishingMessages.JUnitLogPublisher_EXCEPTION_GETTING_COMPONENT, testSuiteName), exception);
            }

            if (componentName != null) {
                contributions[i].setComponentName(componentName);
            }

            if (mayFailPattern != null && mayFailPattern.length() > 0) {
                if (contributions[i].getLabel().indexOf(mayFailPattern) > -1) {
                    contributions[i].setImpactsPrimaryResult(false);
                }
            }

            ClientFactory.getTeamBuildClient(teamRepository).save(
                    (IJUnitTestSuiteContribution) contributions[i].getExtendedContribution(), null);
        }

        ClientFactory.getTeamBuildClient(teamRepository).addBuildResultContributions(buildResult, contributions, null);

        fPublishedContributions.addAll(Arrays.asList(contributions));

        return parser.getParseErrors();
    }

    /**
     * Get the component name.
     * 
     * @param testSuiteData
     *            The JUnit test suite data.
     * @return The component name, or <code>null</code> if no component is
     *         set.
     * @throws SAXException
     * @throws IOException
     * @throws ParserConfigurationException 
     */
    private String getComponentName(JUnitParserTestSuiteData testSuiteData) throws IOException, SAXException, ParserConfigurationException {
        String componentName = fComponentName;

        if (fComponentMapFile != null && testSuiteData.getTestSuiteName() != null) {
            if (fComponentMapRegistry == null) {
                fComponentMapRegistry = new ComponentMapRegistry(fComponentMapFile.getAbsolutePath());
            }
            componentName = fComponentMapRegistry.mapPackageToComponent(testSuiteData.getTestSuiteName());
        }

        return componentName;
    }

    /**
     * Creates a build result contribution that represents a JUnit test suite
     * contribution using the JUnit test suite parse data as the source for the
     * contribution.
     * 
     * @param buildResult
     *            The build result to create the contributions for.
     * @param testSuiteData
     *            The JUnit test suite parse data to use as the source of the
     *            JUnit test suite contribution.
     * @param teamRepository
     *            The repository being using.
     * @return The build result JUnit test suite contribution that was created
     *         using the JUnit test suite parse data as the source.
     * @throws TeamRepositoryException
     *             if the JUnit error or failure data could not be stored as
     *             content.
     */
    private IBuildResultContribution createBuildResultContribution(IBuildResult buildResult,
            JUnitParserTestSuiteData testSuiteData, ITeamRepository teamRepository) throws TeamRepositoryException {
        IBuildResultContribution buildResultContribution = BuildItemFactory.createBuildResultContribution();
        IJUnitTestSuiteContribution testSuiteContribution = BuildItemFactory.createJUnitTestSuiteContribution();

        String testSuiteName = testSuiteData.getTestSuiteName() == null ? PublishingMessages.JUnitLogPublisher_UNKNOWN_TEST_SUITE_NAME : testSuiteData.getTestSuiteName();

        // Update the build result contribution
        buildResultContribution.setLabel(testSuiteName);
        buildResultContribution.setTimeTaken(testSuiteData.getTimeTaken());
        buildResultContribution.setImpactsPrimaryResult(true);
        buildResultContribution.setExtendedContributionTypeId(IJUnitTestSuiteContribution.EXTENDED_CONTRIBUTION_TYPE_ID);
        buildResultContribution.setExtendedContribution(testSuiteContribution);
        if (testSuiteData.getErrorCount() == 0 && testSuiteData.getFailureCount() == 0) {
            buildResultContribution.setStatus(BuildStatus.OK);
        } else {
            buildResultContribution.setStatus(BuildStatus.ERROR);
        }

        // Update the test suite contribution
        testSuiteContribution.setTestSuiteClassName(testSuiteName);
        testSuiteContribution.setErrorCount(testSuiteData.getErrorCount());
        testSuiteContribution.setFailureCount(testSuiteData.getFailureCount());
        testSuiteContribution.setTestCount(testSuiteData.getTestCount());
        testSuiteContribution.setBuildResult(buildResult);
        if (testSuiteData.getErrorData() != null) {
            testSuiteContribution.setErrorData(ContentUtil.stringToContent(teamRepository,
                    testSuiteData.getErrorData().getStackTrace()));
        }

        JUnitParserTestClassData[] testClassesData = testSuiteData.getTestClasses();
        for (JUnitParserTestClassData testClassData : testClassesData) {
            IJUnitTestClass testClass = BuildItemFactory.createJUnitTestClass();
            testClass.setClassName(testClassData.getTestClassName());
            testClass.setErrorCount(testClassData.getErrorCount());
            testClass.setFailureCount(testClassData.getFailureCount());
            testClass.setTestCount(testClassData.getTestCount());
            testClass.setTimeTaken(testClassData.getTimeTaken());
            testClass.setOrdinal(testClassData.getOrdinal());
            testClass.setBuildResult(buildResult);

            JUnitParserTestCaseData[] testCasesData = testClassData.getTestCases();
            for (JUnitParserTestCaseData testCaseData : testCasesData) {
                IJUnitTestCase testCase = BuildItemFactory.createJUnitTestCase();
                testCase.setName(testCaseData.getName());
                testCase.setClassName(testCaseData.getTestClassName());
                testCase.setTimeTaken(testCaseData.getTimeTaken());
                testCase.setOrdinal(testCaseData.getOrdinal());
                testCase.setError(testCaseData.isError());
                if (testCaseData.getErrorData() != null) {
                    testCase.setErrorData(ContentUtil.stringToContent(teamRepository,
                            testCaseData.getErrorData().getStackTrace()));
                }
                testCase.setFailure(testCaseData.isFailure());
                if (testCaseData.getFailureData() != null) {
                    testCase.setFailureData(ContentUtil.stringToContent(teamRepository,
                            testCaseData.getFailureData().getStackTrace()));
                }
                testClass.getTestCases().add(testCase);
            }
            testSuiteContribution.getTestClasses().add(testClass);
            ClientFactory.getTeamBuildClient(teamRepository).save(testClass, null);
        }
        return buildResultContribution;
    }
}
