/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.publishing;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.util.NLS;
import org.xml.sax.SAXParseException;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.CompileProblemType;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompileOutputFile;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompileProblem;
import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.common.helper.FileHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.parser.JdtSimpleCompileLogParser;
import com.ibm.team.build.internal.parser.JdtXmlCompileLogParser;
import com.ibm.team.build.internal.parser.data.JdtParserCompileData;
import com.ibm.team.build.internal.parser.data.JdtParserProblemData;
import com.ibm.team.build.internal.parser.data.JdtParserSourceData;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Publisher for JDT compile logs. The publisher parses the logs in a given
 * directory and creates build result contributions of type
 * <code>CompileContribution</code>.
 */
public class JdtCompileLogPublisher {
    /**
     * Default suffix for simple log files.
     */
    private static final String SIMPLE_LOG_SUFFIX = ".log"; //$NON-NLS-1$

    /**
     * Default suffix for xml log files.
     */
    private static final String XML_LOG_SUFFIX = ".xml"; //$NON-NLS-1$

    /**
     * Suffix that JDT compiler output log files are expected to have in a PDE
     * build environment.
     */
    private static final String PDE_BUILD_SIMPLE_LOG_SUFFIX = ".bin.log"; //$NON-NLS-1$

    /**
     * Suffix that JDT compiler xml output log files are expected to have in a
     * PDE build environment.
     */
    private static final String PDE_BUILD_XML_LOG_SUFFIX = ".bin.xml"; //$NON-NLS-1$

    /**
     * JDT compiler output log file name expected in a PDE
     * build environment for jar'd plug-ins.
     */
    private static final String PDE_BUILD_DOT_SIMPLE_LOG = "@dot.log"; //$NON-NLS-1$
    
    /**
     * JDT compiler output xml file name expected in a PDE
     * build environment for jar'd plug-ins.
     */
    private static final String PDE_BUILD_DOT_XML_LOG = "@dot.xml"; //$NON-NLS-1$

    /**
     * The (optional) name of the component this compile contribution is
     * associated with.
     */
    private String fComponentName;

    /**
     * The (optional) file containing package-name-to-component mapping
     * information
     */
    private File fComponentMapFile;

    /**
     * The component map registry used to map compiled packages to components
     */
    private ComponentMapRegistry fComponentMap;

    /**
     * The collection of simple compile contributions that will be published.
     */
    private List<IBuildResultContribution> fSimpleCompileContributions = new LinkedList<IBuildResultContribution>();

    /**
     * The collection of compile contributions that will be published.
     */
    private Hashtable<String, IBuildResultContribution> fCompileContributions = new Hashtable<String, IBuildResultContribution>();

    /**
     * Flag specifying if this is a pdebuild-specific contribution.
     */
    private boolean fPdeBuild = false;

    /**
     * Set the component name that the published compile contribution will be
     * associated with.
     * 
     * @param componentName
     *            The name of the component.
     */
    public void setComponentName(String componentName) {
        ValidationHelper.validateNotEmpty("componentName", componentName); //$NON-NLS-1$
        fComponentName = componentName;
    }

    /**
     * Set the component mapping file containing mapping information that the
     * publisher will use to map packages in the compile result to components.
     * 
     * @param componentMap
     *            The file containing the package-name-to-component mapping
     *            information.
     */
    public void setComponentMap(File componentMap) {
        fComponentMapFile = componentMap;
    }

    /**
     * Set the flag specifying if the logs being published are build logs
     * 
     * @param isPdeBuild
     *            true if the logs being published are pde build logs, false if
     *            they are generic jdt logs
     */
    public void setPdeBuild(boolean isPdeBuild) {
        fPdeBuild = isPdeBuild;
    }

    /**
     * Update the given build result with JDT compiler output from the given log
     * file or directory. If a directory is specified, the directory is searched
     * recursively for compile log files and all log files found are used to
     * update the build result. The build result must exist in the repository
     * specified by <code>teamRepository</code>. Log files are found by
     * inspecting file suffixes. In a PDE build environment, all filenames
     * ending with <code>.bin.log</code> or <code>.bin.xml</code> are
     * assumed to be logfiles. In a non-PDE environment, filenames ending in
     * <code>.log</code> or <code>.xml</code> are assumed to be logfiles.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param logFilePath
     *            The absolute path to a log file or to a directory containing
     *            (at any depth) log files. If the file or directory isn't found
     *            a <code>TeamBuildException</code> is thrown.
     * @param teamRepository
     * @param monitor
     *            A progress monitor. May be <code>null</code> if progress
     *            reporting is not desired.
     * @return The Collection of contributions of type
     *         <code>BuildResultContribution</code> that were published.
     * @throws TeamRepositoryException
     */
    public Collection publish(IBuildResult buildResult, String logFilePath, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws TeamRepositoryException {
        String sourceDirectory = logFilePath;
        String[] logFiles = null;

        File logFile = new File(logFilePath);

        if (logFile.isDirectory()) {
            logFiles = getLogFiles(logFile);
        } else {
            logFiles = new String[] { logFilePath };
            sourceDirectory = null;
        }

        createBuildResultContributions(buildResult, logFiles, sourceDirectory, teamRepository, monitor);

        Collection<IBuildResultContribution> contributions = fSimpleCompileContributions;
        if (contributions.size() == 0) {
            contributions = fCompileContributions.values();
        } else {
            contributions.addAll(fCompileContributions.values());
        }

        // Save all the compile contribution auditables.
        for (IBuildResultContribution contribution : contributions) {
            ICompileContribution compileContribution = (ICompileContribution) contribution.getExtendedContribution();

            for (Object object : compileContribution.getCompilePackages()) {
                saveCompilePackage((ICompilePackage) object, teamRepository, monitor);
            }

            ClientFactory.getTeamBuildClient(teamRepository).save(compileContribution, monitor);
        }

        // Add the contributions to the build result
        ClientFactory.getTeamBuildClient(teamRepository).addBuildResultContributions(buildResult,
                contributions.toArray(new IBuildResultContribution[contributions.size()]), monitor);

        return contributions;
    }

    private void saveCompilePackage(ICompilePackage compilePackage, ITeamRepository teamRepository, IProgressMonitor monitor)
            throws TeamRepositoryException {
        for (Object object : compilePackage.getSubCompilePackages()) {
            saveCompilePackage((ICompilePackage) object, teamRepository, monitor);
        }

        ClientFactory.getTeamBuildClient(teamRepository).save(compilePackage, monitor);
    }

    /**
     * Parse the given log files and create a
     * <code>BuildResultContribution</code> for each.
     * 
     * @param logFiles
     * @throws TeamRepositoryException
     */
    private void createBuildResultContributions(IBuildResult buildResult, String[] logFiles, String directory,
            ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {
        if (fComponentMapFile != null) {
            try {
                fComponentMap = new ComponentMapRegistry(fComponentMapFile.getAbsolutePath());
            } catch (Exception exception) {
                throw new TeamRepositoryException(exception);
            }
        }

        String simpleLogSuffix = fPdeBuild == true ? PDE_BUILD_SIMPLE_LOG_SUFFIX : SIMPLE_LOG_SUFFIX;

        for (int i = 0; i < logFiles.length; i++) {
            if (logFiles[i].endsWith(simpleLogSuffix)) {
                createSimpleCompileContribution(buildResult, logFiles[i], directory, teamRepository, monitor);
            } else {
                createCompileContribution(logFiles[i], directory, teamRepository);
            }
        }

        /*
         * Now go through each newly created contributions and recursively
         * update the error, warning and sourcefile counts in compile packages,
         * sources and contributions. Also set the build status.
         */
        for (IBuildResultContribution buildResultContribution : fCompileContributions.values()) {
            ICompileContribution compileContribution = (ICompileContribution) buildResultContribution.getExtendedContribution();
            compileContribution.setBuildResult(buildResult);
            for (Object object : compileContribution.getCompilePackages()) {
                ICompilePackage compilePackage = ((ICompilePackage) object);
                compilePackage.setBuildResult(buildResult);
                updateCompilePackage(buildResult, compilePackage);
                compileContribution.setErrorCount(compileContribution.getErrorCount() + compilePackage.getErrorCount());
                compileContribution.setWarningCount(compileContribution.getWarningCount()
                        + compilePackage.getWarningCount());
                compileContribution.setClassCount(compileContribution.getClassCount() + compilePackage.getClassCount());
            }

            BuildStatus currentStatus = buildResultContribution.getStatus();
            if (currentStatus != BuildStatus.ERROR) {
                BuildStatus status = getBuildStatus(compileContribution);
                if (status != BuildStatus.OK) {
                    buildResultContribution.setStatus(status);
                }
            }
        }
    }

    /*
     * Create a simple compile contribution that does not use any of the compile
     * contribution model objects other than the top-level CompileContribution.
     */
    private void createSimpleCompileContribution(IBuildResult buildResult, String logFile, String directory,
            ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {
        JdtSimpleCompileLogParser parser = new JdtSimpleCompileLogParser(logFile);
        try {
            parser.parse();
        } catch (IOException exception) {
            throw new TeamBuildException(exception);
        }

        ICompileContribution extendedContribution = (ICompileContribution) BuildItemFactory.createCompileContribution();
        extendedContribution.setWarningCount(parser.getWarningCount());
        extendedContribution.setErrorCount(parser.getErrorCount());
        extendedContribution.setBuildResult(buildResult);
        if (parser.getWarningCount() > 0) {
            extendedContribution.setWarnings(ContentUtil.stringToContent(teamRepository, parser.getWarnings(), monitor));
        }
        if (parser.getErrorCount() > 0) {
            extendedContribution.setErrors(ContentUtil.stringToContent(teamRepository, parser.getErrors(), monitor));
        }

        IBuildResultContribution contribution = BuildItemFactory.createBuildResultContribution();
        contribution.setLabel(getContributionLabel(logFile, directory));
        contribution.setStatus(getBuildStatus(parser));
        contribution.setImpactsPrimaryResult(true);
        contribution.setExtendedContribution(extendedContribution);
        contribution.setExtendedContributionTypeId(ICompileContribution.EXTENDED_CONTRIBUTION_TYPE_ID);

        if (fComponentName != null) {
            contribution.setComponentName(fComponentName);
        }

        fSimpleCompileContributions.add(contribution);
    }

    private IBuildResultContribution getCompileContribution(String componentName, String logFile, String directory) {
        if (componentName == null) {
            componentName = ""; //$NON-NLS-1$
        }

        if (fCompileContributions.get(componentName) == null) {
            ICompileContribution compileContribution = (ICompileContribution) BuildItemFactory.createCompileContribution();

            IBuildResultContribution contribution = BuildItemFactory.createBuildResultContribution();
            if (componentName.length() > 0) {
                contribution.setComponentName(componentName);
                contribution.setLabel(componentName);
            } else {
                contribution.setLabel(getContributionLabel(logFile, directory));
            }
            contribution.setImpactsPrimaryResult(true);
            contribution.setExtendedContribution(compileContribution);
            contribution.setExtendedContributionTypeId(ICompileContribution.EXTENDED_CONTRIBUTION_TYPE_ID);

            fCompileContributions.put(componentName, contribution);
        }

        return fCompileContributions.get(componentName);
    }

    /*
     * Create a full compile contribution using the complex model structure
     */
    private void createCompileContribution(String logFile, String directory, ITeamRepository teamRepository)
            throws TeamRepositoryException {
        JdtXmlCompileLogParser parser = new JdtXmlCompileLogParser(new File(logFile));

        try {
            parser.parse();
        } catch (SAXParseException exception) {
            throw new TeamBuildException(NLS.bind(PublishingMessages.JdtCompileLogPublisher_SAX_PARSING_EXCEPTION,
                    exception.getMessage(), exception.getLineNumber()), exception);
        } catch (Exception exception) {
            throw new TeamBuildException(exception);
        }

        JdtParserCompileData compileData = parser.getParsedData();

        /*
         * Check for a bad xml file.
         */
        if (compileData == null) {
            return;
        }

        String componentName = null;
        IBuildResultContribution buildResultContribution = null;
        ICompileContribution compileContribution = null;

        if (!fPdeBuild) {
            for (String packageName : compileData.getPackageNames()) {
                if (fComponentName != null) {
                    componentName = fComponentName;
                } else if (fComponentMap != null) {
                    componentName = fComponentMap.mapPackageToComponent(packageName);
                }

                buildResultContribution = getCompileContribution(componentName, logFile, directory);
                compileContribution = (ICompileContribution) buildResultContribution.getExtendedContribution();
                addSourcesToJavaPackage(compileContribution.getCompilePackages(), packageName,
                        compileData.getSourcesInPackage(packageName));
            }
        } else {
            /*
             * If component name is not set, get the component name for this pde
             * plugin from the component mapping file. If neither are set, we
             * will have a single, no-name component.
             */
            if (fComponentName != null) {
                componentName = fComponentName;
            } else if (fComponentMap != null) {
                for (String packageName : compileData.getPackageNames()) {
                    componentName = fComponentMap.mapPackageToComponent(packageName);

                    /*
                     * Stop looking as soon as a package maps to a component. We
                     * assume the entire plugin belongs to the same component.
                     */
                    if (componentName != null) {
                        break;
                    }
                }
            }

            buildResultContribution = getCompileContribution(componentName, logFile, directory);
            compileContribution = (ICompileContribution) buildResultContribution.getExtendedContribution();
            addSourcesToPlugin(compileContribution, fileNameToPluginName(logFile), compileData);
            if (buildResultContribution.getTimeTaken() == -1) {
                buildResultContribution.setTimeTaken(compileData.getCompileTime());
            } else {
                buildResultContribution.setTimeTaken(buildResultContribution.getTimeTaken()
                        + compileData.getCompileTime());
            }
        }
    }

    /**
     * Find the plugin name by first finding the plugins directory, then parsing
     * the plugin name from the subdirectory.
     * 
     * @param logFileName
     *            The full path to the logfile. Must be a real path to an
     *            existing logfile.
     */
    protected String fileNameToPluginName(String logFileName) {
        File logFile = new File(logFileName);
        File pluginNameDirectory = logFile.getParentFile();
        File pluginParentDirectory = pluginNameDirectory.getParentFile();

        String pluginsDirectoryName = "plugins"; //$NON-NLS-1$

        /*
         * Search the directory hierarchy until we find the plugins directory.
         */
        while (pluginParentDirectory != null && !pluginParentDirectory.getName().equalsIgnoreCase(pluginsDirectoryName)) {
            pluginNameDirectory = pluginParentDirectory;
            pluginParentDirectory = pluginParentDirectory.getParentFile();
        }

        /*
         * If we haven't found the plugins directory now, give up.
         */
        if (pluginParentDirectory == null || !pluginParentDirectory.getName().equalsIgnoreCase(pluginsDirectoryName)) {
            return logFileName;
        }

        String pluginName = pluginNameDirectory.getName();

        /*
         * Parse the plugin name from the plugin directory name.
         */
        if (pluginName.indexOf('_') != -1) {
            pluginName = pluginName.substring(0, pluginName.indexOf('_'));
        }

        return pluginName;
    }

    private void updateCompilePackage(IBuildResult buildResult, ICompilePackage compilePackage) {
        int errors = 0;
        int warnings = 0;
        int sourceFiles = 0;

        for (Object object : compilePackage.getSubCompilePackages()) {
            ICompilePackage subPackage = (ICompilePackage) object;
            updateCompilePackage(buildResult, subPackage);

            errors += subPackage.getErrorCount();
            warnings += subPackage.getWarningCount();
            sourceFiles += subPackage.getClassCount();
        }

        for (Object object : compilePackage.getCompileSources()) {
            ICompileSource source = (ICompileSource) object;
            source.setErrorCount(source.getErrors().size());
            source.setWarningCount(source.getWarnings().size());
            errors += source.getErrorCount();
            warnings += source.getWarningCount();
            sourceFiles++;
        }

        compilePackage.setErrorCount(errors);
        compilePackage.setWarningCount(warnings);
        compilePackage.setClassCount(sourceFiles);
        compilePackage.setBuildResult(buildResult);
    }

    /*
     * Create the plugin package if it doesn't already exist, then add the
     * java-package packages as children and add the sources to them.
     */
    private void addSourcesToPlugin(ICompileContribution compileContribution, String pluginName,
            JdtParserCompileData compileData) {
        ICompilePackage pluginPackage = null;

        for (Object object : compileContribution.getCompilePackages()) {
            ICompilePackage loopPackage = (ICompilePackage) object;
            if (loopPackage.getPackageType().equals(ICompilePackage.PACKAGE_TYPE_PLUGIN)
                    && loopPackage.getPackageName().equals(pluginName)) {
                pluginPackage = loopPackage;
                break;
            }
        }

        if (pluginPackage == null) {
            pluginPackage = BuildItemFactory.createCompilePackage();
            pluginPackage.setPackageType(ICompilePackage.PACKAGE_TYPE_PLUGIN);
            pluginPackage.setPackageName(pluginName);
            compileContribution.getCompilePackages().add(pluginPackage);
        }

        for (String packageName : compileData.getPackageNames()) {
            addSourcesToJavaPackage(pluginPackage.getSubCompilePackages(), packageName,
                    compileData.getSourcesInPackage(packageName));
        }
    }

    /*
     * Ensure that the java package exists, then add the specified sources.
     */
    private void addSourcesToJavaPackage(List compilePackages, String packageName,
            Collection<JdtParserSourceData> sourcesInPackage) {
        ICompilePackage compilePackage = null;
        for (Object object : compilePackages) {
            ICompilePackage loopCompilePackage = (ICompilePackage) object;
            if (loopCompilePackage.getPackageType().equals(ICompilePackage.PACKAGE_TYPE_JAVA_PACKAGE)
                    && loopCompilePackage.getPackageName().equals(packageName)) {
                compilePackage = loopCompilePackage;
                break;
            }
        }

        if (compilePackage == null) {
            compilePackage = BuildItemFactory.createCompilePackage();
            compilePackage.setPackageType(ICompilePackage.PACKAGE_TYPE_JAVA_PACKAGE);
            compilePackage.setPackageName(packageName);
            compilePackages.add(compilePackage);
        }

        addSources(compilePackage.getCompileSources(), sourcesInPackage);
    }

    /*
     * Convert the specified JdtParserSourceData objects to model object and add
     * them to the list.
     */
    private void addSources(List compileSources, Collection<JdtParserSourceData> sourcesInPackage) {
        for (JdtParserSourceData sourceData : sourcesInPackage) {
            ICompileSource source = BuildItemFactory.createCompileSource();
            source.setSourceName(sourceData.getClassName());
            source.setSourcePath(sourceData.getPath());
            source.setErrorCount(sourceData.getErrorCount());
            source.setWarningCount(sourceData.getWarningCount());

            for (JdtParserProblemData problemData : sourceData.getErrors()) {
                ICompileProblem error = createCompileProblem(problemData);
                source.getErrors().add(error);
            }

            for (JdtParserProblemData problemData : sourceData.getWarnings()) {
                ICompileProblem warning = createCompileProblem(problemData);
                source.getWarnings().add(warning);
            }

            for (String classfileName : sourceData.getClassfileNames()) {
                ICompileOutputFile outputFile = BuildItemFactory.createCompileOutputFile();
                outputFile.setOutputFilePath(classfileName);
                source.getCompileOutputFiles().add(outputFile);
            }

            compileSources.add(source);
        }
    }

    private ICompileProblem createCompileProblem(JdtParserProblemData problemData) {
        ICompileProblem problem = BuildItemFactory.createCompileProblem();
        problem.setLineNumber(problemData.getLineNumber());
        problem.setMessageText(problemData.getMessage());
        problem.setSourceText(problemData.getSource());
        problem.setSourceStart(problemData.getSourceStart());
        problem.setSourceEnd(problemData.getSourceEnd());
        problem.setProblemType(problemData.isError() ? CompileProblemType.ERROR : CompileProblemType.WARNING);
        return problem;
    }

    private BuildStatus getBuildStatus(JdtSimpleCompileLogParser parser) {
        if (parser.getErrorCount() == 0 && parser.getWarningCount() == 0) {
            return BuildStatus.OK;
        } else if (parser.getErrorCount() == 0) {
            return BuildStatus.WARNING;
        } else {
            return BuildStatus.ERROR;
        }
    }

    private BuildStatus getBuildStatus(ICompileContribution contribution) {
        if (contribution.getErrorCount() == 0 && contribution.getWarningCount() == 0) {
            return BuildStatus.OK;
        } else if (contribution.getErrorCount() == 0) {
            return BuildStatus.WARNING;
        } else {
            return BuildStatus.ERROR;
        }
    }

    /**
     * Get the contribution label by stripping off the log file suffix from the
     * log file name.
     * 
     * @param logFilePath
     * @param directory
     * @return a label for the contribution
     */
    private String getContributionLabel(String logFilePath, String directory) {
        String fileName = null;

        if (fPdeBuild) {
            fileName = fileNameToPluginName(logFilePath);
            if (!fileName.equals(logFilePath)) {
                return fileName;
            }
            fileName = null;
        } else {
            File file = new File(logFilePath);
            fileName = file.getName();

            /*
             * Even if pdeBuild was not set, still check for and strip off the
             * pde-specific suffixes for backwards compatibility.
             */
            if (fileName.indexOf(PDE_BUILD_SIMPLE_LOG_SUFFIX) != -1) {
                fileName = fileName.substring(0, fileName.lastIndexOf(PDE_BUILD_SIMPLE_LOG_SUFFIX));
            } else if (fileName.indexOf(PDE_BUILD_XML_LOG_SUFFIX) != -1) {
                fileName = fileName.substring(0, fileName.lastIndexOf(PDE_BUILD_XML_LOG_SUFFIX));
            } else if (fileName.indexOf(SIMPLE_LOG_SUFFIX) != -1) {
                fileName = fileName.substring(0, fileName.lastIndexOf(SIMPLE_LOG_SUFFIX));
            } else {
                fileName = fileName.substring(0, fileName.lastIndexOf(XML_LOG_SUFFIX));
            }
        }

        return ensureGoodFileName(fileName, directory, logFilePath);
    }

    protected String ensureGoodFileName(String fileName, String directory, String logFilePath) {
        if (fileName == null || fileName.equals("@dot")) { //$NON-NLS-1$
            // Avoid the useless @dot name. If we're stuck with a @dot file,
            // use the relative path from the given directory as the label.
            try {
                fileName = getFileNameFromPath(directory, logFilePath);
            } catch (IOException exception) {
                fileName = logFilePath.substring(directory.length());
            }
        }
        return fileName;
    }

    protected String getFileNameFromPath(String directory, String logFilePath) throws IOException {
        String fileName = null;

        if (directory != null) {
            // Canonical paths are better to deal with extra separators.
            String canonicalDir = new File(directory).getCanonicalPath();
            String canonicalFile = new File(logFilePath).getCanonicalPath();
            fileName = canonicalFile.substring(canonicalDir.length());
        } else {
            fileName = logFilePath;
        }

        return fileName;
    }

    /**
     * Recursively search the given directory for any files ending with log file
     * suffixes.
     * 
     * @param directory
     *            The directory to search.
     * @return An array of absolute path names of the files that were found.
     * @throws TeamBuildException
     */
    private String[] getLogFiles(File directory) {
        final ArrayList list = new ArrayList();

        // Find log files in this directory.
        File[] files = directory.listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                if (pathname.isFile()) {
                    if (fPdeBuild) {
                        return FileHelper.hasFileSuffix(pathname, PDE_BUILD_SIMPLE_LOG_SUFFIX, PDE_BUILD_XML_LOG_SUFFIX, PDE_BUILD_DOT_SIMPLE_LOG, PDE_BUILD_DOT_XML_LOG);
                    } else {
                        return FileHelper.hasFileSuffix(pathname, SIMPLE_LOG_SUFFIX, XML_LOG_SUFFIX);
                    }
                }
                return false;
            }
        });

        for (int i = 0; i < files.length; i++) {
            list.add(files[i].getAbsolutePath());
        }

        // Recurse into subdirectories.
        File[] directories = directory.listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                return pathname.isDirectory();
            }
        });

        for (int i = 0; i < directories.length; i++) {
            String[] filePaths = getLogFiles(directories[i]);
            for (int j = 0; j < filePaths.length; j++) {
                list.add(filePaths[j]);
            }
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

}
