/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.publishing;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.parser.MSBuildSimpleCompileLogParser;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Publisher for MSBuild compile logs. The publisher parses the logs in a given
 * directory and creates build result contributions of type
 * <code>CompileContribution</code>.
 */
public class MSBuildCompileLogPublisher {

    /**
     * Update the given build result with MSBuild compiler output from the given
     * log directory. The build result must exist in the repository specified by
     * <code>teamRepository</code>.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param logDirectoryPath
     *            The absolute path to a log directory. If a parser error occurs
     *            a <code>TeamBuildException</code> is thrown.
     * @param teamRepository
     * @param monitor
     *            A progress monitor. May be <code>null</code> if progress
     *            reporting is not desired.
     * @return The Collection of contributions of type
     *         <code>BuildResultContribution</code> that were published.
     * @throws TeamRepositoryException
     */
    public Collection publish(IBuildResult buildResult, String logDirectoryPath, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws TeamRepositoryException {

        List<IBuildResultContribution> compileContributions = createCompileContribution(buildResult, logDirectoryPath,
                teamRepository, monitor);

        if (compileContributions != null && compileContributions.size() >= 1) {
            // Save all the compile contribution auditables.
            for (IBuildResultContribution contribution : compileContributions) {
                ICompileContribution compileContribution = (ICompileContribution) contribution.getExtendedContribution();

                for (Object object : compileContribution.getCompilePackages()) {
                    saveCompilePackage((ICompilePackage) object, teamRepository, monitor);
                }

                ClientFactory.getTeamBuildClient(teamRepository).save(compileContribution, monitor);
            }

            // Add the contributions to the build result
            ClientFactory.getTeamBuildClient(teamRepository).addBuildResultContributions(buildResult,
                    compileContributions.toArray(new IBuildResultContribution[compileContributions.size()]), monitor);
        }

        return compileContributions;

    }

    /*
     * Create a simple compile contribution that does not use any of the compile
     * contribution model objects other than the top-level CompileContribution.
     */
    private List<IBuildResultContribution> createCompileContribution(IBuildResult buildResult, String logDirectory,
            ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {
        MSBuildSimpleCompileLogParser parser = new MSBuildSimpleCompileLogParser(logDirectory);
        try {
            parser.parse();
        } catch (IOException exception) {
            throw new TeamBuildException(exception);
        }

        List<IBuildResultContribution> compileContributions = new LinkedList<IBuildResultContribution>();

        if (parser.getProjectsData() != null) {
            for (Iterator<MSBuildSimpleCompileLogParser.ProjectData> pDataIter = parser.getProjectsData().iterator(); pDataIter.hasNext();) {
                MSBuildSimpleCompileLogParser.ProjectData pData = pDataIter.next();

                ICompileContribution extendedContribution = (ICompileContribution) BuildItemFactory.createCompileContribution();
                extendedContribution.setWarningCount(pData.getWarningCount());
                extendedContribution.setErrorCount(pData.getErrorCount());
                extendedContribution.setBuildResult(buildResult);
                if (pData.getWarningCount() > 0) {
                    extendedContribution.setWarnings(ContentUtil.stringToContent(teamRepository, pData.getWarnings(),
                            monitor));
                }
                if (pData.getErrorCount() > 0) {
                    extendedContribution.setErrors(ContentUtil.stringToContent(teamRepository, pData.getErrors(),
                            monitor));
                }

                IBuildResultContribution contribution = BuildItemFactory.createBuildResultContribution();
                contribution.setComponentName(pData.getSolutionName());
                contribution.setLabel(pData.getProjectName());
                contribution.setStatus(getBuildStatus(pData));
                contribution.setImpactsPrimaryResult(true);
                contribution.setExtendedContribution(extendedContribution);
                contribution.setExtendedContributionTypeId(ICompileContribution.EXTENDED_CONTRIBUTION_TYPE_ID);

                compileContributions.add(contribution);
            }
        }
        return compileContributions;
    }

    private void saveCompilePackage(ICompilePackage compilePackage, ITeamRepository teamRepository,
            IProgressMonitor monitor) throws TeamRepositoryException {
        for (Object object : compilePackage.getSubCompilePackages()) {
            saveCompilePackage((ICompilePackage) object, teamRepository, monitor);
        }

        ClientFactory.getTeamBuildClient(teamRepository).save(compilePackage, monitor);
    }

    private BuildStatus getBuildStatus(MSBuildSimpleCompileLogParser.ProjectData pData) {
        if (pData.getErrorCount() == 0 && pData.getWarningCount() == 0) {
            return BuildStatus.OK;
        } else if (pData.getErrorCount() == 0) {
            return BuildStatus.WARNING;
        } else {
            return BuildStatus.ERROR;
        }
    }
}
