/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.publishing;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.client.workitem.WorkItemHelper;
import com.ibm.team.build.internal.common.ChangeSetInfo;
import com.ibm.team.build.internal.common.helper.ItemHelper;
import com.ibm.team.build.internal.common.links.BuildLinkTypes;
import com.ibm.team.build.internal.publishing.workitem.WorkItemPublishStatus;
import com.ibm.team.build.publishing.workitem.FileTypes;
import com.ibm.team.filesystem.common.workitems.ILinkConstants;
import com.ibm.team.links.client.ILinkManager;
import com.ibm.team.links.common.IItemReference;
import com.ibm.team.links.common.ILink;
import com.ibm.team.links.common.ILinkCollection;
import com.ibm.team.links.common.ILinkQueryPage;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.factory.ILinkFactory;
import com.ibm.team.links.common.factory.IReferenceFactory;
import com.ibm.team.links.common.registry.ILinkTypeRegistry;
import com.ibm.team.links.internal.links.LinksPackage;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IHelperType;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.links.ChangeSetLinks;
import com.ibm.team.scm.common.providers.ProviderFactory;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.text.WorkItemTextUtilities;

/**
 * Updates a build result with the fixed work items for the build. Tags the
 * related work items with a link to the build result.
 */
public class WorkItemPublisher {
    private static final IWorkItemHandle[] EMPTY_WORKITEM_ARRAY = new IWorkItemHandle[0];

    /**
     * Updates <code>buildResult</code> with fixed work items derived from
     * <code>changeSetFilePath</code>. Adds a comment to each work item. The
     * comment links to the build result. The build result must exist in the
     * repository specified by <code>teamRepository</code>.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param inputFilePath
     *            The path to a file containing a list of change set uuids or
     *            work item ids.
     * @param inputFileType
     *            The type of the input file; may be CHANGESET or WORKITEM.
     * @param teamRepository
     *            The repository where the build result is saved.
     * @return The work items that were fixed.
     * @throws TeamRepositoryException
     */
    public WorkItemPublishStatus publish(IBuildResultHandle buildResult, String inputFilePath,
            ITeamRepository teamRepository, FileTypes inputFileType) throws TeamRepositoryException {
        try {
            if (inputFileType == null || inputFileType.equals(FileTypes.CHANGESET)) {
                IChangeSetHandle[] changeSetHandles = getChangeSetHandles(inputFilePath);
                return new WorkItemPublishStatus(publish(buildResult, changeSetHandles, teamRepository));
            } else {
                WorkItemPublishStatus status = getWorkItemHandles(inputFilePath, teamRepository);
                publish(buildResult, status.getWorkItemHandles(), teamRepository);
                return status;
            }

        } catch (IOException exception) {
            throw new TeamRepositoryException(exception);
        }
    }

    /**
     * Updates <code>buildResult</code> with fixed work items derived from the
     * given change sets.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param changeSetHandles
     *            The change sets.
     * @param teamRepository
     *            The repository where the build result is saved.
     * @return The work items that were fixed.
     * @throws TeamRepositoryException
     */
    public IWorkItemHandle[] publish(IBuildResultHandle buildResult, IChangeSetHandle[] changeSetHandles,
            ITeamRepository teamRepository) throws TeamRepositoryException {
        IWorkItemHandle[] workItemHandles = null;
        workItemHandles = getWorkItemHandles(changeSetHandles, teamRepository);
        publish(buildResult, workItemHandles, teamRepository);
        return workItemHandles;
    }

    /**
     * Updates <code>buildResult</code> with fixed work items specified in
     * <code>workItemHandles</code>. Adds a comment to each work item. The
     * comment links to the build result. The build result must exist in the
     * repository specified by <code>teamRepository</code>.
     * 
     * @param buildResult
     *            A build result from the repository. If the build result
     *            doesn't exist in the repository a
     *            <code>TeamBuildException</code> is thrown.
     * @param workItemHandles
     *            The list of work items to be marked as fixed in the
     *            <code>buildResult</code>
     * @param teamRepository
     *            The repository where the build result is saved.
     * @throws TeamRepositoryException
     */
    public void publish(IBuildResultHandle buildResult, IWorkItemHandle[] workItemHandles,
            ITeamRepository teamRepository) throws TeamRepositoryException {
        IBuildResult result = WorkItemHelper.setFixedInBuild(teamRepository, buildResult, workItemHandles);
        tagWorkItems(workItemHandles, result, teamRepository);
    }

    /**
     * Get the work item handles associated with the given change sets. The
     * returned array contains no duplicates.
     * 
     * @throws TeamRepositoryException
     */
    private IWorkItemHandle[] getWorkItemHandles(IChangeSetHandle[] changeSetHandles, ITeamRepository teamRepository)
            throws TeamRepositoryException {

        Hashtable workItemHandles = new Hashtable();

        if (isLinkTypeRegistered()) {

            for (int i = 0; i < changeSetHandles.length; i++) {
                IChangeSetHandle changeSetHandle = changeSetHandles[i];

                List<ILink> links = ChangeSetLinks.findLinks(
                        (ProviderFactory) teamRepository.getClientLibrary(ProviderFactory.class), changeSetHandle,
                        new String[] { ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID }, new NullProgressMonitor());

                for (ILink link : links) {
                    IReference targetRef = link.getTargetRef();

                    if (targetRef.isItemReference()
                            && ((IItemReference) targetRef).getReferencedItem() instanceof IWorkItemHandle) {

                        IWorkItemHandle workItemHandle = (IWorkItemHandle) ((IItemReference) targetRef).getReferencedItem();
                        // Need to use a HashTable in this way instead of a
                        // HashSet because we may have distinct work item
                        // handles, but
                        // they are pointing at the same uuidValue. We don't
                        // want to
                        // add handles with the same uuidValue.
                        workItemHandles.put(workItemHandle.getItemId().getUuidValue(), workItemHandle);
                    }
                }
            }

        }

        return (IWorkItemHandle[]) workItemHandles.values().toArray(new IWorkItemHandle[workItemHandles.size()]);
    }

    /**
     * Determine if the workitem/changeset link type is registered. If its not,
     * we won't be able to find any links.
     */
    protected boolean isLinkTypeRegistered() {
        return ILinkTypeRegistry.INSTANCE.isRegistered(ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID);
    }

    /**
     * Tag work items with a link to the given build result.
     * <p>
     * 
     * @param workItemHandles
     * @param buildResult
     * @throws TeamRepositoryException
     */
    private void tagWorkItems(IWorkItemHandle[] workItemHandles, IBuildResult buildResult,
            ITeamRepository teamRepository) throws TeamRepositoryException {

        ILinkManager linkManager = (ILinkManager) teamRepository.getClientLibrary(ILinkManager.class);
        List unlinkedWorkItems = findUnlinkedWorkItems(workItemHandles, buildResult, linkManager);
        List links = new ArrayList();

        List workItems = teamRepository.itemManager().fetchCompleteItems(unlinkedWorkItems, IItemManager.REFRESH, null);
        for (int i = 0; i < unlinkedWorkItems.size(); ++i) {
            IWorkItemHandle workItemHandle = (IWorkItemHandle) unlinkedWorkItems.get(i);
            IWorkItem workItem = (IWorkItem) workItems.get(i);
            links.add(createLink(teamRepository, workItemHandle, workItem, buildResult));
        }

        linkManager.saveLinks(links, null);
    }

    private List findUnlinkedWorkItems(IWorkItemHandle[] workItemHandles, IBuildResult buildResult,
            ILinkManager linkManager) throws TeamRepositoryException {

        ILinkQueryPage results = linkManager.findLinksByTarget(BuildLinkTypes.INCLUDED_WORK_ITEMS,
                IReferenceFactory.INSTANCE.createReferenceToItem(buildResult), null);

        ILinkCollection linkCollection = results.getAllLinksFromHereOn();

        Map unlinked = new HashMap();
        for (int i = 0; i < workItemHandles.length; i++) {
            unlinked.put(workItemHandles[i].getItemId(), workItemHandles[i]);
        }

        for (Iterator iter = linkCollection.iterator(); iter.hasNext();) {
            IReference reference = ((ILink) iter.next()).getSourceRef();
            if (reference.isItemReference() && ((IItemReference) reference).getReferencedItem() != null) {
                unlinked.remove(((IItemReference) reference).getReferencedItem().getItemId());
            }
        }

        return new ArrayList(unlinked.values());
    }

    private ILink createLink(ITeamRepository teamRepository, IWorkItemHandle workItemHandle, IWorkItem workItem, IBuildResult result)
            throws TeamRepositoryException {
        IBuildDefinition definition = (IBuildDefinition) teamRepository.itemManager().fetchCompleteItem(
                result.getBuildDefinition(), IItemManager.DEFAULT, null);

        IHelperType helperType = IHelperType.IRegistry.INSTANCE.getHelperType(
                LinksPackage.eINSTANCE.getReference().getName(), LinksPackage.eNS_URI);

        String sourceComment = ItemHelper.validateStringAttributeLength(definition.getId() + " " + result.getLabel(), //$NON-NLS-1$ 
                helperType, LinksPackage.eINSTANCE.getReference_Comment().getName());

        IReference source = IReferenceFactory.INSTANCE.createReferenceToItem(result, sourceComment);

        IReference target;
        if (workItem == null) {
            // work item is missing or invisible, so create a link but provide no comment
            target = IReferenceFactory.INSTANCE.createReferenceToItem(workItemHandle);
        } else {
            String targetComment = ItemHelper.validateStringAttributeLength(
                    WorkItemTextUtilities.getWorkItemText(workItem), helperType,
                    LinksPackage.eINSTANCE.getReference_Comment().getName());
            target = IReferenceFactory.INSTANCE.createReferenceToItem(workItemHandle, targetComment);
        }

        return ILinkFactory.INSTANCE.createLink(BuildLinkTypes.INCLUDED_WORK_ITEMS, source, target);
    }

    /**
     * Get the work item handles from the file at <code>inputFilePath</code>.
     * 
     * @param inputFilePath
     *            The absolute path to a file that contains work item ids, one
     *            per line.
     * @return Array of {@link IWorkItemHandle}, may be empty but not
     *         <code>null</code>.
     * 
     * @throws IOException
     *             if the file at <code>inputFilePath</code> cannot be read.
     */
    private WorkItemPublishStatus getWorkItemHandles(String inputFilePath, ITeamRepository teamRepository)
            throws TeamBuildException, TeamRepositoryException, IOException {

        List workItemIds = new ArrayList<Integer>();
        BufferedReader reader = new BufferedReader(new FileReader(inputFilePath));
        String workItemId = reader.readLine();
        while (workItemId != null) {
            workItemId = workItemId.trim();
            try {
                // watch out for blank lines -- do not try to add them as work
                // items
                if (!workItemId.equals("")) { //$NON-NLS-1$
                    workItemIds.add(Integer.valueOf(workItemId));
                }
            } catch (NumberFormatException exception) {
                throw new TeamBuildException(NLS.bind(PublishingMessages.WorkItemPublisher_BAD_WORKITEM_ID, workItemId,
                        inputFilePath), exception);
            }
            workItemId = reader.readLine();
        }
        
        IWorkItemHandle[] foundWorkItems = getWorkItemHandles(workItemIds, teamRepository);
        
        return new WorkItemPublishStatus(foundWorkItems, workItemIds.size());
    }

    private IWorkItemHandle[] getWorkItemHandles(List<Integer> workItemIds, ITeamRepository teamRepository)
            throws TeamRepositoryException {
        if (workItemIds.size() == 0) {
            return EMPTY_WORKITEM_ARRAY;
        }
        IWorkItemClient workItemClient = (IWorkItemClient) teamRepository.getClientLibrary(IWorkItemClient.class);

        List<IWorkItemHandle> handles = workItemClient.findWorkItemsById(workItemIds, null);

        return handles.toArray(new IWorkItemHandle[handles.size()]);
    }

    /**
     * Get the change set handles from the file at
     * <code>changeSetFilePath</code>.
     * 
     * @param changeSetFilePath
     *            The full path to the file output from the
     *            {@link com.ibm.team.build.ant.task.TeamAcceptTask}.
     * @return Array of {@link IChangeSetHandle}, may be empty but never
     *         <code>null</code>.
     * @throws IOException
     *             if there is a problem creating a {@link File} from the
     *             <code>changeSetFilePath</code>.
     * @throws TeamRepositoryException
     *             if there is a problem contacting the repository.
     */
    private IChangeSetHandle[] getChangeSetHandles(String changeSetFilePath) throws IOException,
            TeamRepositoryException {
        ChangeSetInfo changeSetInfo = new ChangeSetInfo(new File(changeSetFilePath));
        IChangeSetHandle[] changeSetHandles = changeSetInfo.getChangeSets();
        return changeSetHandles;
    }
}
