/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import com.ibm.team.scm.common.IBaseline;
import com.ibm.team.scm.common.IBaselineHandle;
import com.ibm.team.scm.common.IBaselineSet;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IComponentHandle;

/**
 * This utility class provides back information about changes to the workspace
 * that would be made (or have been made).
 */
public class AcceptReport {
    private Collection fComponentAdds;
    private Collection fComponentRemovals;
    private List fAcceptChangeSets;
    private List fDiscardChangeSets;
    private HashMap fComponentBaselines;
    private IBaselineSet fSnapshot;

    /**
     * Accept report describes what the differences are between the build
     * workspace and the source workspace.
     */
    public AcceptReport() {
        fComponentAdds = new ArrayList();
        fComponentRemovals = new ArrayList();
        fAcceptChangeSets = new ArrayList();
        fDiscardChangeSets = new ArrayList();
        fComponentBaselines = new HashMap();
    }

    /**
     * @return The new components added (or would be added if checking only) to
     *         the build workspace from the source collaborations
     */
    public IComponentHandle[] getComponentAdds() {
        return (IComponentHandle[]) fComponentAdds.toArray(new IComponentHandle[fComponentAdds.size()]);
    }

    /**
     * @return The changesets that were accepted (or would be accepted if
     *         checking only) into the build workspace from the source
     *         collaborations
     */
    public IChangeSetHandle[] getAcceptChangeSets() {
        return (IChangeSetHandle[]) fAcceptChangeSets.toArray(new IChangeSetHandle[fAcceptChangeSets.size()]);
    }

    /**
     * @return The baselines used for each component in the build workspace.
     *         This will be an empty array if only checking was done (i.e. we
     *         did not create/change/identify any baselines)
     */
    public IBaselineHandle[] getComponentBaselines() {
        return (IBaselineHandle[]) fComponentBaselines.values().toArray(new IBaselineHandle[fComponentBaselines.size()]);
    }
    
    /**
     * @return The snapshot created for this build, or <code>null</code> if
     * a snapshot was not created.
     */
    public IBaselineSet getSnapshot() {
        return fSnapshot;
    }

    /**
     * Returns the baseline configured for the component after an accept of all 
     * changes has taken place.  The baseline includes all changes in the component's 
     * configuration.
     * 
     * @param componentHandle The component whose baseline we want to know
     * @return the baseline of the component identified. This will be
     *         <code>null</code> if the component does not have a baseline
     *         recorded for it, or only checking was done (i.e. we did not
     *         create/change/identify any baselines)
     */
    public IBaselineHandle getBaselineForComponent(IComponentHandle componentHandle) {
        return (IBaselineHandle) fComponentBaselines.get(componentHandle.getItemId());
    }

    /**
     * @return The components that were removed (or would be removed if checking only)
     *         from the build workspace from the source collaborations.  
     *         Never <code>null</code>
     */
   
     public IComponentHandle[] getComponentRemovals() {
        return (IComponentHandle[]) fComponentRemovals.toArray(new IComponentHandle[fComponentRemovals.size()]);
    }

    /**
     * @return The changesets that were removed from the build workspace (or
     *         would be removed if checking only) in order to make its
     *         configuration the same as the source collaboration.
     */
    public IChangeSetHandle[] getDiscardChangeSets() {
        return (IChangeSetHandle[]) fDiscardChangeSets.toArray(new IChangeSetHandle[fDiscardChangeSets.size()]);
    }
    
    /**
     * @return The count of all of the accepts, discards, adds and removes.
     */
    public int getChangesAcceptedCount(){
        int changes = fAcceptChangeSets.size();
        changes += fDiscardChangeSets.size();
        changes += fComponentAdds.size();
        changes += fComponentRemovals.size();
        return changes;
    }

    void componentAdd(IComponentHandle componentHandle) {
        fComponentAdds.add(componentHandle);
    }
    
    /* package */ void componentRemoval(IComponentHandle componentHandle) {
        fComponentRemovals.add(componentHandle);
    }

    void addAcceptChangeSets(List changeSets) {
        fAcceptChangeSets.addAll(changeSets);
    }

    /* package */void addDiscardChangeSets(List changeSets) {
        fDiscardChangeSets.addAll(changeSets);
    }

    /* package */void addComponentBaseline(IBaseline baseline) {

        fComponentBaselines.put(baseline.getComponent().getItemId(), baseline.getItemHandle());
    }

    /* package */void addSnapshot(IBaselineSet snapshot) {
        fSnapshot = snapshot;
    }
}
