/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import java.util.HashMap;
import java.util.Map;

import com.ibm.team.repository.common.UUID;

/**
 * This utility class is used to store the details of a successful auto-deliver.
 * Information includes:
 * <ul>
 * <li>components which were ADDED to the deliver target</li>
 * <li>components which were REPLACED in the deliver target</li>
 * <li>components which were REMOVED from the deliver target</li>
 * <li>components which were unchanged because they were already at the specific baseline</li>
 * </ul>
 * 
 * @since 0.8.1
 */
public class AutoDeliverReport {
    private UUID fSnapshotUUID;
    private String fSnapshotName;
    private UUID fWorkspaceUUID;
    private String fWorkspaceName;
    private UUID fDeliverTargetUUID;
    private String fDeliverTargetName;
    private Map<UUID, String> fComponentAdds;
    private Map<UUID, String> fComponentReplaces;
    private Map<UUID, String> fTargetComponentsAlreadyAtBaseline;
    private Map<UUID, String> fComponentRemovals;
    private Map<UUID, UUID> fComponentToBaseline;
    private Map<UUID, String> fBaselineNames;

    // Keep track if a StaleDataException was thrown during the
    // applyComponentOperations() method in
    // SourceControlUtility.autoDeliverComponentsToTarget().
    // This is needed so that the AutoDeliverPostBuildParticipant can log this
    // issue in a more informative manner.
    private boolean fIsStaleDataExceptionThrownDuringAutoDeliver;

    /**
     * Auto-deliver report which describes which component operations were
     * applied to the deliver target (ADD/REPLACE/REMOVE).
     */
    public AutoDeliverReport() {
        fSnapshotUUID = null;
        fSnapshotName = null;
        fWorkspaceUUID = null;
        fWorkspaceName = null;
        fDeliverTargetUUID = null;
        fDeliverTargetName = null;
        fComponentAdds = new HashMap<UUID, String>();
        fComponentReplaces = new HashMap<UUID, String>();
        fTargetComponentsAlreadyAtBaseline = new HashMap<UUID, String>();
        fComponentRemovals = new HashMap<UUID, String>();
        fComponentToBaseline = new HashMap<UUID, UUID>();
        fBaselineNames = new HashMap<UUID, String>();
    }

    /**
     * @return The UUID of the snapshot to deliver.
     */
    public UUID getSnapshotUUID() {
        return fSnapshotUUID;
    }

    /**
     * @param snapshotUUID
     *            The UUID of the snapshot.
     */
    public void setSnapshotUUID(UUID snapshotUUID) {
        fSnapshotUUID = snapshotUUID;
    }

    /**
     * @return The snapshot name.
     */
    public String getSnapshotName() {
        return fSnapshotName;
    }

    /**
     * @param snapshotName
     *            The snapshot name.
     */
    public void setSnapshotName(String snapshotName) {
        fSnapshotName = snapshotName;
    }

    /**
     * @return The UUID of the build workspace.
     */
    public UUID getWorkspaceUUID() {
        return fWorkspaceUUID;
    }

    /**
     * @param workspaceUUID
     *            The UUID of the build workspace.
     */
    public void setWorkspaceUUID(UUID workspaceUUID) {
        fWorkspaceUUID = workspaceUUID;
    }

    /**
     * @return The build workspace name.
     */
    public String getWorkspaceName() {
        return fWorkspaceName;
    }

    /**
     * @param workspaceName
     *            The build workspace name.
     */
    public void setWorkspaceName(String workspaceName) {
        fWorkspaceName = workspaceName;
    }

    /**
     * @return The UUID of the deliver target.
     */
    public UUID getDeliverTargetUUID() {
        return fDeliverTargetUUID;
    }

    /**
     * @param deliverTargetUUID
     *            The UUID of the deliver target.
     */
    public void setDeliverTargetUUID(UUID deliverTargetUUID) {
        fDeliverTargetUUID = deliverTargetUUID;
    }

    /**
     * @return The name of the deliver target.
     */
    public String getDeliverTargetName() {
        return fDeliverTargetName;
    }

    /**
     * @param deliverTargetName
     *            The name of the deliver target.
     */
    public void setDeliverTargetName(String deliverTargetName) {
        fDeliverTargetName = deliverTargetName;
    }

    /**
     * @return The components which were added to the deliver target.
     */
    public Map<UUID, String> getComponentAdds() {
        return fComponentAdds;
    }

    /**
     * @param uuid
     *            The UUID of the component that was added.
     * @param componentName
     *            The name of the component that was added.
     */
    public void addComponentAdd(UUID uuid, String componentName) {
        fComponentAdds.put(uuid, componentName);
    }

    /**
     * @return The components which were replaced in the deliver target.
     */
    public Map<UUID, String> getComponentReplaces() {
        return fComponentReplaces;
    }

    /**
     * @param uuid
     *            The UUID of the component that was replaced.
     * @param componentName
     *            The name of the component that was replaced.
     */
    public void addComponentReplace(UUID uuid, String componentName) {
        fComponentReplaces.put(uuid, componentName);
    }

    /**
     * @return The components which were not replaced in the deliver target
     *         because they were already at the specific baseline.
     */
    public Map<UUID, String> getTargetComponentsAlreadyAtBaseline() {
        return fTargetComponentsAlreadyAtBaseline;
    }

    /**
     * @param uuid
     *            The UUID of the component that was not replaced (because it
     *            was already at the specific baseline).
     * @param componentName
     *            The name of the component that was not replaced (because it
     *            was already at the specific baseline).
     */
    public void addTargetComponentAlreadyAtBaseline(UUID uuid, String componentName) {
        fTargetComponentsAlreadyAtBaseline.put(uuid, componentName);
    }

    /**
     * @return The components which were removed from the deliver target.
     */
    public Map<UUID, String> getComponentRemovals() {
        return fComponentRemovals;
    }

    /**
     * @param uuid
     *            The UUID of the component that was removed.
     * @param componentName
     *            The name of the component that was removed.
     */
    public void addComponentRemoval(UUID uuid, String componentName) {
        fComponentRemovals.put(uuid, componentName);
    }

    /**
     * @return <code>true</code> if a StaleDataException was thrown during
     *         deliver.
     */
    public boolean getIsStaleDataExceptionThrownDuringAutoDeliver() {
        return fIsStaleDataExceptionThrownDuringAutoDeliver;
    }

    /**
     * @param isStaleDataExceptionThrownDuringAutoDeliver
     *            <code>true</code> if a StaleDataException was thrown during
     *            auto-deliver.
     */
    public void setIsStaleDataExceptionThrownDuringAutoDeliver(boolean isStaleDataExceptionThrownDuringAutoDeliver) {
        fIsStaleDataExceptionThrownDuringAutoDeliver = isStaleDataExceptionThrownDuringAutoDeliver;
    }

    /**
     * @param componentID
     *            The UUID of the component.
     * 
     * @return The ID of the baseline that was added/replaced/unchanged for the
     *         given component.
     */
    public UUID getComponentToBaseline(UUID componentID) {
        return fComponentToBaseline.get(componentID);
    }

    /**
     * @param componentID
     *            The UUID of the component.
     * @param baselineID
     *            The UUID of the baseline corresponding to the component.
     */
    public void addComponentToBaseline(UUID componentID, UUID baselineID) {
        fComponentToBaseline.put(componentID, baselineID);
    }

    /**
     * @param baselineID
     *            The UUID of the baseline
     * 
     * @return The name of the baseline.
     */
    public String getBaselineName(UUID baselineID) {
        return fBaselineNames.get(baselineID);
    }

    /**
     * @param baselineID
     *            The UUID of the baseline.
     * @param baselineName
     *            The name of the baseline.
     */
    public void addBaselineName(UUID baselineID, String baselineName) {
        fBaselineNames.put(baselineID, baselineName);
    }

}
