/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.client.SCMPlatform;
import com.ibm.team.scm.common.IWorkspace;
import com.ibm.team.scm.common.IWorkspaceHandle;


/**
 * Class that provides the information needed to identify a build workspace
 */
public class BuildWorkspaceDescriptor {
    
    private final String repositoryUri;
    private final String workspaceId;
    private final String workspaceName;
    
    private IWorkspace workspace;
    
    /**
     * Create a workspace descriptor using the provided details. A team repository
     * must be provided along with either a workspace id or a workspace name.
     * If both a name and an id is provided, only the id will be used to fetch additional
     * information.
     * @param teamRepository a team repository (not <code>null</code>)
     * @param workspaceUUID a workspace id (may be <code>null</code> if a name is provided)
     * @param workspaceName a workspace name (may be <code>null</code> if an id is provided)
     */
    public BuildWorkspaceDescriptor(ITeamRepository teamRepository, String workspaceUUID, String workspaceName) {
        Assert.isNotNull(teamRepository);
        Assert.isTrue(workspaceUUID != null || workspaceName != null);
        this.repositoryUri = teamRepository.getRepositoryURI();
        this.workspaceId = workspaceUUID;
        this.workspaceName = workspaceName;
    }

    /**
     * Create a workspace descriptor using the provided details.
     * @param teamRepository a team repository (not <code>null</code>)
     * @param workspace a workspace  (not <code>null</code>)
     */
    public BuildWorkspaceDescriptor(ITeamRepository teamRepository, IWorkspace workspace) {
        Assert.isNotNull(teamRepository);
        Assert.isNotNull(workspace);
        this.repositoryUri = teamRepository.getRepositoryURI();
        this.workspaceId = workspace.getItemId().getUuidValue();
        this.workspace = workspace;
        this.workspaceName = workspace.getName();
    }

    /**
     * Return the workspace that this descriptor represents.
     * @param repositoryManager a repository manager
     * @param monitor a progress monitor
     * @return the workspace that this descriptor represents (not <code>null</code>)
     * @throws TeamRepositoryException if the workspace represented by this descriptor could not be found or fetched
     */
    public IWorkspace getWorkspace(RepositoryManager repositoryManager, IProgressMonitor monitor) throws TeamRepositoryException {
        if (workspace == null) {
            workspace = fetchWorkspace(repositoryManager, monitor);
        }
        return workspace;
    }
    
    private IWorkspace fetchWorkspace(RepositoryManager repositoryManager, IProgressMonitor monitor)
            throws TeamRepositoryException {
        ITeamRepository teamRepository = getRepository(repositoryManager);
        if (workspaceId != null) {
            return fetchWorkspace(teamRepository, getWorkspaceHandle(), monitor);
        } else {
            return fetchWorkspace(teamRepository, workspaceName, monitor);
        }
    }

    private ITeamRepository getRepository(RepositoryManager repositoryManager) throws TeamRepositoryException {
        return repositoryManager.getRepository(repositoryUri, true);
    }

    private IWorkspace fetchWorkspace(ITeamRepository teamRepository, String name, IProgressMonitor monitor) throws TeamRepositoryException {
        IWorkspace ws = SourceControlUtility.getWorkspace(teamRepository, name, monitor);
        if (ws == null) {
            throw new TeamBuildException(NLS.bind(ScmMessages.BuildWorkspaceDescriptor_WORKSPACE_WITH_NAME_NOT_FOUND, name));
        }
        return ws;
    }

    private IWorkspace fetchWorkspace(ITeamRepository teamRepository, IWorkspaceHandle workspaceHandle,
            IProgressMonitor monitor) throws TeamRepositoryException, TeamBuildException {
        IItem item;
        try {
            item = teamRepository.itemManager().fetchCompleteItem(workspaceHandle, IItemManager.REFRESH, monitor);
        } catch (ItemNotFoundException exception) {
            throw new TeamBuildException(NLS.bind(
                    ScmMessages.BuildWorkspaceDescriptor_WORKSPACE_WITH_UUID_NOT_FOUND, workspaceId), exception);
        } catch (PermissionDeniedException exception) {
            throw new TeamBuildException(NLS.bind(
                    ScmMessages.BuildWorkspaceDescriptor_USER_LACKS_READ_ACCESS,
                    teamRepository.getUserId(), workspaceId));
        }
   
        if (!(item instanceof IWorkspaceHandle)) {
            throw new TeamBuildException(NLS.bind(
                    ScmMessages.BuildWorkspaceDescriptor_ITEM_NOT_A_WORKSPACE, workspaceId,
                    item.getItemType().getName()));
        }
   
        return  (IWorkspace) item;
    }

    /**
     * Return the workspace handle for the workspace represented by this descriptor.
     * If the workspace descriptor was created using a name but not
     * an id then {@link #getWorkspace(RepositoryManager, IProgressMonitor)} must be called first.
     * @return the workspace handle for the workspace represented by this descriptor
     * @throws TeamBuildException if the handle could not be created
     */
    public IWorkspaceHandle getWorkspaceHandle() throws TeamBuildException {
        if (workspace != null) {
            return workspace;
        }
        if (workspaceId != null) {
            try {
                UUID workspaceUuid = UUID.valueOf(workspaceId);
                return (IWorkspaceHandle) IWorkspace.ITEM_TYPE.createItemHandle(workspaceUuid, null);
            } catch (IllegalArgumentException exception) {
                throw new TeamBuildException(NLS.bind(ScmMessages.BuildWorkspaceDescriptor_UUID_INVALID,
                        workspaceId), exception);
            }
        }
        throw new IllegalStateException("getWorkpace must be called for descriptors created using a name"); //$NON-NLS-1$
    }
    
    /**
     * Return the workspace connection for this workspace.
     * @param manager a repository manager
     * @param refresh when <code>true</code> the connection is to be refreshed before it is returned
     * @param monitor a progress monitor
     * @return the workspace connection for this workspace
     * @throws TeamRepositoryException
     */
    public IWorkspaceConnection getConnection(RepositoryManager manager, boolean refresh, IProgressMonitor monitor) throws TeamRepositoryException {
        SubMonitor progress = SubMonitor.convert(monitor, refresh ? 3 : 2);
        IWorkspaceConnection workspaceConnection = SCMPlatform.getWorkspaceManager(getRepository(manager)).getWorkspaceConnection(
                getWorkspace(manager, progress.newChild(1)), progress.newChild(1));
        if (refresh) {
            workspaceConnection.refresh(progress.newChild(1));
        }
        return workspaceConnection;
    }
    

}
