/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import static com.ibm.team.filesystem.client.rest.IFilesystemRestClient.ADD_COMPONENT;
import static com.ibm.team.filesystem.client.rest.IFilesystemRestClient.CHANGELOG_ENTRY_TYPE_COMPONENT;
import static com.ibm.team.filesystem.client.rest.IFilesystemRestClient.CHANGELOG_ENTRY_TYPE_DIRECTION;
import static com.ibm.team.filesystem.client.rest.IFilesystemRestClient.FLOW_DIRECTION_INCOMING;
import static com.ibm.team.filesystem.client.rest.IFilesystemRestClient.REMOVE_COMPONENT;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.osgi.util.NLS;

import com.ibm.team.filesystem.client.internal.snapshot.BaselineSetId;
import com.ibm.team.filesystem.client.internal.snapshot.SnapshotId;
import com.ibm.team.filesystem.client.internal.snapshot.SnapshotSyncReport;
import com.ibm.team.filesystem.common.changemodel.IPathResolver;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogChangeSetEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogComponentEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogDirectionEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogVersionableEntry2DTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogVersionableEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.changelog.ChangeLogWorkItemEntryDTO;
import com.ibm.team.filesystem.common.internal.rest.client.core.ContributorDTO;
import com.ibm.team.filesystem.rcp.core.internal.changelog.BaseChangeLogEntryVisitor;
import com.ibm.team.filesystem.rcp.core.internal.changelog.ChangeLogCustomizer;
import com.ibm.team.filesystem.rcp.core.internal.changelog.ChangeLogEntryVisitor;
import com.ibm.team.filesystem.rcp.core.internal.changelog.ChangeLogStreamOutput;
import com.ibm.team.filesystem.rcp.core.internal.changelog.GenerateChangeLogOperation;
import com.ibm.team.filesystem.rcp.core.internal.changelog.IChangeLogOutput;
import com.ibm.team.filesystem.rcp.core.internal.changes.model.FallbackPathResolver;
import com.ibm.team.filesystem.rcp.core.internal.changes.model.NullPathResolver;
import com.ibm.team.filesystem.rcp.core.internal.changes.model.SnapshotPathResolver;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.common.IBaselineSetHandle;

/**
 * This utility class centralizes all the information required for a change log.
 * It is capable of then writing the change log to files in a human readable
 * way.
 */
public class ChangeLogReport {

    /**
     * 
     */
    public static final String DATE_FORMAT_STRING = "MMM d, yyyy hh:mm aaa";  //$NON-NLS-1$
    private ChangeLogEntryDTO fChangeLog;

    /**
     * This utility class centralizes all the information required for a change
     * log. It is capable of then writing the change log to files in a human
     * readable way.
     * 
     * @param teamRepository
     *            The Team Repository in which the compare takes place.
     * @param currentSnapshot
     *            The "current" snapshot's baseline set handle.
     * @param beforeSnapshot
     *            The "before" snapshot's baseline set handle.
     * @param showFileChanges
     *            If <code>true</code>, file level changes should be shown in
     *            the change log.
     * @param showVersionIdentifiers 
     *            If <code>true</code>, the version identifier will be shown in 
     *            the change log. showFileChanges must also be enabled for the 
     *            version identifiers to show up.
     * @throws TeamRepositoryException
     */
    public ChangeLogReport(ITeamRepository teamRepository, IBaselineSetHandle currentSnapshot,
            IBaselineSetHandle beforeSnapshot, boolean showFileChanges, boolean showVersionIdentifiers) throws TeamRepositoryException {

        // sets up the components required for the change log operation
        SnapshotId currentId = BaselineSetId.create(teamRepository, currentSnapshot);
        SnapshotId beforeId = BaselineSetId.create(teamRepository, beforeSnapshot);
        SnapshotSyncReport syncReport = SnapshotSyncReport.compare(currentId.getSnapshot(null),
                beforeId.getSnapshot(null), null, null);
        IPathResolver pathResolver = getPathResolver(currentId, beforeId);
        ChangeLogCustomizer customizer = getCustomizer(showFileChanges, showVersionIdentifiers);

        // creates and executes the change log operation
        GenerateChangeLogOperation operation = new GenerateChangeLogOperation();
        operation.setChangeLogRequest(teamRepository, syncReport, pathResolver, customizer);
        fChangeLog = operation.run(null);
    }

    private IPathResolver getPathResolver(SnapshotId currentId, SnapshotId beforeId) {
        List<IPathResolver> pathResolvers = new LinkedList<IPathResolver>();
        pathResolvers.add(SnapshotPathResolver.create(currentId));
        pathResolvers.add(SnapshotPathResolver.create(beforeId));
        pathResolvers.add(NullPathResolver.create());
        return new FallbackPathResolver(pathResolvers);
    }

    private ChangeLogCustomizer getCustomizer(boolean showFileChanges, boolean showVersionIdentifiers) {
        ChangeLogCustomizer customizer = new ChangeLogCustomizer();
        customizer.setPruneUnchangedComponents(true);
        customizer.setPruneEmptyDirections(true);
        customizer.setIncludeDirection(true);
        customizer.setIncludeWorkItems(true);
        customizer.setIncludePaths(showFileChanges);
        customizer.setIncludeVersionIdentifiers(showVersionIdentifiers);
        return customizer;
    }

    /**
     * Prints the change log to the file specified in the parameters.
     * 
     * @param changeLogFile
     *            The file to which the change log should be written. Must exist
     *            and be writable.
     * @throws IOException 
     */
    public void outputToFile(File changeLogFile) throws IOException {
        if (changeLogFile.exists()) {
            changeLogFile.delete();
        } else {
            File parentDir = changeLogFile.getParentFile();
            if (parentDir != null && !parentDir.exists()) {
                parentDir.mkdirs();
            }
        }
        
        PrintStream printStream = new PrintStream(new BufferedOutputStream(new FileOutputStream(changeLogFile)));
        try {
            ChangeLogStreamOutput output = new ChangeLogStreamOutput(printStream);
            AntTaskChangeLogEntryVisitor visitor = new AntTaskChangeLogEntryVisitor(output);
            ChangeLogEntryVisitor.acceptInto(fChangeLog, visitor);
            visitor.printComponentSections();
        } finally {
            printStream.close();
        }
    }

    /**
     * Formats the given date.  Visibility is protected for testing purposes only.
     */
    protected String formatDate(Date date) {
        SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT_STRING);

        return NLS.bind(ScmMessages.ChangeLogReport_CHANGE_SET_OUTPUT_DATE_FORMATTING, formatter.format(date));
    }

    private class AntTaskChangeLogEntryVisitor extends BaseChangeLogEntryVisitor {
        private boolean fIsIncoming;
        private boolean fAddedWritten;
        private boolean fRemovedWritten;
        private List<String> fAddedComponents = new LinkedList<String>();
        private List<String> fRemovedComponents = new LinkedList<String>();

        public AntTaskChangeLogEntryVisitor(IChangeLogOutput output) {
            super();
            setOutput(output);
            setShowChangeSetWorkItems(true);
            fIsIncoming = false;
            fAddedWritten = false;
            fRemovedWritten = false;
        }

        @Override
        protected void visitDirection(ChangeLogEntryDTO parent, ChangeLogDirectionEntryDTO dto) {
            fIsIncoming = !FLOW_DIRECTION_INCOMING.equals(dto.getFlowDirection());

        }

        @Override
        protected void visitComponent(ChangeLogEntryDTO parent, ChangeLogComponentEntryDTO dto) {
            String output;
            if (ADD_COMPONENT.equals(dto.getChangeType())) {
                if (fIsIncoming) {
                    fRemovedComponents.add(findComponentName(dto));
                } else {
                    fAddedComponents.add(findComponentName(dto));
                }
            } else if (REMOVE_COMPONENT.equals(dto.getChangeType())) {
                if (fIsIncoming) {
                    fAddedComponents.add(findComponentName(dto));
                } else {
                    fRemovedComponents.add(findComponentName(dto));
                }
            } else {
                output = NLS.bind(ScmMessages.EclipseChangeLogEntryVisitor_COMPONENT, findComponentName(dto));

                out.writeLine(output);
            }
        }

        @Override
        protected void visitVersionable(ChangeLogEntryDTO parent, ChangeLogVersionableEntryDTO dto) {
            if (dto instanceof ChangeLogVersionableEntry2DTO) {
                ChangeLogVersionableEntry2DTO dto2 = (ChangeLogVersionableEntry2DTO) dto;
                if (dto2.getShortVersionId() != null) {
                    out.writeLine(NLS.bind(ScmMessages.ChangeLogReport_VERSIONABLE_NAME_AND_IDENTIFIER, findVersionableName(dto2), dto2.getShortVersionId()));
                } else {
                    out.writeLine(findVersionableName(dto2));
                }
            } else {
                out.writeLine(findVersionableName(dto));
            }
        }

        @Override
        protected void visitChangeSet(ChangeLogEntryDTO parent, ChangeLogChangeSetEntryDTO dto) {
            String output = sanitizeText(dto.getEntryName());
            if (dto.getWorkItems().size() > 0) {
                String wiText = findMultiWorkItemName((List<ChangeLogWorkItemEntryDTO>) dto.getWorkItems());

                if (output == null || "".equals(output)) { //$NON-NLS-1$
                    output = wiText;
                } else {
                    output = NLS.bind(ScmMessages.EclipseChangeLogEntryVisitor_WORKITEM_TEXT_AND_CHANGESET_COMMENT,
                            wiText, sanitizeText(output));
                }
            } else {
                if (output == null || "".equals(output)) { //$NON-NLS-1$
                    output = ScmMessages.EclipseChangeLogEntryVisitor_MISSING_CHANGESET_COMMENT;
                }
            }

            if (dto.getCreationDate() != 0) {
                output = NLS.bind(ScmMessages.ChangeLogReport_CHANGE_SET_OUTPUT_SPACING, output, formatDate(new Date(dto.getCreationDate())));
            }

            if (dto.getCreator() != null) {
                output = NLS.bind(ScmMessages.ChangeLogReport_CHANGE_SET_OUTPUT_SPACING, output, getName(dto.getCreator()));
            }

            out.writeLine(output);
        }

        private String getName(ContributorDTO contributor) {
            // all options for formating :
            // contrib.getUserId())
            // contrib.getFullName())
            // "email", contrib.getEmailAddress())
            // "itemid", contrib.getContributorItemId()

            // we want name <email> format
            return NLS.bind(ScmMessages.ChangeLogReport_CHANGE_SET_CONTRIBUTOR_FORMAT, contributor.getFullName(), contributor.getEmailAddress());
        }

        /** Create the name for a set of work items. */
        private String findMultiWorkItemName(List<ChangeLogWorkItemEntryDTO> list) {
            // Create the WI number
            String numberBuf = Long.toString(list.get(0).getWorkItemNumber());
            for (int i = 1; i < list.size(); i++) {
                numberBuf = NLS.bind(ScmMessages.BaseChangeLogEntryVisitor_WORKITEM_LIST_GENERATOR, numberBuf,
                        Long.toString(list.get(i).getWorkItemNumber()));
            }

            return NLS.bind(ScmMessages.BaseChangeLogEntryVisitor_MULTIPLE_WORKITEM_NUMBERS_AND_SINGLE_NAME, numberBuf,
                    list.get(0).getEntryName());
        }

        public void visitChild(ChangeLogEntryDTO parent, ChangeLogEntryDTO child) {
            handleChild(parent, child);
            super.visitChild(parent, child);
        }

        private void handleChild(ChangeLogEntryDTO parent, ChangeLogEntryDTO child) {
            if (CHANGELOG_ENTRY_TYPE_DIRECTION.equals(parent.getEntryType())) {
                if (CHANGELOG_ENTRY_TYPE_COMPONENT.equals(child.getEntryType())) {
                    ChangeLogComponentEntryDTO dto = (ChangeLogComponentEntryDTO) child;
                    if (ADD_COMPONENT.equals(dto.getChangeType()) || REMOVE_COMPONENT.equals(dto.getChangeType())) {
                        return;
                    }
                }

                out.setIndent(0);
                if (fIsIncoming) {
                    if (!fRemovedWritten) {
                        if (fAddedWritten) {
                            printNewLine();
                        }
                        fRemovedWritten = true;
                        out.writeLine(ScmMessages.ChangeLogReport_ADDED_CHANGE_SETS_HEADING);
                    }
                } else {
                    if (!fAddedWritten) {
                        if (fRemovedWritten) {
                            printNewLine();
                        }
                        fAddedWritten = true;
                        out.writeLine(ScmMessages.ChangeLogReport_REMOVED_CHANGE_SETS_HEADING);
                    }
                }
            }
        }

        private void printNewLine() {
            out.writeLine(""); //$NON-NLS-1$
        }

        public void printComponentSections() {

            boolean addedComponentSectionExists = fAddedComponents.size() > 0;

            if (addedComponentSectionExists) {
                out.setIndent(0);
                if (fAddedWritten || fRemovedWritten) {
                    printNewLine();
                }
                out.writeLine(ScmMessages.ChangeLogReport_ADDED_COMPONENTS_HEADING);
                printComponents(fAddedComponents);
            }

            if (fRemovedComponents.size() > 0) {
                out.setIndent(0);
                if (fAddedWritten || fRemovedWritten || addedComponentSectionExists) {
                    printNewLine();
                }
                out.writeLine(ScmMessages.ChangeLogReport_REMOVED_COMPONENTS_HEADING);
                printComponents(fRemovedComponents);
            }
        }

        private void printComponents(List<String> componentList) {
            Iterator<String> componentIterator = componentList.iterator();
            while (componentIterator.hasNext()) {
                out.setIndent(1);
                out.writeLine(componentIterator.next());
            }
        }

    }

}
