/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.build.internal.scm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.filesystem.client.ILoadRuleFactory;
import com.ibm.team.filesystem.client.operations.ILoadRule2;
import com.ibm.team.filesystem.common.IFileItem;
import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.client.SCMPlatform;
import com.ibm.team.scm.common.IComponent;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IWorkspace;
import com.ibm.team.scm.common.VersionablePermissionDeniedException;

/**
 * Encapsulates the Component load rules Jazz SCM build property
 */
public class ComponentLoadRules {

    HashMap<IComponentHandle, IFileItemHandle> fLoadRuleFiles;
    
    /**
     * @param files The files that contain the component load rules to be used when loading the 
     * workspace for the build.
     */
    public ComponentLoadRules(Map<IComponentHandle, IFileItemHandle> files) {
        fLoadRuleFiles = new HashMap<IComponentHandle, IFileItemHandle>();
        if (files != null) {
            HashSet<UUID> componentIds = new HashSet<UUID>();
            for (Map.Entry<IComponentHandle, IFileItemHandle> entry : files.entrySet()) {
                if (!componentIds.add(entry.getKey().getItemId())) {
                    // already have a file for that component
                    throw new IllegalArgumentException();
                }
                fLoadRuleFiles.put(entry.getKey(), entry.getValue());
            }
        }
    }
    
    private static final String COMPONENT = "component="; //$NON-NLS-1$
    private static final String FILEITEM = "fileitem="; //$NON-NLS-1$
    
    /**
     * @param propertySetting The build property setting that describes the files containing
     * the component load rules to be used when loading a workspace for a build.
     */
    public ComponentLoadRules(String propertySetting) {

        this.fLoadRuleFiles = new HashMap<IComponentHandle, IFileItemHandle>();
        
        // if no parameter, no load rules
        if (propertySetting == null || propertySetting.length() == 0) {
            return;
        }
        
        String UUIDs = propertySetting.trim();
        while (UUIDs.length() > 0) {
            if (!UUIDs.startsWith(COMPONENT)) {
                throw new IllegalArgumentException(NLS.bind(ScmMessages.ComponentLoadRules_MALFORMED_PROPERTY, propertySetting));
            }
            UUIDs = UUIDs.substring(COMPONENT.length());
            int idx = UUIDs.indexOf(FILEITEM);
            if (idx == -1) {
                throw new IllegalArgumentException(NLS.bind(ScmMessages.ComponentLoadRules_MALFORMED_PROPERTY_3, propertySetting));
            }
            String componentID = UUIDs.substring(0, idx);
            UUID componentUUID;
            try {
                componentUUID = UUID.valueOf(componentID);
            } catch (IllegalArgumentException e) {
                throw new IllegalArgumentException(NLS.bind(ScmMessages.ComponentLoadRules_MALFORMED_PROPERTY_COMPONENT, componentID, propertySetting));
            }
            
            if (idx + FILEITEM.length() + 1 > UUIDs.length()) {
                throw new IllegalArgumentException(NLS.bind(ScmMessages.ComponentLoadRules_MALFORMED_PROPERTY_2, propertySetting));
            }
            
            UUIDs = UUIDs.substring(idx + FILEITEM.length());
            idx = UUIDs.indexOf(COMPONENT);
            String fileID;
            if (idx == -1) {
                // at the end
                fileID = UUIDs;
                UUIDs = ""; //$NON-NLS-1$
            } else {
                fileID = UUIDs.substring(0, idx);
                UUIDs = UUIDs.substring(idx);
            }
            
            UUID fileUUID;
            try {
                fileUUID = UUID.valueOf(fileID);
            } catch (IllegalArgumentException e) {
                throw new IllegalArgumentException(NLS.bind(ScmMessages.ComponentLoadRules_MALFORMED_PROPERTY_FILE, fileID, propertySetting));
            }
            
            fLoadRuleFiles.put((IComponentHandle) IComponent.ITEM_TYPE.createItemHandle(componentUUID, null),
                    (IFileItemHandle) IFileItem.ITEM_TYPE.createItemHandle(fileUUID, null));
        }
    }
    
    /**
     * The load rules described by the build property or the file items supplied on construction
     * @param workspaceConnection The workspace connection containing the files with the load rules
     * @param progress A monitor to mark progress on
     * @return The load rules described by the build parameter
     * @throws TeamRepositoryException if something goes wrong
     */
    public Collection<ILoadRule2> getLoadRules(IWorkspaceConnection workspaceConnection, IProgressMonitor progress) throws TeamRepositoryException {
        SubMonitor monitor = SubMonitor.convert(progress, fLoadRuleFiles.size());
        
        ArrayList<ILoadRule2> loadRules = new ArrayList<ILoadRule2>(fLoadRuleFiles.size());
        for (Map.Entry<IComponentHandle, IFileItemHandle> entry : fLoadRuleFiles.entrySet()) {
            IComponentHandle componentHandle = entry.getKey();;
            try {
                ILoadRule2 rule = ILoadRuleFactory.loadRuleFactory.getLoadRule(workspaceConnection, componentHandle, entry.getValue(), monitor.newChild(1));
                loadRules.add(rule);
            } catch (VersionablePermissionDeniedException e1) {
                // Lets try to build up a more informative error message.
                monitor.setWorkRemaining(1);
                ITeamRepository repo = (ITeamRepository) workspaceConnection.getResolvedWorkspace().getOrigin();
                IContributor contributor = repo.loggedInContributor();
                IComponent component = null;
                try {
                    component = (IComponent) repo.itemManager().fetchCompleteItem(componentHandle, IItemManager.DEFAULT, monitor.newChild(1));
                } catch (TeamRepositoryException e2) {
                    // Just throw the original error.
                    throw e1;
                }
                if (contributor != null && component != null) {
                    String errMsg = NLS.bind(ScmMessages.ComponentLoadRules_PRIVATE_LOAD_RULE, new String[] {
                            contributor.getName(), contributor.getUserId(), component.getName() });
                    throw new VersionablePermissionDeniedException(errMsg, e1);
                } else {
                    throw e1;
                }
            }
        }
        return loadRules;
    }
    
    /**
     * @return The set of file items within a component that contain the rules for what should
     * be loaded from the component during the build.
     */
    public Map<IComponentHandle, IFileItemHandle> getLoadRuleFiles() {
        return Collections.unmodifiableMap(fLoadRuleFiles);
    }
    /**
     * @return The property that describes the files that contain the load rules for
     * components within a workspace.
     */
    public String getBuildPropertySetting() {
        StringBuilder builder = new StringBuilder();
        for (Map.Entry<IComponentHandle, IFileItemHandle> entry : fLoadRuleFiles.entrySet()) {
            builder.append(COMPONENT);
            builder.append(entry.getKey().getItemId().getUuidValue());
            builder.append(' ');
            builder.append(FILEITEM);
            builder.append(entry.getValue().getItemId().getUuidValue());
            builder.append(' ');
        }
        return builder.toString();
    }

    
    /**
     * The load rules described by the build property or the file items supplied on construction
     * @param teamRepository The team repository containing the workspace
     * @param workspace The workspace to load
     * @param progress A monitor to mark progress on
     * @return The load rules
     * @throws TeamRepositoryException
     */
    public Collection<ILoadRule2> getLoadRules(ITeamRepository teamRepository, IWorkspace workspace, IProgressMonitor progress) throws TeamRepositoryException {
        SubMonitor monitor = SubMonitor.convert(progress, 100);
        IWorkspaceConnection workspaceConnection = SCMPlatform.getWorkspaceManager(teamRepository)
            .getWorkspaceConnection(workspace, monitor.newChild(10));
        return getLoadRules(workspaceConnection, monitor.newChild(90));
    }
}
