/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.scm.common.IComponent;
import com.ibm.team.scm.common.IComponentHandle;

/**
 * Responsible for taking a list of component uuids and transforming them into
 * a list of component handles.
 */
public class LoadComponents {
    private ITeamRepository repository;
    private String componentUuidProperty;
    private Collection<IComponentHandle> components;

    /**
     * @param repository The repository containing the components. Never 
     * <code>null</code>
     * @param componentUUIDs list of whitespace separated component uuids.
     */
    public LoadComponents(ITeamRepository repository, String componentUUIDs) {
        this.repository = repository;
        this.componentUuidProperty = componentUUIDs;
        this.components = buildComponents();
    }

    /**
     * @param loadComponents
     */
    public LoadComponents(Collection<IComponentHandle> loadComponents) {
        components = new ArrayList<IComponentHandle>(loadComponents.size());
        components.addAll(loadComponents);
        componentUuidProperty = buildProperty();
    }

    /**
     * From the string parameter list of component uuids extract a list of
     * component handles;
     * @return The component handles specified in the parameter.  An empty
     * list will be returned if no component handles.  Never <code>null</code>
     */
    public Collection<IComponentHandle> getComponentHandles() {
        return components;
    }
    
    /**
     * From the list of components extract the build parameter to be stored.
     * @return The build property representation of the list of components.
     */
    public String getBuildProperty() {
        return componentUuidProperty;
    }
    
    private String buildProperty() {
        StringBuffer result = new StringBuffer();
        for (IComponentHandle componentHandle : components) {
            result.append(componentHandle.getItemId().getUuidValue()).append(' ');
        }
        return result.toString().trim();
    }
    
    private Collection<IComponentHandle> buildComponents() {
        if (componentUuidProperty == null) {
            return Collections.EMPTY_LIST;
        }

        String componentsProperty = componentUuidProperty.trim();
        if (componentsProperty.length() == 0) {
            return Collections.EMPTY_LIST;
        }
        
        Collection<IComponentHandle> result = new ArrayList<IComponentHandle>();
        boolean done = false;
        while (!done) {
            int index = componentsProperty.indexOf(" "); //$NON-NLS-1$
            String uuidValue;
            if (index == -1) {
                done = true;
                uuidValue = componentsProperty;
            } else {
                uuidValue = componentsProperty.substring(0, index);
                componentsProperty = componentsProperty.substring(index).trim();
            }
            UUID itemId = UUID.valueOf(uuidValue);
            result.add((IComponentHandle) IComponent.ITEM_TYPE.createItemHandle(repository, itemId, null));
        }
        return result;
    }
    
    /**
     * Determine if any of the components supplied are in the list of load components.
     * Expectation is this can be used to validate if components of load files are
     * in the excluded list.
     * @param otherComponents The collection of other components to check against
     * Never <code>null</code>
     * @return The component handles in otherComponents that are present in the
     * collection of load components.
     */
    public Collection<IComponentHandle> getIntersectingComponents(Collection<IComponentHandle> otherComponents) {
        HashSet<UUID> loadComponentUuids = new HashSet<UUID>();
        for (IComponentHandle component : components) {
            loadComponentUuids.add(component.getItemId());
        }
        
        ArrayList<IComponentHandle> result = new ArrayList<IComponentHandle>();
        for (IComponentHandle component : otherComponents) {
            if (loadComponentUuids.contains(component.getItemId())) {
                result.add(component);
            }
        }
        return result;
    }
}
