/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.scm;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.team.build.internal.client.ProxyHelper;
import com.ibm.team.filesystem.client.internal.utils.RepositoryRegistry;
import com.ibm.team.filesystem.client.util.IRepositoryRecord;
import com.ibm.team.repository.client.ILoginHandler2;
import com.ibm.team.repository.client.ILoginInfo2;
import com.ibm.team.repository.client.IRetryPolicy;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.ITeamRepository.ILoginHandler;
import com.ibm.team.repository.client.ITeamRepository.ILoginHandler.ILoginInfo;
import com.ibm.team.repository.client.SavedPasswordNotObtainedException;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.client.login.IAliasChooser;
import com.ibm.team.repository.client.login.IntegratedWindowsLoginInfo;
import com.ibm.team.repository.client.login.KerberosLoginInfo;
import com.ibm.team.repository.client.login.SSLCertificateLoginInfo;
import com.ibm.team.repository.client.login.SmartCardLoginInfo;
import com.ibm.team.repository.client.login.UsernameAndPasswordLoginInfo;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.transport.client.AuthenticationException;

/**
 * Class that provides the build framework with access to a set of repositories
 * and the credentials required to connect to those repositories.
 * 
 * @since 0.8
 */
public class RepositoryManager {
    
    private Map<String, ITeamRepository> repositories = new HashMap<String, ITeamRepository>();
    private Set<String> authFailures = new HashSet<String>();
    private IRetryPolicy retryPolicy;
    
    /**
     * An interface that allows feedback/debug info to be printed
     */
    public interface IConsole {
        
        /**
         * Print the given message to the console
         * @param message the message
         */
        void print(String message);
    }
    
    /**
     * Return a repository manager that contains the given repository.
     * @param repository a repository (never <code>null</code>)
     * @return a repository manager that contains the given repository
     */
    public static RepositoryManager createWith(ITeamRepository repository) {
        RepositoryManager manager = new RepositoryManager();
        manager.add(repository);
        return manager;
    }
    
    /**
     * Create a manager from the data in the given registry.
     * @param file a repository registry (never <code>null</code>)
     * @param console a console for printing feedback info (may be <code>null</code>)
     * @return a repository manager
     */
    public static RepositoryManager createFromRegistryFile(File file, IConsole console) {
        RepositoryManager manager = new RepositoryManager();
        RepositoryRegistry registry = new RepositoryRegistry(file);
        for (Iterator<IRepositoryRecord> iterator = registry.iterator(); iterator.hasNext();) {
            IRepositoryRecord record = iterator.next();
            ITeamRepository teamRepository = null;
            if (record.getUsername() != null && record.getPassword() != null) {
                teamRepository = getTeamRepository(record.getUrl(), record.getUsername(), record.getPassword(), console);
                updateCredentialsFromUsernameAndPassword(teamRepository, record.getUsername(), record.getPassword());
            } else if (record.getCertficiateLocation() != null && record.getPassword() != null) {
                teamRepository = getTeamRepository(record.getUrl(), console);
                updateCredentialsFromCertificateFile(teamRepository, record.getCertficiateLocation(), record.getPassword());
            } else if (record.isSmartCard() && record.getUsername() != null) {
                teamRepository = getTeamRepository(record.getUrl(), console);
                updateCredentialsFromSmartCard(teamRepository, record.getUsername());
            } else if (record.isKerberos()) {
                teamRepository = getTeamRepository(record.getUrl(), console);
                updateCredentialsFromKerberos(teamRepository);
            } else if (record.isIntegratedWindows()) {
                teamRepository = getTeamRepository(record.getUrl(), console);
                updateCredentialsFromIntegratedWindows(teamRepository);
            }
            manager.add(teamRepository);
        }
        return manager;
    }
    
    /**
     * Return a repository manager that contains the given repository which uses
     * basic userId/password authentication.
     * 
     * @param repositoryAddress
     *            the repository URI
     * @param userId
     *            the user id
     * @param password
     *            the password
     * @param console
     *            the console for printing feedback info (may be
     *            <code>null</code> )
     * @return a repository manager
     */
    public static RepositoryManager createForUsernameAndPassword(String repositoryAddress, String userId,
            String password, IConsole console) {
        ITeamRepository teamRepository = getTeamRepository(repositoryAddress, userId, password, console);
        updateCredentialsFromUsernameAndPassword(teamRepository, userId, password);
        return createWith(teamRepository);
    }

    /**
     * Return a repository manager that contains the given repository which uses
     * a certificate file for authentication.
     * 
     * @param repositoryAddress
     *            the repository URI
     * @param certificateFile
     *            the certificate file containing the users login certificate
     * @param password
     *            the password
     * @param console
     *            the console for printing feedback info (may be
     *            <code>null</code> )
     * @return a repository manager
     */
    public static RepositoryManager createForCertificateFile(String repositoryAddress, File certificateFile,
            String password, IConsole console) {

        if (!certificateFile.exists()) {
            throw new IllegalArgumentException(ScmMessages.RepositoryManager_CERTIFICATE_FILE_MUST_EXIST);
        }

        ITeamRepository teamRepository = getTeamRepository(repositoryAddress, console);
        updateCredentialsFromCertificateFile(teamRepository, certificateFile.getAbsolutePath(), password);
        return createWith(teamRepository);
    }

    /**
     * Return a repository manager that contains the given repository which uses
     * a smart card for authentication.
     * 
     * @param repositoryAddress
     *            the repository URI
     * @param userId
     *            the user id (alias)
     * @param console
     *            the console for printing feedback info (may be
     *            <code>null</code> )
     * @return a repository manager
     */
    public static RepositoryManager createForSmartCard(String repositoryAddress, String userId, IConsole console) {
        ITeamRepository teamRepository = getTeamRepository(repositoryAddress, console);
        updateCredentialsFromSmartCard(teamRepository, userId);
        return createWith(teamRepository);
    }
    
    /**
     * Return a repository manager that contains the given repository which uses
     * Kerberos for authentication
     * @param repositoryAddress
     * @param console
     * @return a repository manager
     */
    public static RepositoryManager createForKerberos(String repositoryAddress, IConsole console) {
        ITeamRepository teamRepository = getTeamRepository(repositoryAddress, console);
        updateCredentialsFromKerberos(teamRepository);
        return createWith(teamRepository);
    }
    
    /**
     * Return a repository manager that contains the given repository which uses 
     * Windows Integrated authentication
     * @param repositoryAddress 
     * @param console
     * @return a repository manager
     */
    public static RepositoryManager createForIntegratedWindows(String repositoryAddress, IConsole console) {
        ITeamRepository teamRepository = getTeamRepository(repositoryAddress, console);
        updateCredentialsFromIntegratedWindows(teamRepository);
        return createWith(teamRepository);
    }

    private static ITeamRepository getTeamRepository(String url, final String jazzUsername, final String jazzPassword,
            IConsole console) {
        ITeamRepository teamRepository = TeamPlatform.getTeamRepositoryService().getUnmanagedRepository(url);
        setProxyDetails(url, teamRepository, console, jazzUsername, jazzPassword);     
        return teamRepository;
    }

    private static ITeamRepository getTeamRepository(String url, IConsole console) {
        ITeamRepository teamRepository = TeamPlatform.getTeamRepositoryService().getUnmanagedRepository(url);
        setProxyDetails(url, teamRepository, console, null, null);     
        return teamRepository;
    }
    
    private static void setProxyDetails(String url, ITeamRepository teamRepository, IConsole console, String defaultProxyUsername, 
            String defaultProxyPassword) {
        String host = ProxyHelper.getProxyHost(url);
        if (host != null) {
            int port = ProxyHelper.getProxyPort(url);
            //Proxy type must always be null until we support HTTPS for proxy
            String proxyType = null;
            String proxyUser = ProxyHelper.getProxyUser(url, defaultProxyUsername);
            String proxyPassword = ProxyHelper.getProxyPassword(url, defaultProxyPassword);
            teamRepository.setProxy(host, proxyType, port, proxyUser, proxyPassword);
            if (console != null) {
                console.print(NLS.bind(ScmMessages.RepositoryManager_USING_PROXY, host, port, url));
            }
        } else if (console != null) {
            console.print(NLS.bind(ScmMessages.RepositoryManager_NOT_USING_PROXY, url));
        }
    }

    private static void updateCredentialsFromUsernameAndPassword(ITeamRepository teamRepository, final String username,
            final String password) {
        teamRepository.registerLoginHandler(new LoginHandler(username, password));
    }

    private static void updateCredentialsFromCertificateFile(ITeamRepository teamRepository,
            final String certificateFilePath, final String password) {
        teamRepository.registerLoginHandler(new CertificateLoginHandler(certificateFilePath, password));
    }

    private static void updateCredentialsFromSmartCard(ITeamRepository teamRepository, final String username) {
        teamRepository.registerLoginHandler(new SmartCardLoginHandler(username));
    }
    
    private static void updateCredentialsFromKerberos(ITeamRepository teamRepository) {
        teamRepository.registerLoginHandler(new KerberosLoginHandler());
    }
    
    private static void updateCredentialsFromIntegratedWindows(ITeamRepository teamRepository) {
        teamRepository.registerLoginHandler(new IntegratedWindowsLoginHandler());
    }

    /**
     * Return the repository for the given URI. If <code>null</code>
     * is returned, there is not a known repository.
     * @param repositoryUri the URI of a repository
     * @param failIfAbsent fail with an exception if the repository is absent or had a previous auth failure
     * @return the team repository for the URI configured with credentials or <code>null</code>
     * @throws TeamRepositoryException 
     */
    public ITeamRepository getRepository(String repositoryUri, boolean failIfAbsent) throws TeamRepositoryException {
        repositoryUri = canonicalizeRepositoryURI(repositoryUri);
        if ( hasAuthenticationFailure(repositoryUri)) {
            if (failIfAbsent) {
                throw new TeamRepositoryException(NLS.bind(ScmMessages.RepositoryManager_AUTH_PREVIOUSLY_FAILED, repositoryUri));
            }
            return null;
        }
        ITeamRepository repo;
        synchronized (repositories) {
            repo = repositories.get(repositoryUri);
        }
        if (repo == null && failIfAbsent) {
            throw new TeamRepositoryException(NLS.bind(ScmMessages.RepositoryManager_AUTH_INFO_MISSING, repositoryUri));
        }
        return repo;
    }
    
    /**
     * Add the repository to this manager
     * @param repository a repository
     */
    public void add(ITeamRepository repository) {
        if (repository != null) {
            synchronized (repositories) {
                if (retryPolicy != null) {
                    repository.setRetryPolicy(retryPolicy);
                }
                repositories.put(canonicalizeRepositoryURI(repository.getRepositoryURI()), repository);
            }
        }
    }

    /**
     * Set the retry policy for all the repositories managed by this manager
     * Any repositories previously managed by this manager will have their
     * retry policy set to this policy. New repositories added will also have
     * their retry policy set to this policy.
     * @param policy
     * 
     * @since 0.9.5 (RTC 4.0.5)
     */
    public void setRetryPolicy(IRetryPolicy policy) {
        synchronized (repositories) {
            retryPolicy = policy;
            for (ITeamRepository repository : repositories.values()) {
                repository.setRetryPolicy(policy);
            }
        }
    }

    /**
     * Handle the authentication failure by recording it and throwing the exception
     * @param repository the repository
     * @param exception the auth exception
     * @throws AuthenticationException 
     */
    public void handleAuthenticationFailure(ITeamRepository repository, AuthenticationException exception) throws AuthenticationException {
        authFailures.add(canonicalizeRepositoryURI(repository.getRepositoryURI()));
        throw exception;
    }

    /**
     * Return whether the given repository has a previous authentication failure
     * @param remoteRepositoryURI the repository
     * @return whether the given repository has a previous authentication failure
     */
    public boolean hasAuthenticationFailure(String remoteRepositoryURI) {
        return authFailures.contains(canonicalizeRepositoryURI(remoteRepositoryURI));
    }

    /**
     * Log into the team repository
     * @param repository the repository
     * @param monitor a progress monitor
     * @throws TeamRepositoryException 
     */
    public void login(ITeamRepository repository, IProgressMonitor monitor) throws TeamRepositoryException {
        try {
            try {
                repository.login(monitor);
            } catch (OperationCanceledException exception) {
                // The repository location will throw a canceled exception
                // if authentication failed and, on the second attempt, the login handler
                // returned null
                if (repository.getErrorState() == ITeamRepository.ERROR_STATE_AUTH) {
                    throw asAuthenticationException(repository);
                }
                throw exception;
            }
        } catch (AuthenticationException exception) {
            handleAuthenticationFailure(repository, exception);
        }
    }
    
    /*
     * Convert the exception to an authentication exception
     */
    private AuthenticationException asAuthenticationException(ITeamRepository repository) {
        Throwable lastError = repository.getLastError();
        if (lastError != null) {
            if (lastError instanceof AuthenticationException) {
                return (AuthenticationException) lastError;
            }
            return new AuthenticationException(lastError.getMessage(), lastError);
        }
        return new AuthenticationException(NLS.bind(ScmMessages.RepositoryManager_AUTHENTICATION_FAILED, repository.getRepositoryURI()));
    }

    private String canonicalizeRepositoryURI(String repositoryURI) {
        if (!repositoryURI.endsWith("/")) { //$NON-NLS-1$
            repositoryURI += "/"; //$NON-NLS-1$
        }
        return repositoryURI;
    }

    private static abstract class AbstractLoginHandler extends Object {
        private final ILoginInfo2 loginInfo;

        protected AbstractLoginHandler(ILoginInfo2 loginInfo) {
            super();
            this.loginInfo = loginInfo;
        }

        protected final ILoginInfo2 getLoginInfo() {
            return loginInfo;
        }
    }

    private static abstract class AbstractLoginHandler2 extends AbstractLoginHandler implements ILoginHandler2 {
        protected AbstractLoginHandler2(ILoginInfo2 loginInfo) {
            super(loginInfo);
        }

        public ILoginInfo2 challenge(ITeamRepository repository) {
            return getLoginInfo();
        }
    }

    private static class CertificateLoginHandler extends AbstractLoginHandler2 {
        private CertificateLoginHandler(String certificateFile, String password) {
            super(new SSLCertificateLoginInfo(certificateFile, password));
        }
    }

    private static class LoginHandler extends AbstractLoginHandler implements ILoginHandler, ILoginInfo {
        private LoginHandler(String username, String password) {
            super(new UsernameAndPasswordLoginInfo(username, password));
        }

        public ILoginInfo challenge(ITeamRepository repository) {
            return this;
        }

        /**
         * The contract of {@link ILoginInfo#getPassword()} is to return null
         * if the login attempt should be abandoned.
         */
        public String getPassword() {
            try {
                return getUsernameAndPasswordLoginInfo().getPassword();
            } catch (SavedPasswordNotObtainedException e) {
                return null;
            }
        }

        public String getUserId() {
            return getUsernameAndPasswordLoginInfo().getUsername();
        }

        private UsernameAndPasswordLoginInfo getUsernameAndPasswordLoginInfo() {
            return (UsernameAndPasswordLoginInfo) getLoginInfo();
        }
    }

    private static class SmartCardLoginHandler extends AbstractLoginHandler2 {
        public SmartCardLoginHandler(String alias) {
            super(SmartCardLoginHandler.createSmartCardLoginInfo(alias));
        }

        private static SmartCardLoginInfo createSmartCardLoginInfo(String requestedAlias) {
            IAliasChooser chooser = SmartCardLoginHandler.createAliasChooser(requestedAlias);
            return new SmartCardLoginInfo(chooser);
        }
        
        private static IAliasChooser createAliasChooser(final String requestedAlias) {
            return new IAliasChooser() {
                public String chooseAlias(List<String> aliases) {
                    for (String alias : aliases) {
                        if (alias.equals(requestedAlias))
                            return alias;
                    }
                    return null;
                }
            };
        }
    }
    
    
    private static class KerberosLoginHandler extends AbstractLoginHandler2 {
        public KerberosLoginHandler() {
           super(new KerberosLoginInfo());
        }
    }
    
    private static class IntegratedWindowsLoginHandler extends AbstractLoginHandler2 {
        public  IntegratedWindowsLoginHandler() {
            super(new IntegratedWindowsLoginInfo());
        }
    }
}
